Package: distfreereg
Type: Package
Title: Distribution-Free Goodness-of-Fit Testing for Regression
Version: 1.2
Depends: R (>= 4.4)
Imports: calculus, clue, lme4, methods
Suggests: knitr, rmarkdown
VignetteBuilder: knitr, rmarkdown
Date: 2026-05-05
Authors@R: person(given = "Jesse",
                    family = "Miller",
                    role = c("aut", "cre"),
                    email = "smallepsilon@proton.me",
                    comment = c(ORCID = "0009-0005-9465-7461"))
Maintainer: Jesse Miller <smallepsilon@proton.me>
Description: Implements the distribution-free goodness-of-fit regression testing
  procedure, introduced by Estate Khmaladze (2021, <doi:10.1007/s10463-021-00786-3>)
  to test whether or not the mean structure of a parametric model belongs to a
  specified model family. The test is implemented for general mean functions
  with minimal distributional assumptions as well as common models (e.g., lm,
  glm) with the usual model assumptions.
License: GPL-3
NeedsCompilation: no
Packaged: 2026-05-06 20:33:47 UTC; j
Author: Jesse Miller [aut, cre] (ORCID:
    <https://orcid.org/0009-0005-9465-7461>)
Repository: CRAN
Date/Publication: 2026-05-08 08:00:13 UTC
Built: R 4.5.2; ; 2026-05-08 09:49:01 UTC; unix

importFrom("stats", "aggregate", "approxfun", "coef", "coefficients",
           "complete.cases", "confint", "confint.default", "cov2cor", "density",
           "ecdf", "fitted", "formula", "getCall", "ks.test", "lm",
           "model.frame", "model.matrix", "na.fail", "nls", "optim", "pnorm",
           "ppoints", "predict", "punif", "qnorm", "qqplot", "quantile",
           "rchisq", "reformulate", "residuals","rnorm", "sigma", "simulate",
           "update", "update.default", "vcov", "weights")
importFrom("graphics", "abline", "legend", "lines", "polygon", "text")
importFrom("utils", "hasName")
importFrom("grDevices", "rgb")
importFrom("methods", "is")
importFrom("clue", "solve_LSAP")
importFrom("lme4", "fixef", "getME")
importFrom("calculus", "hessian", "jacobian")
export("asymptotics", "compare", "distfreereg", "rejection")
S3method(coef, distfreereg)
S3method(confint, distfreereg)
S3method(fitted, distfreereg)
S3method(formula, distfreereg)
S3method(plot, distfreereg)
S3method(predict, distfreereg)
S3method(print, compare)
S3method(print, distfreereg)
S3method(residuals, distfreereg)
S3method(update, distfreereg)
S3method(vcov, distfreereg)
S3method(ks.test, compare)
S3method(plot, compare)

library(distfreereg)
all.equal.distfreereg <- distfreereg:::all.equal.distfreereg
test_dfr_functions <- distfreereg:::test_dfr_functions

n <- 1e2

### Poisson

set.seed(20250302)
p_pois <- 2
X_pois <- matrix(rexp(n*p_pois), ncol = p_pois)
theta_pois <- c(2,1)
means <- exp(colSums(theta_pois * t(X_pois)) + 1)
Y_pois <- rpois(n, means)
df_pois <- data.frame(y = Y_pois, x = X_pois[,1], z = X_pois[,2],
                      g = rep(1:10, 10))
form_pois <- y ~ x + z
m_pois <- glm(form_pois, data = df_pois, family = "poisson", x = TRUE, y = TRUE)

set.seed(20250302)
dfr_pois <- distfreereg(test_mean = m_pois, verbose = FALSE,
                        control = list(return_on_error = FALSE))

dfr_pois

set.seed(20250302)
dfr_pois_x_false <- distfreereg(test_mean = update(m_pois, x = FALSE),
                                control = list(return_on_error = FALSE))

set.seed(20250302)
dfr_pois_J <- distfreereg(test_mean = m_pois,
                          override = list(J = dfr_pois[["J"]]),
                          control = list(return_on_error = FALSE))

set.seed(20250302)
dfr_pois_fitted <- distfreereg(test_mean = m_pois,
                               override = list(fitted_values = dfr_pois[["fitted_values"]]),
                               control = list(return_on_error = FALSE))

stopifnot(all.equal(dfr_pois, dfr_pois_x_false))
stopifnot(all.equal(dfr_pois, dfr_pois_J))
stopifnot(all.equal(dfr_pois, dfr_pois_fitted))

set.seed(20250516)
dfr_form_glm_verbose <- distfreereg(test_mean = form_pois, data = df_pois,
                                    method = "glm",
                                    method_args = list(family = "poisson"),
                                    control = list(return_on_error = FALSE))

newdata_pois <- data.frame(y = rpois(10, lambda = 1), x = rexp(10), z = rexp(10))
test_dfr_functions(dfr_pois, newdata = newdata_pois)

set.seed(20250302)
dfr_form_pois <- distfreereg(test_mean = form_pois, data = df_pois,
                             method = "glm",
                             method_args = list(family = "poisson"),
                             verbose = FALSE,
                             control = list(return_on_error = FALSE))

dfr_form_pois
test_dfr_functions(dfr_form_pois, newdata = newdata_pois)

stopifnot(all.equal(dfr_pois, dfr_form_pois))

set.seed(20250225)
cdfr_form_pois <- asymptotics(dfr_form_pois, reps = 5)
set.seed(20250225)
cdfr_pois <- asymptotics(dfr_pois, reps = 5)

signif(rejection(cdfr_form_pois, alpha = c(0.1, 0.5))[,2:3], digits = 3)
signif(rejection(cdfr_pois, alpha = c(0.1, 0.5))[,2:3], digits = 3)


# Orderings

set.seed(20250516)
dfr_pois_asis <- update(dfr_pois, ordering = "asis")
set.seed(20250516)
dfr_form_pois_asis <- update(dfr_form_pois, ordering = "asis")
stopifnot(all.equal(dfr_pois_asis, dfr_form_pois_asis))

set.seed(20250516)
dfr_pois_optimal <- update(dfr_pois, ordering = "optimal")
set.seed(20250516)
dfr_form_pois_optimal <- update(dfr_form_pois, ordering = "optimal")
stopifnot(all.equal(dfr_pois_optimal, dfr_form_pois_optimal))

set.seed(20250516)
dfr_pois_natural <- update(dfr_pois, ordering = "natural")
set.seed(20250516)
dfr_form_pois_natural <- update(dfr_form_pois, ordering = "natural")
stopifnot(all.equal(dfr_pois_natural, dfr_form_pois_natural))

set.seed(20250516)
dfr_pois_g <- update(dfr_pois, ordering = list("g"))
set.seed(20250516)
dfr_form_pois_g <- update(dfr_form_pois, ordering = list("g"))
stopifnot(all.equal(dfr_pois_g, dfr_form_pois_g))

df_pois[dfr_pois_g[["res_order"]],][["g"]]
df_pois[dfr_form_pois_g[["res_order"]],][["g"]]

set.seed(20250516)
dfr_pois_g_grouped <- update(dfr_pois_g, group = TRUE)
set.seed(20250516)
dfr_form_pois_g_grouped <- update(dfr_form_pois_g, group = TRUE)
stopifnot(all.equal(dfr_pois_g_grouped, dfr_form_pois_g_grouped))



### Partial output

dfr_pois_partial <- distfreereg(test_mean = m_pois, verbose = FALSE,
                                control = list(orth_tol = 1e-100))
names(dfr_pois_partial)



### Binomial

theta_binom <- c(1,2,-1,-3)/30
p_binom <- length(theta_binom) - 1
set.seed(20250225)
X_binom <- cbind(1, matrix(rexp(n*p_binom, rate = 1/5), ncol = p_binom))
probs <- 1/(1 + exp(-colSums(theta_binom * t(X_binom)) - 1))
Y_binom <- rbinom(n, size = 1, prob = probs)

df_binom <- as.data.frame(cbind(Y_binom, X_binom[,-c(1,4)], rep(1:10, 10)))# omit intercept and one covariate
colnames(df_binom) <- c("y", letters[1:(length(theta_binom)-2)], "g")
df_binom$a <- df_binom$a^2
form_binom <- reformulate(termlabels = colnames(df_binom)[-1], response = "y")
m_binom <- glm(form_binom, data = df_binom, family = "binomial", x = TRUE, y = TRUE)

set.seed(20250302)
dfr_binom <- distfreereg(test_mean = m_binom, verbose = FALSE,
                         control = list(return_on_error = TRUE))
dfr_binom
newdata_binom <- data.frame(y = rbinom(10, size = 1, prob = runif(10)),
                            a = rexp(10, rate = 1/5), b = rexp(10, rate = 1/5))
test_dfr_functions(dfr_binom, newdata = newdata_binom)

set.seed(20250302)
dfr_form_binom <- distfreereg(test_mean = form_binom, data = df_binom,
                              method = "glm",
                              method_args = list(family = "binomial"),
                              verbose = FALSE,
                              control = list(return_on_error = FALSE))
dfr_form_binom
test_dfr_functions(dfr_form_binom, newdata = newdata_binom)

stopifnot(all.equal(dfr_binom, dfr_form_binom))

cdfr_form_binom <- asymptotics(dfr_form_binom, reps = 5)
cdfr_binom <- asymptotics(dfr_binom, reps = 5)

signif(rejection(cdfr_form_binom, alpha = c(0.1, 0.5))[,2:3], digits = 3)
signif(rejection(cdfr_binom, alpha = c(0.1, 0.5))[,2:3], digits = 3)

# Orderings

set.seed(20250516)
dfr_binom_asis <- update(dfr_binom, ordering = "asis")
set.seed(20250516)
dfr_form_binom_asis <- update(dfr_form_binom, ordering = "asis")
stopifnot(all.equal(dfr_binom_asis, dfr_form_binom_asis))

set.seed(20250516)
dfr_binom_optimal <- update(dfr_binom, ordering = "optimal")
set.seed(20250516)
dfr_form_binom_optimal <- update(dfr_form_binom, ordering = "optimal")
stopifnot(all.equal(dfr_binom_optimal, dfr_form_binom_optimal))

set.seed(20250516)
dfr_binom_natural <- update(dfr_binom, ordering = "natural")
set.seed(20250516)
dfr_form_binom_natural <- update(dfr_form_binom, ordering = "natural")
stopifnot(all.equal(dfr_binom_natural, dfr_form_binom_natural))

set.seed(20250516)
dfr_binom_g <- update(dfr_binom, ordering = list("g"))
set.seed(20250516)
dfr_form_binom_g <- update(dfr_form_binom, ordering = list("g"))
stopifnot(all.equal(dfr_binom_g, dfr_form_binom_g))

df_binom[dfr_binom_g[["res_order"]],][["g"]]
df_binom[dfr_form_binom_g[["res_order"]],][["g"]]

set.seed(20250516)
dfr_binom_g_grouped <- update(dfr_binom_g, group = TRUE)
set.seed(20250516)
dfr_form_binom_g_grouped <- update(dfr_form_binom_g, group = TRUE)
stopifnot(all.equal(dfr_binom_g_grouped, dfr_form_binom_g_grouped))




### Failures

tryCatch(distfreereg(test_mean = form_pois, data = df_pois,
                     method = "glm",
                     method_args = list(family = "poisson",
                                        weights = rep(1, n)),
                     verbose = FALSE,
                     control = list(return_on_error = FALSE)),
         error = function(e) warning(e))

f2ftheta <- function(f, X, n){
  stopifnot(is.function(f))
  if("x" %in% names(formals(f))){
    stopifnot(is.matrix(X))
    validate_numeric(x = X)
    output <- function(theta) apply(X, MARGIN = 1, FUN = f, theta = theta)
  } else {
    if("X" %in% names(formals(f))){
      output <- function(theta) f(X = X, theta = theta)
    } else {
      stopifnot(is.numeric(n), n > 0)
      output <- function(theta) rep(unname(theta), n)
    }
  }
  return(output)
}

validate_args_rejection <- function(object, alpha, stat){
  if(!(is(object, "compare")))
    stop("'object' must have class 'compare'")
  validate_numeric(alpha, min_len = 1, min_val = 0, max_val = 1)
  if(isTRUE(!all(stat %in% names(object[["empirical_stats"]]))))
    stop("Some specified statistic(s) not found in ",
         deparse1(substitute(object)))
}

formula.distfreereg <-
  function(x, ...){
    stopifnot(is.list(x))
    if(is.null(x[["test_mean"]])){
      output <- NULL
    } else {
      output <- tryCatch(formula(x[["test_mean"]]), error = function(e) NULL)
    }
    return(output)
  }

# Intended for package testing only.
test_dfr_functions <- function(object, digits = 3, newdata){
  signif_or_null <- function(x, ...) tryCatch(signif(x, ...), error = function(e) NULL)
  print(signif_or_null(coef(object), digits = digits))
  print(signif_or_null(confint(object), digits = digits))
  print(signif_or_null(fitted(object), digits = digits))
  form <- formula(object)
  print(signif_or_null(predict(object), digits = digits))
  print(signif_or_null(predict(object, newdata = newdata), digits = digits))
  print(signif_or_null(vcov(object), digits = digits))
}

calc_p <-
function(observed_stats, theoretical_stats){
  stopifnot(is.list(observed_stats), is.list(theoretical_stats),
            identical(length(observed_stats), length(theoretical_stats)),
            isTRUE(all(names(observed_stats) == names(theoretical_stats))))
  m <- length(observed_stats)
  stopifnot(m == length(theoretical_stats))
  output <- vector(mode = "list", length = m)
  names(output) <- names(observed_stats)
  for(i in seq_len(m)){
    output[i] <- mean(observed_stats[[i]] <= theoretical_stats[[i]])
  }
  return(output)
}

model2method <- function(model){
  switch(EXPR = class(model)[1],
         "lm" = "lm",
         "glm" = "glm",
         "nls" = "nls",
         "lmerMod" = "lmer",
         stop("invalid model type found: ", class(model)[1]))
}

define_r <- function(n, p, res_order, gs_tol){
  stopifnot(isOrdering(x = res_order, len = n), is.numeric(gs_tol), gs_tol > 0)
  # First create orthogonal vectors, then normalize.
  validate_numeric(n, len = 1, pos_int = TRUE)
  validate_numeric(p, len = 1, pos_int = TRUE)
  n <- as.integer(n)
  p <- as.integer(p)
  r <- matrix(NA, nrow = n, ncol = p)
  # First column of r is unit vector with equal elements.
  r[,1] <- rep(1/sqrt(n), n)
  if(p > 1){
    # Create second column, which also serves as a template for all remaining
    # columns.
    r[,2] <- sqrt(12/n) * ((1:n)/n - (n+1)/(2*n))
    # All remaining columns are powers of second column.
    for(i in (seq_len(p-2))){
      r[,i+2] <- r[,2]^(i+1)
    }
  }
  r <- tryCatch(qr.Q(qr(x = r, tol = gs_tol)),
                error = function(e) stop("qr() unable to orthogonalize r: ", e))
  return(r)
}

validate_symmetric <- function(covmat, name, sym_tol, sym_tol1){
  stopifnot(is.numeric(sym_tol), is.numeric(sym_tol1),
            sym_tol > 0, sym_tol1 > 0,
            is.list(covmat) || is.matrix(covmat) || is.numeric(covmat))
  if(!is.null(covmat)){
    if(is.matrix(covmat)) covmat <- list(covmat)#so all tests can be done with single line
    if(is.list(covmat)){
      if(!all(sapply(covmat, isSymmetric, tol = sym_tol, tol1 = sym_tol1,
                     check.attributes = FALSE)))
        stop(name, " not symmetric")
    }
  }
}

predict.distfreereg <-
  function(object, ...){
    stopifnot(!is.null(object[["test_mean"]]))
    test_mean <- object[["test_mean"]]
    if(isModel(test_mean) || is(test_mean, "formula")){
      if(is(test_mean, "formula")){
        m <- object[["model"]]
        stopifnot(!is.null(m))
      } else {
        m <- test_mean
      }
      output <- predict(m, ...)
    } else {
      newdata <- list(...)[["newdata"]]
      if(is.null(newdata)){
        output <- fitted(object)
      } else {
        if(is.function(test_mean)){
          stopifnot(!is.null(object[["theta_hat"]]))
          if(!is.numeric(newdata)) stop("'newdata' must be numeric when object$test_mean does not have a formula method")
          if(!is.matrix(newdata)){
            warning("Vector supplied as 'newdata' converted into single row of newdata matrix...")
            newdata <- t(as.matrix(newdata))
          }
          output <- unname(f2ftheta(f = test_mean, X = newdata, n = ncol(newdata))(object[["theta_hat"]]))
        }
      }
    }
    return(output)
  }

calc_stats <-
function(epsp, stat){
  stopifnot(is.numeric(epsp), is.vector(epsp))
  output <- numeric(length(stat))
  for(i in seq_along(stat)){
    output[[i]] <- get(stat[i])(epsp)
  }
  return(output)
}

print.compare <- function(x, ...){
  stopifnot(is(x, "compare"), !is.null(x[["Y"]]))
  
  err_dist_fun <- if(is.null(getCall(x)[["err_dist_fun"]])){
    "'rmvnorm' (default)"
  } else {
    paste0("'", getCall(x)[["err_dist_fun"]], "'")
  }
  
  cat("'compare' object details:\n- Sample size: ", nrow(x[["Y"]]), "\n",
      "- Repetitions: ", ncol(x[["Y"]]), "\n",
      "- Error-generating function: ", err_dist_fun, "\n",
      "- 'distfreereg' objects saved: ", length(x[["dfrs"]]), "\n",
      sep = "")
}

get_data.default <- function(m, ..., generation = 2){
  tryCatch(eval(getCall(m)[["data"]], envir = parent.frame(n = generation)),
           error = function(e) stop("Unable to get model object's data: ", e))
}

combine_lists <-
  function(...){
    comb_lists <- c(...)
    # Precedence is given to named elements of lists earliest in "..."
    # containing those elements.
    return(comb_lists[which(!duplicated(names(comb_lists)))])
  }

validate_mean_function_arg_names <- function(f){
  arg_names <- names(formals(f))
  if(setequal(arg_names, c("x", "theta"))){
    # if(isTRUE(verbose)) message("Interpreting 'x' in ", deparse1(substitute(f)), " as a vector argument...")
  } else {
    if(setequal(arg_names, c("X", "theta"))){
      # if(isTRUE(verbose)) message("Interpreting 'X' in ", deparse1(substitute(f)), " as a matrix argument...")
    } else {
      if(!setequal(arg_names, c("theta"))){
        stop("Invalid argument specification of ", deparse1(substitute(f)))
      }
    } 
  }
}

validate_control_tols <- function(control){
  lapply(control[c("solve_tol", "qr_tol", "orth_tol", "trans_tol", "sym_tol", "sym_tol1")],
         validate_numeric, positive = TRUE, len = 1,
         message = "Invalid tolerance specification: ")
  validate_numeric(control[["matsqrt_tol"]], max_val = 0, len = 1)
}

distfreereg <-
  function(test_mean = NULL,
           covariance = NULL,
           Y = NULL,
           X = NULL,
           theta_init = NULL,
           data = NULL,
           method = NULL,
           method_args = NULL,
           stat = c("KS", "CvM"),
           B = 1e4,
           ordering = "simplex",
           group = TRUE,
           control = NULL,
           override = NULL,
           verbose = TRUE
  ){
    output <- list()
    
    cl <- match.call()
    output[["call"]] <- cl
    
    vargs <-
      validate_args_distfreereg(test_mean = test_mean,
                                ordering = ordering,
                                group = group,
                                stat = stat,
                                B = B,
                                control = control,
                                override = override,
                                verbose = verbose,
                                method = method,
                                method_args = method_args,
                                theta_init = theta_init,
                                Y = Y,
                                X = X,
                                data = data,
                                covariance = covariance)
    Y <- vargs[["Y"]]
    X <- vargs[["X"]]
    n <- vargs[["n"]]
    control <- vargs[["control"]]
    override <- vargs[["override"]]
    B <- vargs[["B"]]
    return_on_error <- control[["return_on_error"]]
    
    if(isModel(test_mean)){
      validate_weights(test_mean)
      test_mean <- refit_model(m = test_mean, verbose = verbose)
    }

    # Note that for consistency, the output value of test_mean is defined here
    # before possibly changing test_mean from a formula to a model.
    output[["test_mean"]] <- test_mean
    
    if(is(test_mean, "formula")){
      test_mean <- fit_model(test_mean, data, method, method_args)
      validate_weights(test_mean)
      output[["model"]] <- test_mean
    }
    
    if(isModel(test_mean)){
      data <- get_data(test_mean)
    }
    
    if(is.null(X) && isModel(test_mean)){
      X <- get_X(test_mean)
    }
    
    if(is.null(Y)){
      Y <- get_Y(test_mean)
    }
    
    output[["data"]] <- list(data = data, Y = Y, X = X)
    
    covariance <- get_cov(m = test_mean, covariance = covariance, verbose = verbose)
    
    if(is.null(covariance[["Q"]])){
      if(isTRUE(verbose)) message("Calculating the inverse square root of the covariance matrix...")
      covariance <- fill_covariance_list(need = "Q", covariance_list = covariance,
                                         matsqrt_tol = control[["matsqrt_tol"]],
                                         solve_tol = control[["solve_tol"]])
    } else{
      if(isTRUE(verbose)) message("Using supplied inverse square root of the covariance matrix...")
    }
    
    output[["covariance"]] <- covariance
    Q <- covariance[["Q"]]
    
    if(is(test_mean, "function")){
      ftheta <- f2ftheta(test_mean, X, n)
    } else {
      ftheta <- NULL
    }
    
    if(is.null(override[["theta_hat"]])){
      if(isTRUE(verbose)) message("Estimating parameters...")
      theta_hat <- get_theta_hat(test_mean = test_mean, ftheta = ftheta, Y = Y,
                                 Q = Q, control = control, override = override,
                                 verbose = verbose)
      if(is.function(test_mean)){
        output[["optimization_output"]] <- theta_hat
        theta_hat <- theta_hat[[control[["theta_hat_name"]]]]
      }
    } else {
      if(isTRUE(verbose)) message("Using supplied parameter estimates...")
      theta_hat <- override[["theta_hat"]]
    }
    if(!is.null(theta_hat) && is.null(names(theta_hat)))
      names(theta_hat) <- paste0("theta", seq_along(theta_hat))
    output[["theta_hat"]] <- theta_hat
    
    if(is.null(override[["fitted_values"]])){
      if(isTRUE(verbose)) message("Calculating fitted values...")
      fitted_values <- get_fitted_values(test_mean = test_mean,
                                         ftheta = ftheta,
                                         theta_hat = theta_hat)
    } else {
      if(isTRUE(verbose)) message("Using supplied fitted values...")
      fitted_values <- override[["fitted_values"]]
    }
    output[["fitted_values"]] <- fitted_values
    
    if(is.null(override[["J"]])){
      if(isTRUE(verbose)) message("Calculating Jacobian...")
      J <- get_jacobian(test_mean = test_mean, override = override,
                        verbose = verbose, ftheta = ftheta,
                        theta_hat = theta_hat, control = control)
      if(is(test_mean, "nls")){
        covariance <- fill_covariance_list(need = "SqrtSigma",
                                           covariance_list = covariance,
                                           matsqrt_tol = control[["matsqrt_tol"]],
                                           solve_tol = control[["solve_tol"]])
        J_tilde <- J
        J <- covprod(covariance[["SqrtSigma"]], J)
      } else {
        J_tilde <- covprod(Q, J)
      }
    } else {
      if(isTRUE(verbose)) message("Using supplied Jacobian...")
      J <- override[["J"]]
      J_tilde <- covprod(Q, J)
    }
    output[["J"]] <- J
    p <- ncol(J)
    
    
    if(isTRUE(verbose)) message("Calculating mu...")
    output[["mu"]] <- mu <- tryReturn(
      calc_mu(J = J_tilde, solve_tol = control[["solve_tol"]], orth_tol = control[["orth_tol"]]),
      output = output, return_on_error = return_on_error)
    
    if(is.null(override[["res_order"]])){
      if(is(test_mean, "formula") || isModel(test_mean)){# should probably not include "formula" part here
        X_for_ordering <- define_data_for_res_order(test_mean = test_mean,
                                                    data = data,
                                                    ordering = ordering)
      } else {
        X_for_ordering <- X
      }
      output[["res_order"]] <- res_order <- tryReturn(
        determine_order(X = X_for_ordering, ordering = ordering, n = n,
                        verbose = verbose),
        output = output,
        return_on_error = return_on_error)
    } else {
      if(isTRUE(verbose)) message("Using supplied observation ordering...")
      output[["res_order"]] <- res_order <- override[["res_order"]]
    }
    
    # Verify that res_order is an ordering.
    tryReturn(
      if(!isOrdering(x = res_order, len = n)) stop("res_order not an ordering"),
      return_on_error = return_on_error, output = output
    )
    
    
    # Define r
    if(is.null(override[["r"]])){
      if(isTRUE(verbose)) message("Calculating transformation anchors...")
      output[["r"]] <- r <- tryReturn(
        define_r(n = n, p = p, res_order = order(res_order), gs_tol = control[["qr_tol"]]),
        output = output, return_on_error = return_on_error)
    } else {
      if(isTRUE(verbose)) message("Using supplied transformation anchors...")
      output[["r"]] <- r <- as.matrix(override[["r"]])
      stopifnot(all(dim(mu) == dim(r)))
    }
    
    # Validate r
    rtr <- crossprod(r)
    tryReturn(
      if(!isTRUE(all.equal(rtr, diag(nrow(rtr)), check.attributes = FALSE,
                           tolerance = control[["orth_tol"]])))
        stop("Columns of r are not orthogonal"),
      output = output, return_on_error = return_on_error
    )
    r <- r[order(res_order), , drop = FALSE]
    
    if(isTRUE(verbose)) message("Calculating r_tilde...")
    output[["r_tilde"]] <- r_tilde <- tryReturn(
      define_r_tilde(r = r, mu = mu, k2_tol = control[["trans_tol"]]),
      output = output, return_on_error = return_on_error)
    
    if(isTRUE(verbose)) message("Calculating residuals...")
    output[["residuals"]][["raw"]] <- raw_residuals <- tryReturn(
      calc_resid(Y = Y, fitted_values = fitted_values),
      output = output, return_on_error = return_on_error)
    
    tryReturn(
      validate_numeric(raw_residuals, min_len = 1, func = warning,
                       message = "Raw residuals failed validation: "),
      output = output, return_on_error = return_on_error)
    
    output[["residuals"]][["sphered"]] <- sphered_residuals <- tryReturn(
      calc_sphered_resid(raw_residuals = raw_residuals, Q = Q),
      output = output, return_on_error = return_on_error)
    
    tryReturn(
      validate_numeric(sphered_residuals, min_len = 1, func = warning,
                       message = "Sphered residuals failed validation: "),
      output = output, return_on_error = return_on_error)
    
    output[["residuals"]][["transformed"]] <- transformed_residuals <- tryReturn(
      calc_k2_resid(x = sphered_residuals, r_tilde = r_tilde, mu = mu,
                    k2_tol = control[["trans_tol"]]),
      output = output, return_on_error = return_on_error)
    
    tryReturn(
      validate_numeric(transformed_residuals, min_len = 1, func = warning,
                       message = "Transformed residuals failed validation: "),
      output = output, return_on_error = return_on_error)
    
    
    if(isTRUE(group)){
      if(length(unique(res_order)) == n) {
        if(isTRUE(verbose)) message("All covariate observations are unique; no grouping done...")
        grouping_matrix <- NULL
      } else {
        if(isTRUE(verbose)) message("Determining grouping...")
        # control[["data"]] should NOT have response in it here if "ordering" is
        # "natural".
        X_for_gm <- if(is.null(control[["data"]])) as.data.frame(X) else control[["data"]]
        output[["grouping_matrix"]] <- grouping_matrix <- tryReturn(
          define_grouping_matrix(X = X_for_gm, res_order = res_order),
          output = output, return_on_error = return_on_error)
      }
    } else {
      grouping_matrix <- NULL
    }
    
    if(isTRUE(verbose)) message("Calculating empirical partial sum process...")
    output[["epsp"]] <- epsp <- tryReturn(
      calc_epsp(transformed_residuals = transformed_residuals,
                res_order = order(res_order), grouping_matrix = grouping_matrix),
      output = output, return_on_error = return_on_error)
    
    tryReturn(
      validate_numeric(epsp, min_len = 1, func = warning,
                       message = "Partial sum process failed validation: "),
      output = output, return_on_error = return_on_error)
    
    if(isTRUE(verbose)) message("Calculating observed statistic(s)...")
    observed_stats <- tryReturn(
      as.list(calc_stats(epsp = epsp, stat = stat)),
      output = output, return_on_error = return_on_error)
    names(observed_stats) <- stat
    output[["observed_stats"]] <- observed_stats
    
    if(is.null(override[["theoretical_stats"]])){
      if(isTRUE(verbose)) message("Running Monte Carlo simulation...")
      theoretical_stats <- mcsim(B = B, r = r, grouping_matrix = grouping_matrix,
                           stat = stat, verbose = verbose)
    } else {
      if(isTRUE(verbose)) message("Using supplied Monte Carlo simulation results...")
      theoretical_stats <- override[["theoretical_stats"]]
    }
    output[["theoretical_stats"]] <- theoretical_stats
    
    # Now redefine "p" to hold p values (not dimension of parameter space).
    p <- tryReturn(
      calc_p(observed_stats = observed_stats, theoretical_stats = theoretical_stats),
      output = output, return_on_error = return_on_error)
    output[["p"]] <- list()
    output[["p"]][["value"]] <- p
    output[["p"]][["mcse"]] <- tryReturn(
      calc_mcse_binom(p = p, B = B),
      output = output, return_on_error = return_on_error)
    
    
    class(output) <- "distfreereg"
    return(output)
  }

simulate_response.formula <- function(true_mean, ..., true_method, true_method_args, true_data){
  if(all(c("x", "y") %in% names(formals(true_method)))){
    true_method_args[["x"]] <- TRUE
    true_method_args[["y"]] <- TRUE
  }
  model <- do.call(get(true_method),
                   args = combine_lists(list(formula = true_mean, data = true_data),
                                        true_method_args))
  simulate_response(true_mean = model, ...)
}

library(distfreereg)
all.equal.compare <- distfreereg:::all.equal.compare

set.seed(20250509)
n <- 60
Sig <- rWishart(1, df = n, Sigma = diag(n))[,,1]
theta <- c(2,3)
true_func <- function(theta, x) theta[1]*x[1] + theta[2]*x[2]
test_func <- function(theta, x) theta[1]*x[1] + theta[2]*x[2]^2
true_data <- data.frame(a = rexp(n, rate = 1), b = rnorm(n))
true_X <- as.matrix(true_data)
X <- true_X + rexp(length(true_X))
m <- lm(b ~ a, data = true_data)

cdfr <- compare(true_mean = true_func,
                test_mean = test_func,
                true_X = true_X,
                X = X,
                true_covariance = list(Sigma = Sig),
                covariance = list(Sigma = Sig),
                theta_init = rep(1, length(theta)),
                theta = theta, prog = Inf, keep = 1,
                manual = function(x) sum(x$theta_hat),
                reps = 3e2,
                control = list(return_on_error = FALSE))

cdfr

edf <- function(n, reps) matrix(rnorm(n*reps), nrow = n)

cdfr_edf <- update(cdfr, err_dist_fun = "edf")
cdfr_edf

signif(rejection(cdfr, alpha = c(0.1, 0.05))[,2:3], digits = 3)

signif(ks.test(cdfr)[["p.value"]], digits = 3)

plot(cdfr, confband_args = NULL)
plot(cdfr, confband_args = list(N = 1e3))
plot(cdfr, which = "dens", confband_args = NULL)
plot(cdfr, which = "qq")
plot(cdfr, which = "qqp")



# No reps, Y warning, custom tolerances

cdfr_no_reps <- compare(true_mean = true_func,
                        test_mean = test_func,
                        true_X = true_X,
                        Y = rep(1, n),
                        X = X,
                        true_covariance = list(Sigma = Sig),
                        covariance = list(Sigma = Sig),
                        theta_init = rep(1, length(theta)),
                        theta = theta, prog = Inf,
                        manual = function(x) sum(x$theta_hat),
                        control = list(return_on_error = FALSE,
                                       matsqrt_tol = -1e-3,
                                       solve_tol = 1e-3,
                                       sym_tol = 1e-3,
                                       sym_tol1 = 1e-3),
                        reps = 0)



# Override

smaller_n <- 1:(n/2)
cdfr_override <- compare(true_mean = true_func,
                                test_mean = test_func,
                                true_X = true_X[smaller_n,],
                                # Y = rep(1, smaller_n),
                                X = X[smaller_n,],
                                true_covariance = list(Sigma = Sig[smaller_n,smaller_n]),
                                covariance = list(Sigma = Sig[smaller_n,smaller_n]),
                                theta_init = rep(1, length(theta)),
                                theta = theta, prog = Inf,
                                manual = function(x) sum(x$theta_hat),
                                control = list(return_on_error = FALSE),
                                override = list(theta_hat = theta),
                                reps = 10)




# Keep all

cdfr_keep_all <- compare(true_mean = true_func,
                         test_mean = test_func,
                         true_X = true_X,
                         Y = rep(1, n),
                         X = X, keep = "all",
                         true_covariance = list(Sigma = Sig),
                         covariance = list(Sigma = Sig),
                         theta_init = rep(1, length(theta)),
                         theta = theta, prog = Inf,
                         manual = function(x) sum(x$theta_hat),
                         control = list(return_on_error = FALSE),
                         override = list(theta_hat = theta),
                         reps = 10)

stopifnot(length(cdfr_keep_all[["dfrs"]]) == 10)


# n supplied

cdfr_n <- compare(true_mean = true_func,
                  test_mean = test_func,
                  n = length(smaller_n),
                  true_X = true_X[smaller_n,],
                  X = X[smaller_n,], keep = "all",
                  true_covariance = list(Sigma = Sig[smaller_n, smaller_n]),
                  covariance = list(Sigma = Sig[smaller_n, smaller_n]),
                  theta_init = rep(1, length(theta)),
                  theta = theta, prog = Inf,
                  manual = function(x) sum(x$theta_hat),
                  control = list(return_on_error = FALSE),
                  # override = list(theta_hat = theta),
                  reps = 4e2)

plot(x = cdfr, y = cdfr_n, confband_args = NULL)
plot(x = cdfr, y = cdfr_n, confband_args = NULL, which = "dens")
plot(x = cdfr, y = cdfr_n, confband_args = NULL, which = "qq")

# List supplied for Sigma

Sig_array <- rWishart(2, df = n/2, Sigma = diag(n/2))
Sig_list <- list(Sig_array[,,1], Sig_array[,,2])

cdfr_list <- compare(true_mean = true_func,
                     test_mean = test_func,
                     true_X = true_X,
                     X = X,
                     true_covariance = list(Sigma = Sig_list),
                     covariance = list(Sigma = Sig),
                     theta_init = rep(1, length(theta)),
                     theta = theta, prog = Inf, reps = 3,
                     control = list(return_on_error = FALSE))



### Partial outputs

cdfr_partial <- compare(true_mean = true_func,
                        test_mean = test_func,
                        true_X = true_X,
                        Y = rep(1, n),
                        X = X, keep = "all",
                        true_covariance = list(Sigma = Sig),
                        covariance = list(Sigma = Sig),
                        theta_init = rep(1, length(theta)),
                        theta = theta, prog = Inf,
                        control = list(orth_tol = 1e-100),
                        reps = 10)
names(cdfr_partial)



### Missing values in true_mean model object

true_data_missing <- true_data
missing_indices <- c(1, n+2)
is.na(true_data_missing)[missing_indices] <- TRUE
m_missing <- update(m, data = true_data_missing)
m_omitted <- update(m, data = true_data[-(missing_indices %% n), ])

tryCatch(compare(true_mean = m_missing, test_mean = m, reps = 10),
         error = function(e) warning(e))

set.seed(20250714)
cdfr_missing <- compare(true_mean = m_missing, test_mean = m_missing,
                        prog = Inf, reps = 10)
set.seed(20250714)
cdfr_omitted <- compare(true_mean = m_omitted, test_mean = m_omitted,
                        prog = Inf, reps = 10)

stopifnot(all.equal(cdfr_missing, cdfr_omitted))



### Subsetting in true_mean model object

m_sub_post <- update(m, data = true_data, subset = a > median(a))
m_sub_pre <- update(m, data = true_data[true_data$a > median(true_data$a), ])

tryCatch(compare(true_mean = m_sub_post, test_mean = m, reps = 10),
         error = function(e) warning(e))

set.seed(20250714)
cdfr_sub_post <- compare(true_mean = m_sub_post, test_mean = m_sub_post,
                        prog = Inf, reps = 10)
set.seed(20250714)
cdfr_sub_pre <- compare(true_mean = m_sub_pre, test_mean = m_sub_pre,
                        prog = Inf, reps = 10)

stopifnot(all.equal(cdfr_sub_post, cdfr_sub_pre))


### Failures

tryCatch(plot(cdfr, confband_args = 3), error = function(e) warning(e))
tryCatch(plot(cdfr, density_args = 3), error = function(e) warning(e))
tryCatch(plot(cdfr, hlines = 3), error = function(e) warning(e))
tryCatch(plot(cdfr, curve_args = 3), error = function(e) warning(e))
tryCatch(plot(cdfr, poly = 3), error = function(e) warning(e))
tryCatch(plot(cdfr, legend = 3), error = function(e) warning(e))
tryCatch(plot(cdfr, qqline = 3), error = function(e) warning(e))
tryCatch(plot(cdfr, stat = "hello"), error = function(e) warning(e))

tryCatch(compare(true_mean = true_func,
                 true_X = true_X,
                 X = X,
                 true_covariance = list(Sigma = Sig),
                 covariance = list(Sigma = Sig),
                 theta_init = rep(1, length(theta)),
                 theta = theta, prog = Inf, keep = 1,
                 manual = function(x) sum(x$theta_hat),
                 control = list(return_on_error = FALSE)),
         error = function(e) warning(e))

tryCatch(compare(true_mean = true_func,
                 test_mean = test_func,
                 true_X = true_X,
                 X = 1:10,
                 true_covariance = list(Sigma = Sig),
                 covariance = list(Sigma = Sig),
                 theta_init = rep(1, length(theta)),
                 theta = theta, prog = Inf, keep = 1,
                 manual = function(x) sum(x$theta_hat),
                 control = list(return_on_error = FALSE)),
         error = function(e) warning(e))

tryCatch(compare(true_mean = function(a) a^2,
                 test_mean = test_func,
                 true_X = true_X,
                 X = X,
                 true_covariance = list(Sigma = Sig),
                 covariance = list(Sigma = Sig),
                 theta_init = rep(1, length(theta)),
                 theta = theta, prog = Inf, keep = 1,
                 manual = function(x) sum(x$theta_hat),
                 control = list(return_on_error = FALSE)),
         error = function(e) warning(e))

tryCatch(compare(true_mean = true_func,
                 test_mean = test_func,
                 true_X = NULL,
                 X = X,
                 true_covariance = list(Sigma = Sig),
                 covariance = list(Sigma = Sig),
                 theta_init = rep(1, length(theta)),
                 theta = theta, prog = Inf, keep = 1,
                 manual = function(x) sum(x$theta_hat),
                 control = list(return_on_error = FALSE)),
         error = function(e) warning(e))

tryCatch(compare(true_mean = true_func,
                 test_mean = test_func,
                 true_X = true_X,
                 n = 0,
                 X = X,
                 true_covariance = list(Sigma = Sig),
                 covariance = list(Sigma = Sig),
                 theta_init = rep(1, length(theta)),
                 theta = theta, prog = Inf, keep = 1,
                 manual = function(x) sum(x$theta_hat),
                 control = list(return_on_error = FALSE)),
         error = function(e) warning(e))

tryCatch(compare(true_mean = true_func,
                 test_mean = test_func,
                 true_X = true_X,
                 X = X,
                 true_covariance = NULL,
                 covariance = list(Sigma = Sig),
                 theta_init = rep(1, length(theta)),
                 theta = theta, prog = Inf, keep = 1,
                 manual = function(x) sum(x$theta_hat),
                 control = list(return_on_error = FALSE)),
         error = function(e) warning(e))

tryCatch(compare(true_mean = function(x) x,
                 test_mean = test_func,
                 true_X = true_X,
                 X = X,
                 true_covariance = list(Sigma = Sig),
                 covariance = list(Sigma = Sig),
                 theta_init = rep(1, length(theta)),
                 theta = theta, prog = Inf, keep = 1,
                 manual = function(x) sum(x$theta_hat),
                 control = list(return_on_error = FALSE)),
         error = function(e) warning(e))

err_dist_fun_vec <- function() 1:10
err_dist_fun_mat <- function() matrix(1:10)

tryCatch(compare(true_mean = true_func,
                 test_mean = test_func,
                 true_X = true_X,
                 X = X,
                 true_covariance = list(Sigma = Sig),
                 covariance = list(Sigma = Sig),
                 theta_init = rep(1, length(theta)),
                 theta = theta, prog = Inf, keep = 1,
                 control = list(return_on_error = FALSE),
                 err_dist_fun = c("err_dist_fun_vec", "err_dist_fun_mat")),
         error = function(e) warning(e))

tryCatch(compare(true_mean = true_func,
                 test_mean = test_func,
                 true_X = true_X,
                 X = X,
                 true_covariance = list(Sigma = Sig),
                 covariance = list(Sigma = Sig),
                 theta_init = rep(1, length(theta)),
                 theta = theta, prog = Inf, keep = 1,
                 control = list(return_on_error = FALSE),
                 err_dist_fun = "pi"),
         error = function(e) warning(e))

tryCatch(compare(true_mean = true_func,
                 test_mean = test_func,
                 true_X = true_X,
                 X = X,
                 true_covariance = list(Sigma = Sig),
                 covariance = list(Sigma = Sig),
                 theta_init = rep(1, length(theta)),
                 theta = theta, prog = Inf, keep = 1,
                 control = list(return_on_error = FALSE),
                 err_dist_args = "pi"),
         error = function(e) warning(e))

tryCatch(compare(true_mean = true_func,
                 test_mean = test_func,
                 true_X = true_X,
                 X = X,
                 true_covariance = list(Sigma = Sig),
                 covariance = list(Sigma = Sig),
                 theta_init = rep(1, length(theta)),
                 theta = theta, prog = Inf, keep = 1,
                 manual = function(x) sum(x$theta_hat),
                 control = list(return_on_error = FALSE),
                 err_dist_fun = "err_dist_fun_vec"),
         error = function(e) warning(e))

tryCatch(compare(true_mean = true_func,
                 test_mean = test_func,
                 true_X = true_X,
                 X = X,
                 true_covariance = list(Sigma = Sig),
                 covariance = list(Sigma = Sig),
                 theta_init = rep(1, length(theta)),
                 theta = theta, prog = Inf, keep = 1,
                 manual = function(x) sum(x$theta_hat),
                 control = list(return_on_error = FALSE),
                 err_dist_fun = "err_dist_fun_mat"),
         error = function(e) warning(e))

tryCatch(compare(true_mean = true_func,
                 test_mean = test_func,
                 true_X = true_X,
                 true_data = as.data.frame(true_X),
                 X = X,
                 true_covariance = list(Sigma = Sig),
                 covariance = list(Sigma = Sig),
                 theta_init = rep(1, length(theta)),
                 theta = theta, prog = Inf, keep = 1,
                 manual = function(x) sum(x$theta_hat),
                 control = list(return_on_error = FALSE),
                 err_dist_fun = "err_dist_fun_mat"),
         error = function(e) warning(e))

tryCatch(compare(true_mean = b ~ a,
                 true_method = "lm",
                 true_method_args = "4",
                 test_mean = test_func,
                 true_X = true_X,
                 true_data = as.data.frame(true_X),
                 X = X,
                 true_covariance = list(Sigma = Sig),
                 covariance = list(Sigma = Sig),
                 theta_init = rep(1, length(theta)),
                 theta = theta, prog = Inf, keep = 1,
                 manual = function(x) sum(x$theta_hat),
                 control = list(return_on_error = FALSE),
                 err_dist_fun = "err_dist_fun_mat"),
         error = function(e) warning(e))

tryCatch(compare(true_mean = b ~ a,
                 true_method = "lm",
                 test_mean = test_func,
                 true_X = true_X,
                 X = X,
                 true_covariance = list(Sigma = Sig),
                 covariance = list(Sigma = Sig),
                 theta_init = rep(1, length(theta)),
                 theta = theta, prog = Inf, keep = 1,
                 manual = function(x) sum(x$theta_hat),
                 control = list(return_on_error = FALSE),
                 err_dist_fun = "err_dist_fun_mat"),
         error = function(e) warning(e))

tryCatch(compare(true_mean = b ~ a,
                 true_method = "lm",
                 test_mean = test_func,
                 true_data = as.data.frame(true_X),
                 X = X,
                 true_covariance = list(Sigma = Sig),
                 covariance = list(Sigma = Sig),
                 theta_init = rep(1, length(theta)),
                 theta = theta, prog = Inf, keep = 1,
                 manual = function(x) sum(x$theta_hat),
                 control = list(return_on_error = FALSE),
                 err_dist_fun = "err_dist_fun_mat"),
         error = function(e) warning(e))

tryCatch(compare(true_mean = true_func,
                 true_method = "lm",
                 test_mean = test_func,
                 true_X = true_X,
                 X = X,
                 true_covariance = list(Sigma = Sig),
                 covariance = list(Sigma = Sig),
                 theta_init = rep(1, length(theta)),
                 theta = theta, prog = Inf, keep = 1,
                 manual = function(x) sum(x$theta_hat),
                 control = list(return_on_error = FALSE),
                 err_dist_fun = "err_dist_fun_mat"),
         error = function(e) warning(e))

tryCatch(compare(true_mean = true_func,
                 true_method_args = "lm",
                 test_mean = test_func,
                 true_X = true_X,
                 X = X,
                 true_covariance = list(Sigma = Sig),
                 covariance = list(Sigma = Sig),
                 theta_init = rep(1, length(theta)),
                 theta = theta, prog = Inf, keep = 1,
                 manual = function(x) sum(x$theta_hat),
                 control = list(return_on_error = FALSE),
                 err_dist_fun = "err_dist_fun_mat"),
         error = function(e) warning(e))

tryCatch(compare(true_mean = true_func,
                 true_method_args = "lm",
                 test_mean = test_func,
                 true_X = true_X,
                 X = X,
                 true_covariance = list(Sigma = Sig),
                 covariance = list(Sigma = Sig),
                 theta_init = rep(1, length(theta)),
                 theta = theta, prog = Inf, keep = 1,
                 manual = function(x) sum(x$theta_hat),
                 control = list(return_on_error = FALSE),
                 err_dist_fun = "err_dist_fun_mat"),
         error = function(e) warning(e))

tryCatch(compare(true_mean = b ~ a,
                 true_method = "lm",
                 test_mean = test_func,
                 true_data = as.data.frame(true_X),
                 data = 1:n,
                 true_covariance = list(Sigma = Sig),
                 covariance = list(Sigma = Sig),
                 theta_init = rep(1, length(theta)),
                 prog = Inf, keep = 1,
                 manual = function(x) sum(x$theta_hat),
                 control = list(return_on_error = FALSE),
                 err_dist_fun = "err_dist_fun_mat"),
         error = function(e) warning(e))

tryCatch(compare(true_mean = b ~ a,
                 true_method = "lm",
                 test_mean = test_func,
                 true_data = as.data.frame(true_X),
                 data = data.frame(),
                 true_covariance = list(Sigma = Sig),
                 covariance = list(Sigma = Sig),
                 theta_init = rep(1, length(theta)),
                 prog = Inf, keep = 1,
                 manual = function(x) sum(x$theta_hat),
                 control = list(return_on_error = FALSE),
                 err_dist_fun = "err_dist_fun_mat"),
         error = function(e) warning(e))

tryCatch(compare(true_mean = b ~ a,
                 true_method = "lm",
                 test_mean = test_func,
                 true_data = as.data.frame(true_X),
                 data = as.data.frame(true_X),
                 X = true_X,
                 true_covariance = list(Sigma = Sig),
                 covariance = list(Sigma = Sig),
                 theta_init = rep(1, length(theta)),
                 prog = Inf, keep = 1,
                 manual = function(x) sum(x$theta_hat),
                 control = list(return_on_error = FALSE),
                 err_dist_fun = "err_dist_fun_mat"),
         error = function(e) warning(e))

tryCatch(compare(true_mean = m,
                 test_mean = test_func,
                 true_data = true_data,
                 data = true_X,
                 covariance = list(Sigma = Sig),
                 theta_init = rep(1, length(theta)),
                 prog = Inf, keep = 1),
         error = function(e) warning(e))

tryCatch(compare(true_mean = m,
                 test_mean = test_func,
                 theta = 1,
                 data = true_X,
                 covariance = list(Sigma = Sig),
                 theta_init = rep(1, length(theta)),
                 prog = Inf, keep = 1),
         error = function(e) warning(e))

tryCatch(compare(true_mean = m,
                 test_mean = m,
                 data = true_X,
                 covariance = list(Sigma = Sig),
                 theta_init = rep(1, length(theta)),
                 prog = Inf, keep = 1),
         error = function(e) warning(e))

tryCatch(compare(true_mean = m,
                 test_mean = test_func,
                 covariance = list(Sigma = Sig),
                 theta_init = rep(1, length(theta)),
                 prog = Inf, keep = 1),
         error = function(e) warning(e))

tryCatch(compare(true_mean = m,
                 test_mean = test_func,
                 X = true_X,
                 data = true_X,
                 covariance = list(Sigma = Sig),
                 theta_init = rep(1, length(theta)),
                 prog = Inf, keep = 1),
         error = function(e) warning(e))

tryCatch(compare(true_mean = m,
                 test_mean = b ~ a,
                 X = true_X,
                 data = true_X,
                 covariance = list(Sigma = Sig),
                 theta_init = rep(1, length(theta)),
                 prog = Inf, keep = 1),
         error = function(e) warning(e))

tryCatch(compare(true_mean = m,
                 test_mean = m,
                 true_covariance = list(Sigma = Sig),
                 prog = Inf, keep = 1),
         error = function(e) warning(e))

tryCatch(compare(true_mean = m,
                 test_mean = m,
                 prog = Inf, keep = "hello"),
         error = function(e) warning(e))

tryCatch(compare(true_mean = m,
                 test_mean = m,
                 prog = Inf, manual = "hello"),
         error = function(e) warning(e))

tryCatch(compare(true_mean = m,
                 test_mean = m,
                 prog = Inf, err_dist_fun = "hello"),
         error = function(e) warning(e))

tryCatch(compare(true_mean = m,
                 test_mean = m,
                 prog = Inf, err_dist_args = "hello"),
         error = function(e) warning(e))

tryCatch(compare(true_mean = m,
                 test_mean = m,
                 prog = Inf, keep = rep(1, 2e4)),
         error = function(e) warning(e))

tryCatch(compare(true_mean = m,
                 method = "lm",
                 test_mean = e ~ a,
                 data = true_X,
                 prog = Inf, keep = 1),
         error = function(e) warning(e))

tryCatch(plot(x = cdfr, y = cdfr), error = function(e) warning(e))
tryCatch(plot(x = cdfr, y = "cdfr"), error = function(e) warning(e))
tryCatch(plot(x = cdfr, y = cdfr_override, which = "qqp"),
         error = function(e) warning(e))
tryCatch(plot(x = cdfr, y = cdfr_override, which = "hello"),
         error = function(e) warning(e))

tryCatch(rejection(cdfr, alpha = 1.1), error = function(e) warning(e))
tryCatch(rejection(cdfr, alpha = -1), error = function(e) warning(e))

tryCatch(plot(x = cdfr, which = "qqqq"), error = function(e) warning(e))

tryCatch(compare(true_mean = m,
                 method = "lm",
                 test_mean = b + e ~ a,
                 data = true_X,
                 prog = Inf, keep = 1),
         error = function(e) warning(e))

tryCatch(compare(true_mean = m,
                 method = "lm",
                 test_mean = b ~ a,
                 prog = Inf, keep = 1),
         error = function(e) warning(e))

tryCatch(compare(true_mean = m,
                 test_mean = m,
                 true_X = true_X,
                 prog = Inf, keep = 1),
         error = function(e) warning(e))

tryCatch(compare(true_mean = true_func,
                 true_X = true_X,
                 true_covariance = list(Sigma = Sig),
                 theta = rep(1, length(theta) + 1),
                 test_mean = m,
                 prog = Inf, keep = 1),
         error = function(e) warning(e))






###########################

# These test various combinations of test mean specifications with compare(),
# specifically to test ordering options.

rm(list=ls())
all.equal.compare <- distfreereg:::all.equal.compare
set.seed(20250613)
n <- 20
reps <- 10
data <- data.frame(x1 = rnorm(n), x2 = rnorm(n), y = rnorm(n))

# lm

form_lm <- y ~ x1 + x2
m_lm_1 <- lm(y ~ x1 + x2, data = data)
m_lm_2 <- lm(form_lm, data = data)

set.seed(20250614)
cdfr_lm_1 <- compare(true_mean = m_lm_1, test_mean = m_lm_1, reps = 10, prog = Inf)
set.seed(20250614)
cdfr_lm_2 <- compare(true_mean = m_lm_1, test_mean = m_lm_2, reps = 10, prog = Inf)
signif(cdfr_lm_1[["empirical_stats"]], 3)
signif(cdfr_lm_1[["p"]][["KS"]], 3)
stopifnot(all.equal(cdfr_lm_1, cdfr_lm_2))

set.seed(20250614)
cdfr_lm_1_optimal <- update(cdfr_lm_1, ordering = "optimal")
set.seed(20250614)
cdfr_lm_2_optimal <- update(cdfr_lm_2, ordering = "optimal")
signif(cdfr_lm_1_optimal[["empirical_stats"]], 3)
signif(cdfr_lm_1_optimal[["p"]][["KS"]], 3)
stopifnot(all.equal(cdfr_lm_1_optimal, cdfr_lm_2_optimal))

set.seed(20250614)
cdfr_lm_1_natural <- update(cdfr_lm_1, ordering = "natural")
set.seed(20250614)
cdfr_lm_2_natural <- update(cdfr_lm_2, ordering = "natural")
signif(cdfr_lm_1_natural[["empirical_stats"]], 3)
signif(cdfr_lm_1_natural[["p"]][["KS"]], 3)
stopifnot(all.equal(cdfr_lm_1_natural, cdfr_lm_2_natural))


# glm

m_glm_1 <- glm(y ~ x1 + x2, data = data)
m_glm_2 <- glm(form_lm, data = data)

set.seed(20250614)
cdfr_glm_1 <- compare(true_mean = m_glm_1, test_mean = m_glm_1, reps = 10, prog = Inf)
set.seed(20250614)
cdfr_glm_2 <- compare(true_mean = m_glm_1, test_mean = m_glm_2, reps = 10, prog = Inf)
signif(cdfr_glm_1[["empirical_stats"]], 3)
signif(cdfr_glm_1[["p"]][["KS"]], 3)
stopifnot(all.equal(cdfr_glm_1, cdfr_glm_2))

set.seed(20250614)
cdfr_glm_1_optimal <- update(cdfr_glm_1, ordering = "optimal")
set.seed(20250614)
cdfr_glm_2_optimal <- update(cdfr_glm_2, ordering = "optimal")
signif(cdfr_glm_1_optimal[["empirical_stats"]], 3)
signif(cdfr_glm_1_optimal[["p"]][["KS"]], 3)
stopifnot(all.equal(cdfr_glm_1_optimal, cdfr_glm_2_optimal))

set.seed(20250614)
cdfr_glm_1_natural <- update(cdfr_glm_1, ordering = "natural")
set.seed(20250614)
cdfr_glm_2_natural <- update(cdfr_glm_2, ordering = "natural")
signif(cdfr_glm_1_natural[["empirical_stats"]], 3)
signif(cdfr_glm_1_natural[["p"]][["KS"]], 3)
stopifnot(all.equal(cdfr_glm_1_natural, cdfr_glm_2_natural))



# nls

form_nls <- y ~ a*x1 + b*x2
m_nls_1 <- nls(y ~ a*x1 + b*x2, data = data, start = c(a = 1, b = 1))
m_nls_2 <- nls(form_nls, data = data, start = c(a = 1, b = 1))

set.seed(20250614)
cdfr_nls_1 <- compare(true_mean = m_nls_1, test_mean = m_nls_1,
                      true_covariance = list(Sigma = 1), reps = 10, prog = Inf)
set.seed(20250614)
cdfr_nls_2 <- compare(true_mean = m_nls_1, test_mean = m_nls_2,
                      true_covariance = list(Sigma = 1), reps = 10, prog = Inf)
signif(cdfr_nls_1[["empirical_stats"]], 3)
signif(cdfr_nls_1[["p"]][["KS"]], 3)
stopifnot(all.equal(cdfr_nls_1, cdfr_nls_2))

set.seed(20250614)
cdfr_nls_1_optimal <- update(cdfr_nls_1, ordering = "optimal")
set.seed(20250614)
cdfr_nls_2_optimal <- update(cdfr_nls_2, ordering = "optimal")
signif(cdfr_nls_1_optimal[["empirical_stats"]], 3)
signif(cdfr_nls_1_optimal[["p"]][["KS"]], 3)
stopifnot(all.equal(cdfr_nls_1_optimal, cdfr_nls_2_optimal))

set.seed(20250614)
cdfr_nls_1_natural <- update(cdfr_nls_1, ordering = "natural")
set.seed(20250614)
cdfr_nls_2_natural <- update(cdfr_nls_2, ordering = "natural")
signif(cdfr_nls_1_natural[["empirical_stats"]], 3)
signif(cdfr_nls_1_natural[["p"]][["KS"]], 3)
stopifnot(all.equal(cdfr_nls_1_natural, cdfr_nls_2_natural))

# Verify failure when subsetting is "self-referential", leading to decreasing
# sample size with each repetition.

nls_bad_subset <- update(m_nls_1, subset = x1 > median(x1))
tryCatch(compare(true_mean = nls_bad_subset, test_mean = nls_bad_subset,
                 true_covariance = list(Sigma = 1),
                 covariance = list(Sigma = 1)),
         error = function(e) warning(e))



# lmer

if(require(lme4)){
  set.seed(20250614)
  data$g <- factor(sample(1:2, size = n, replace = TRUE))
  
  control_list <- list(action = "ignore", tol = 1)
  control_spec <- lmerControl(check.conv.singular = control_list,
                              check.conv.grad = control_list,
                              check.conv.hess = control_list)
  form_lmer <- y ~ x1 + x2 + (1 | g)
  
  suppressWarnings({
    m_lmer_1 <- lmer(y ~ x1 + x2 + (1 | g), data = data, control = control_spec)
    m_lmer_2 <- lmer(form_lmer, data = data, control = control_spec)
    
    set.seed(20250614)
    cdfr_lmer_1 <- compare(true_mean = m_lmer_1, test_mean = m_lmer_1,
                           reps = 10, prog = Inf)
    set.seed(20250614)
    cdfr_lmer_2 <- compare(true_mean = m_lmer_1, test_mean = m_lmer_2,
                           reps = 10, prog = Inf)
    print(signif(cdfr_lmer_1[["empirical_stats"]], 3))
    print(signif(cdfr_lmer_1[["p"]][["KS"]], 3))
    stopifnot(all.equal(cdfr_lmer_1, cdfr_lmer_2))
    
    set.seed(20250614)
    cdfr_lmer_1_optimal <- update(cdfr_lmer_1, ordering = "optimal")
    set.seed(20250614)
    cdfr_lmer_2_optimal <- update(cdfr_lmer_2, ordering = "optimal")
    print(signif(cdfr_lmer_1_optimal[["empirical_stats"]], 3))
    print(signif(cdfr_lmer_1_optimal[["p"]][["KS"]], 3))
    stopifnot(all.equal(cdfr_lmer_1_optimal, cdfr_lmer_2_optimal))
    
    set.seed(20250614)
    cdfr_lmer_1_natural <- update(cdfr_lmer_1, ordering = "natural")
    set.seed(20250614)
    cdfr_lmer_2_natural <- update(cdfr_lmer_2, ordering = "natural")
    print(signif(cdfr_lmer_1_natural[["empirical_stats"]], 3))
    print(signif(cdfr_lmer_1_natural[["p"]][["KS"]], 3))
    stopifnot(all.equal(cdfr_lmer_1_natural, cdfr_lmer_2_natural))
  })
}

simulate_response.nls <- function(true_mean, ...){
  func <- formula(true_mean)[[3]]
  envr <- c(as.list(coefficients(true_mean)), as.list(get_data(true_mean)))
  Y_mean <- tryCatch(with(envr, eval(func)),
                     error = function(e) stop("Unable to generate Y_mean from 'nls' object: ", e))
  errors <- generate_error_matrix(...)
  output <- as.matrix(Y_mean + errors)
  rownames(output) <- seq_len(nrow(output))
  return(output)
}

print.distfreereg <-
  function(x, ..., digits = 3, col_sep = 2, show_params = TRUE){
    stopifnot(is(x, "distfreereg"), is.numeric(digits), is.numeric(col_sep),
              isTRUE(show_params) || isFALSE(show_params))
    
    stopifnot(!is.null(x[["observed_stats"]]), !is.null(names(x[["observed_stats"]])),
              !is.null(x[["p"]][["value"]]), !is.null(x[["data"]][["Y"]]),
              !is.null(x[["theoretical_stats"]][[1]]))
    
    if(is.null(x[["theta_hat"]])){
      show_params <- FALSE
    }
    
    # Parameter_estimates
    if(isTRUE(show_params)){
      parameters <- sprintf(paste0("%1.", digits, "e"), x[["theta_hat"]])
      names(parameters) <- names(x[["theta_hat"]])
    }
    
    # Stats
    B <- length(x[["theoretical_stats"]][[1]])
    j <- length(names(x[["observed_stats"]]))
    col_names <- c("Stat", "Value", "Pr(>Value)", "MCSE")
    stat_matrix <- matrix(NA_character_, nrow = 1 + j, ncol = 4)
    stat_matrix[1,] <- col_names
    for(i in seq_len(j)){
      if(identical(x[["p"]][["value"]][[i]], 0)){
        prepend <- "<"
        x[["p"]][["value"]][[i]] <- 1/B
      } else {
        if(identical(x[["p"]][["value"]][[i]], 1)){
          prepend <- ">"
          x[["p"]][["value"]][[i]] <- 1-1/B
        } else {
          prepend <- ""
        }
      }
      stat_matrix[i+1, 1] <- names(x[["observed_stats"]])[i]
      stat_matrix[i+1, 2:4] <- sprintf(paste0("%1.", digits, "e"),
                                       c(x[["observed_stats"]][[i]],
                                         x[["p"]][["value"]][[i]],
                                         x[["p"]][["mcse"]][[i]]))
      stat_matrix[i+1, 3] <- paste0(prepend, stat_matrix[i+1, 3])
    }
    
    buffered_stat_matrix <- buffer_matrix(stat_matrix)
    
    # Output
    cat("Number of observations:", length(x[["fitted_values"]]))
    cat("\nLength of EPSP:", length(x[["epsp"]]), "\n")
    cat("Monte Carlo simulations:", B, "\n")
    if(isTRUE(show_params)){
      cat("\nEstimated parameter values:\n")
      print.default(parameters, quote = FALSE, digits = digits)
    }
    cat("\nObserved statistics:\n")
    for(i in seq_len(nrow(stat_matrix)))
      cat(paste(buffered_stat_matrix[i,],
                collapse = paste0(rep(" ", times = col_sep), collapse = "")), "\n")
    cat("---\n`MCSE' is the Monte Carlo standard error of the estimated p-value.\n\n")
  }

mcsim <-
  function(B, r, grouping_matrix, stat, verbose){
    stopifnot(isTRUE(as.integer(B) == B), identical(length(B), 1L),
              is.matrix(r), is.numeric(r))
    n <- nrow(r)
    tr <- t(r)
    if(is.null(grouping_matrix)){
      order_for_calc_epsp <- seq_len(n)
    } else {
      stopifnot(is.numeric(grouping_matrix), is.matrix(grouping_matrix),
                identical(nrow(r), nrow(grouping_matrix)))
      order_for_calc_epsp <- seq_len(ncol(grouping_matrix))
      proj_matrix <- grouping_matrix - r %*% (tr %*% grouping_matrix)
    }
    output <- matrix(NA, nrow = B, ncol = length(stat))
    for(i in seq_len(B)){
      if(isTRUE(verbose) && isTRUE(i %% as.integer(B/10) == 0))
        message("...", i, " of ", B)
      epsilon <- matrix(rnorm(n))
      if(is.null(grouping_matrix)){
        e_proj <- as.vector(epsilon - r %*% (tr %*% epsilon))
      } else {
        e_proj <- as.vector(t(epsilon) %*% proj_matrix)
      }
      epsp <- calc_epsp(transformed_residuals = e_proj, res_order = order_for_calc_epsp,
                        grouping_matrix = NULL)
      output[i,] <- calc_stats(epsp = epsp, stat = stat)
    }
    output <- as.list(as.data.frame(output))
    names(output) <- stat
    return(output)
  }

refit_model <- function(m, verbose){
  if((is(m, "lm") || is(m, "glm")) && (is.null(m[["x"]]) || is.null(m[["y"]]))){
    if(isTRUE(verbose)) message("'x' and/or 'y' not found in model object; refitting model...")
    m <- update(object = m, x = TRUE, y = TRUE)
  }
  return(m)
}

fill_covariance_list <- function(need, covariance_list, matsqrt_tol, solve_tol){
    # This function is used by compare.default() to fill in required but missing
    # elements from the covariance list (that is, the list that contains up to
    # four equivalent specifications of a covariance structure).  The function
    # that generates the errors can have any of these as an argument, and this
    # function fills in any gaps.
    #
    # The order of the conditional clauses is determined by trying to find an
    # "optimal" route to the desired matrix given what is already present in the
    # list.  Preference is given to operations on the "square root" level (that
    # is, SqrtSigma and Q) when a direct route from a given matrix to the
    # desired one is not available.

    stopifnot(all(need %in% c("Sigma", "SqrtSigma", "P", "Q")),
              is.list(covariance_list))

    # First, calculate Sigma if necessary.
    if("Sigma" %in% need && is.null(covariance_list[["Sigma"]])){
        if(!is.null(covariance_list[["SqrtSigma"]])){
            covariance_list[["Sigma"]] <- covsquare(covariance_list[["SqrtSigma"]])
        } else {
            if(is.null(covariance_list[["P"]])){
                covariance_list[["SqrtSigma"]] <- covinv(covariance_list[["Q"]], tol = solve_tol)
                covariance_list[["Sigma"]] <- covsquare(covariance_list[["SqrtSigma"]])
            } else {
                covariance_list[["Sigma"]] <- covinv(covariance_list[["P"]], tol = solve_tol)
            }
        }
    }

    # Next, SqrtSigma.
    if("SqrtSigma" %in% need && is.null(covariance_list[["SqrtSigma"]])){
        if(!is.null(covariance_list[["Q"]])){
            covariance_list[["SqrtSigma"]] <- covinv(covariance_list[["Q"]], tol = solve_tol)
        } else {
            if(is.null(covariance_list[["Sigma"]])){
                covariance_list[["Q"]] <- covsqrt(covariance_list[["P"]], tol = matsqrt_tol)
                covariance_list[["SqrtSigma"]] <- covinv(covariance_list[["Q"]], tol = solve_tol)
            } else {
                covariance_list[["SqrtSigma"]] <- covsqrt(covariance_list[["Sigma"]], tol = matsqrt_tol)
            }
        }
    }

    # Now P.
    if("P" %in% need && is.null(covariance_list[["P"]])){
        if(!is.null(covariance_list[["Q"]])){
            covariance_list[["P"]] <- covsquare(covariance_list[["Q"]])
        } else {
            if(is.null(covariance_list[["Sigma"]])){
                covariance_list[["Q"]] <- covinv(covariance_list[["SqrtSigma"]], tol = solve_tol)
                covariance_list[["P"]] <- covsquare(covariance_list[["Q"]])
            } else {
                covariance_list[["P"]] <- covinv(covariance_list[["Sigma"]], tol = solve_tol)
            }
        }
    }

    # Finally, Q.
    if("Q" %in% need && is.null(covariance_list[["Q"]])){
        if(!is.null(covariance_list[["SqrtSigma"]])){
            covariance_list[["Q"]] <- covinv(covariance_list[["SqrtSigma"]], tol = solve_tol)
        } else {
            if(!is.null(covariance_list[["P"]])){
                covariance_list[["Q"]] <- covsqrt(covariance_list[["P"]], tol = matsqrt_tol)
            } else {
                covariance_list[["SqrtSigma"]] <-
                    covsqrt(covariance_list[["Sigma"]], tol = matsqrt_tol)
                covariance_list[["Q"]] <- covinv(covariance_list[["SqrtSigma"]], tol = solve_tol)
            }
        }
    }

    return(covariance_list)
}

# Intended for package testing only.
all.equal.compare <- function(target, current, tolerance = 1e-3, ...){
  Y <- all.equal(target = target[["Y"]],
                 current = current[["Y"]],
                 tolerance = tolerance, ...)
  empirical_stats <- all.equal(target = target[["empirical_stats"]],
                              current = current[["empirical_stats"]],
                              tolerance = tolerance, ...)
  theoretical_stats <- all.equal(target = target[["theoretical_stats"]],
                           current = current[["theoretical_stats"]],
                           tolerance = tolerance, ...)
  p <- all.equal(target = target[["p"]],
                 current = current[["p"]],
                 tolerance = tolerance, ...)
  
  return(all(isTRUE(Y), isTRUE(empirical_stats), isTRUE(theoretical_stats), isTRUE(p)))
}

validate_weights.default <- function(m, method_args = NULL){
  w <- method_args[["weights"]] %||% weights(m)
  if(!is.null(w)) validate_numeric(x = w, positive = TRUE,
                                   message = "Model weights failed validation: ")
}

default_distfreereg_tol <- function(){
  list(matsqrt_tol = -(.Machine[["double.eps"]])^0.25,
       solve_tol = .Machine[["double.eps"]],
       qr_tol = sqrt(.Machine[["double.eps"]]),
       orth_tol = sqrt(.Machine[["double.eps"]]),
       trans_tol = sqrt(.Machine[["double.eps"]]),
       sym_tol = sqrt(.Machine[["double.eps"]]),
       sym_tol1 = 8*sqrt(.Machine[["double.eps"]]))
}

library(distfreereg)

all.equal.distfreereg <- distfreereg:::all.equal.distfreereg
test_dfr_functions <- distfreereg:::test_dfr_functions

n <- 1e2
func <- function(X, theta) theta[1] + theta[2]*X[,1] + theta[3]*X[,2]
set.seed(20250516)
Sig <- rWishart(1, df = n, Sigma = diag(n))[,,1]
X <- matrix(rexp(2*n, rate = 1), nrow = n)
theta <- c(2,5,1)
Y <- distfreereg:::f2ftheta(f = func, X)(theta) +
  as.vector(distfreereg:::rmvnorm(n = n, reps = 1, mean = rep(0,n), SqrtSigma = distfreereg:::matsqrt(Sig)))

df_lm <- as.data.frame(cbind(Y, X, rep(1:10, 10)))
names(df_lm) <- c("z", "x", "y", "g")
set.seed(20250516)
wt <- rexp(n) + 1
set.seed(20250516)
dfr_form_lm <- distfreereg(test_mean = z ~ x + y, data = df_lm,
                           method_args = list(weights = wt),
                           method = "lm", verbose = FALSE,
                           control = list(return_on_error = FALSE))

dfr_form_lm

set.seed(20250516)
dfr_form_lm_no_weights <- distfreereg(test_mean = z ~ x + y, data = df_lm,
                                      method = "lm")

newdata_lm <- data.frame(a = rnorm(10), b = rnorm(10))
test_dfr_functions(dfr_form_lm, newdata = newdata_lm)


m <- lm(z ~ x + y, data = df_lm, weights = wt)

set.seed(20250516)
dfr_lm <- distfreereg(test_mean = m, verbose = FALSE,
                      control = list(return_on_error = FALSE))
set.seed(20250516)
dfr_lm_verbose <- distfreereg(test_mean = m,
                              control = list(return_on_error = FALSE),
                              override = list(J = dfr_lm[["J"]],
                                              fitted_values = dfr_lm[["fitted_values"]]))

dfr_lm
test_dfr_functions(dfr_lm, newdata = newdata_lm)

stopifnot(all.equal(dfr_lm, dfr_form_lm))
stopifnot(all.equal(dfr_lm, dfr_lm_verbose))

cdfr_form_lm <- asymptotics(dfr_form_lm, reps = 5)
cdfr_lm <- asymptotics(dfr_lm, reps = 5)

signif(rejection(cdfr_form_lm, alpha = c(0.1, 0.5))[,2:3], digits = 3)
signif(rejection(cdfr_lm, alpha = c(0.1, 0.5))[,2:3], digits = 3)


dfr_lm_but_not_lm <- dfr_lm
class(dfr_lm_but_not_lm[["test_mean"]]) <- "wrong"
tryCatch(asymptotics(dfr_lm_but_not_lm, reps = 5),
         error = function(e) warning(e))

vcov(dfr_lm, jacobian_args = list("ignored"))

# Orderings

set.seed(20250516)
dfr_lm_asis <- update(dfr_lm, ordering = "asis")
set.seed(20250516)
dfr_form_lm_asis <- update(dfr_form_lm, ordering = "asis")
stopifnot(all.equal(dfr_lm_asis, dfr_form_lm_asis))

set.seed(20250516)
dfr_lm_optimal <- update(dfr_lm, ordering = "optimal")
set.seed(20250516)
dfr_form_lm_optimal <- update(dfr_form_lm, ordering = "optimal")
stopifnot(all.equal(dfr_lm_optimal, dfr_form_lm_optimal))

set.seed(20250516)
dfr_lm_natural <- update(dfr_lm, ordering = "natural")
set.seed(20250516)
dfr_form_lm_natural <- update(dfr_form_lm, ordering = "natural")
stopifnot(all.equal(dfr_lm_natural, dfr_form_lm_natural))

dfr_lm_natural_grouped <- update(dfr_lm, ordering = "natural", group = TRUE,
                                 verbose = TRUE)

set.seed(20250516)
dfr_lm_g_character <- update(dfr_lm, ordering = list("g"))

dfr_lm_g_character

set.seed(20250516)
dfr_form_lm_g <- update(dfr_form_lm, ordering = list("g"))
stopifnot(all.equal(dfr_lm_g_character, dfr_form_lm_g))

df_lm[dfr_lm_g_character[["res_order"]],][["g"]]
df_lm[dfr_form_lm_g[["res_order"]],][["g"]]

set.seed(20250516)
dfr_lm_g_character_grouped <- update(dfr_lm_g_character, group = TRUE)
set.seed(20250516)
dfr_form_lm_g_grouped <- update(dfr_form_lm_g, group = TRUE)
stopifnot(all.equal(dfr_lm_g_character_grouped, dfr_form_lm_g_grouped))



### Failures

tryCatch(update(dfr_lm, theta_init = 1), error = function(e) warning(e))
tryCatch(update(dfr_lm, ordering = list("h")), error = function(e) warning(e))
tryCatch(update(dfr_lm, ordering = list(1, "g")), error = function(e) warning(e))
tryCatch(update(dfr_lm, ordering = c(1)), error = function(e) warning(e))
tryCatch(update(dfr_lm, ordering = list(1:10)), error = function(e) warning(e))

dfr_lm_fail <- update(dfr_lm, override = list(r = matrix(rnorm(length(dfr_lm[["J"]])), nrow = n)),
                      control = list(return_on_error = TRUE))

tryCatch(update(dfr_form_lm, test_mean = z + y ~ x), error = function(e) warning(e))

tryCatch(update(dfr_form_lm, group = "hello"), error = function(e) warning(e))

tryCatch(distfreereg(test_mean = z ~ x + y, data = df_lm,
                     X = matrix(),
                     method = "lm", verbose = FALSE,
                     control = list(return_on_error = FALSE)),
         error = function(e) warning(e)
)

tryCatch(distfreereg(test_mean = z ~ x + y, data = df_lm,
                     Y = vector(),
                     method = "lm", verbose = FALSE,
                     control = list(return_on_error = FALSE)),
         error = function(e) warning(e)
)

tryCatch(distfreereg(test_mean = m,
                     X = matrix(),
                     verbose = FALSE,
                     control = list(return_on_error = FALSE)),
         error = function(e) warning(e)
)

tryCatch(distfreereg(test_mean = m,
                     Y = vector(),
                     verbose = FALSE,
                     control = list(return_on_error = FALSE)),
         error = function(e) warning(e)
)

tryCatch(distfreereg(test_mean = m,
                     covariance = list(Sigma = 1),
                     control = list(return_on_error = FALSE)),
         error = function(e) warning(e)
)

# This default method covers cases when test_mean is a function or NULL.
get_cov.default <- function(m, covariance, verbose){
  if(isTRUE(verbose)) message("Using supplied covariance structure...")
  return(covariance)
}

plot_confband <- function(cb, confband_args){
  stopifnot(is.null(confband_args) || is.list(confband_args))
  density_args <- confband_args[["density_args"]]
  curve_args <- confband_args[["curve_args"]]
  polygon_args <- confband_args[["polygon_args"]]
  stopifnot(is.null(density_args) || is.list(density_args),
            is.null(curve_args) || is.list(curve_args),
            is.null(polygon_args) || is.list(polygon_args))
  shade_col <- if(is.null(confband_args[["shade_col"]])) rgb(0,0,0,0.15) else confband_args[["shade_col"]]

  lower_curve_args_default <- list(lty = "dotted")
  upper_curve_args_default <- list(lty = "dotted")
  lower_curve_args <- combine_lists(curve_args[["lower"]], lower_curve_args_default)
  upper_curve_args <- combine_lists(curve_args[["upper"]], upper_curve_args_default)
  curve_args[["lower"]] <- curve_args[["upper"]] <- NULL
  lower_curve_args <- combine_lists(list(x = cb[["w"]], y = cb[["cb_lower"]]),
                                curve_args, lower_curve_args)
  upper_curve_args <- combine_lists(list(x = cb[["w"]], y = cb[["cb_upper"]]),
                                curve_args, upper_curve_args)

  do.call(lines, args = lower_curve_args)
  do.call(lines, args = upper_curve_args)

  if(!isFALSE(polygon_args)){
    default_polygon_args <- list(x = c(cb[["w"]], rev(cb[["w"]])),
                                 y = c(cb[["cb_upper"]], rev(cb[["cb_lower"]])),
                                 col = shade_col, border = NA)
    polygon_args <- combine_lists(polygon_args, default_polygon_args)
    do.call(polygon, args = polygon_args)
  }
  
  invisible(cb)
}

k2 <-
function(x, a, b, k2_tol){
  stopifnot(is.numeric(x), is.vector(x),
            is.numeric(a), is.vector(a),
            is.numeric(b), is.vector(b),
            identical(length(x), length(a)), identical(length(x), length(b)),
            is.numeric(k2_tol), k2_tol > 0)
  normalizer <- 1 - sum(a*b)
  if(isTRUE(all.equal(normalizer, 0, check.attributes = FALSE,
                      tolerance = k2_tol))){
    warning("Matching columns found in mu and r_tilde that have scalar product nearly equal to 1")
    output <- x
  } else {
    output <- x - sum((a-b)*x)*(a-b)/normalizer
  }
  stopifnot(is.finite(output))
  return(output)
}

create_confband <- function(x, func, confband_args){
  stopifnot(identical(func, "dens") || identical(func, "ecdf"),
            is.numeric(x), is.null(confband_args) || is.list(confband_args))
  density_args <- confband_args[["density_args"]]
  confband_args["density_args"] <- confband_args["curve_args"] <-
    confband_args["polygon_args"] <- confband_args["shade_col"] <- NULL

  if(identical(func, "dens")){
    dens <- function(x){
      dens_out <- do.call(density,
                          args = combine_lists(list(x = x), density_args))
      output <- approxfun(x = dens_out$x, y = dens_out$y, yleft = 0, yright = 0)
      return(output)
    }
  }
  confband_default_args <-
      list(x = x, w = NULL, m = 100, batch_len = 50,
           N = 5e4, conf.level = 0.95, q_probs = c(0.025, 0.975),
           matsqrt_tol = default_distfreereg_tol()[["matsqrt_tol"]])
  confband_args <- combine_lists(list(func = get(func)), confband_args,
                                 confband_default_args)
  cb <- do.call(confband, args = confband_args)
  
  if(identical(func, "ecdf")) cb[["cb_upper"]] <- pmin(1, cb[["cb_upper"]])
  cb[["cb_lower"]] <- pmax(0, cb[["cb_lower"]])
  
  return(cb)
}

# Calculating res_order using column-wise ordering should refer to data argument
# rather than X matrix formed by, e.g., lm().
define_data_for_res_order <- function(test_mean, data, ordering){
  if(identical(ordering, "natural") || is.list(ordering)){
    data_for_ordering <- data
    data_for_ordering[[get_response(test_mean)]] <- NULL
    data_for_ordering[["(weights)"]] <- NULL
  } else {
    data_for_ordering <- get_X(m = test_mean)
  }
  return(data_for_ordering)
}

if(require("lme4")){
  library(distfreereg)
  all.equal.distfreereg <- distfreereg:::all.equal.distfreereg
  test_dfr_functions <- distfreereg:::test_dfr_functions
  
  n <- 1e2
  set.seed(20250516)
  df_lmerMod <- data.frame(x = rnorm(n),
                           z = rnorm(n),
                           state = factor(sample(c("CA", "OR", "WA"), size = n,
                                                 replace = TRUE, prob = c(2,1,1))),
                           party = factor(sample(c("D", "R"), size = n, replace = TRUE)),
                           g = rep(1:10, 10))
  state_num <- ifelse(df_lmerMod$state == "CA", 7, ifelse(df_lmerMod$state == "OR", 2, 3))
  party_num <- ifelse(df_lmerMod$party == "D", 1, -1)
  theta <- c(2,1)
  df_lmerMod$y <- theta[1]*df_lmerMod$x + theta[2]*df_lmerMod$z + state_num + rnorm(n)
  
  m <- lmer(y ~ x + (1|state), data = df_lmerMod)
  
  set.seed(20250516)
  dfr_lmerMod <- distfreereg(m, verbose = FALSE, control = list(return_on_error = FALSE))
  print(dfr_lmerMod)
  
  set.seed(20250516)
  dfr_lmerMod_J <- distfreereg(test_mean = m,
                               override = list(J = dfr_lmerMod[["J"]]),
                               control = list(return_on_error = FALSE))
  
  set.seed(20250516)
  dfr_lmerMod_fitted <- distfreereg(test_mean = m,
                                    override = list(fitted_values = dfr_lmerMod[["fitted_values"]]),
                                    control = list(return_on_error = FALSE))
  
  stopifnot(all.equal(dfr_lmerMod, dfr_lmerMod_J))
  stopifnot(all.equal(dfr_lmerMod, dfr_lmerMod_fitted))
  
  set.seed(20250516)
  newdata_lmerMod <- data.frame(y = rnorm(10), x = rnorm(10), z = rnorm(10))
  test_dfr_functions(dfr_lmerMod, newdata = newdata_lmerMod)
  print(signif(predict(dfr_lmerMod, newdata = newdata_lmerMod, re.form = NA), 3))
  
  set.seed(20250516)
  dfr_form_lmerMod <- distfreereg(y ~ x + (1|state), data = df_lmerMod,
                                  method = "lmer", verbose = FALSE)
  
  print(dfr_form_lmerMod)
  
  set.seed(20250516)
  dfr_form_lmerMod_verbose <- distfreereg(y ~ x + (1|state), data = df_lmerMod,
                                          method = "lmer")
  
  stopifnot(all.equal(dfr_lmerMod, dfr_form_lmerMod))
  
  cdfr_form_lmerMod <- asymptotics(dfr_form_lmerMod, reps = 5)
  cdfr_lmerMod <- asymptotics(dfr_lmerMod, reps = 5)
  
  print(signif(rejection(cdfr_form_lmerMod, alpha = c(0.1, 0.5))[,2:3], digits = 3))
  print(signif(rejection(cdfr_lmerMod, alpha = c(0.1, 0.5))[,2:3], digits = 3))
  
  # Orderings
  
  set.seed(20250516)
  dfr_lmerMod_asis <- update(dfr_lmerMod, ordering = "asis")
  set.seed(20250516)
  dfr_form_lmerMod_asis <- update(dfr_form_lmerMod, ordering = "asis")
  stopifnot(all.equal(dfr_lmerMod_asis, dfr_form_lmerMod_asis))
  
  set.seed(20250516)
  dfr_lmerMod_optimal <- update(dfr_lmerMod, ordering = "optimal")
  set.seed(20250516)
  dfr_form_lmerMod_optimal <- update(dfr_form_lmerMod, ordering = "optimal")
  stopifnot(all.equal(dfr_lmerMod_optimal, dfr_form_lmerMod_optimal))
  
  set.seed(20250516)
  dfr_lmerMod_natural <- update(dfr_lmerMod, ordering = "natural")
  set.seed(20250516)
  dfr_form_lmerMod_natural <- update(dfr_form_lmerMod, ordering = "natural")
  stopifnot(all.equal(dfr_lmerMod_natural, dfr_form_lmerMod_natural))
  
  set.seed(20250516)
  dfr_lmerMod_g <- update(dfr_lmerMod, ordering = list("g"))
  set.seed(20250516)
  dfr_form_lmerMod_g <- update(dfr_form_lmerMod, ordering = list("g"))
  stopifnot(all.equal(dfr_lmerMod_g, dfr_form_lmerMod_g))
  
  print(df_lmerMod[dfr_lmerMod_g[["res_order"]],][["g"]])
  
  set.seed(20250516)
  dfr_lmerMod_g_grouped <- update(dfr_lmerMod_g, group = TRUE)
  set.seed(20250516)
  dfr_form_lmerMod_g_grouped <- update(dfr_form_lmerMod_g, group = TRUE)
  stopifnot(all.equal(dfr_lmerMod_g_grouped, dfr_form_lmerMod_g_grouped))
  
  
  
  
  ### Partial output
  
  dfr_lmerMod_partial <- distfreereg(test_mean = m, verbose = FALSE,
                                     control = list(orth_tol = 1e-100))
  names(dfr_lmerMod_partial)
}

matsquare <-
function(mat){
  stopifnot(is.numeric(mat))
  if(is.matrix(mat)){
    stopifnot(identical(diff(dim(mat)), 0L))
    output <- crossprod(mat)
  } else {
    stopifnot(is.vector(mat))
    # This allows covariance specification in the form of a numeric vector.
    output <- mat * mat
  }
  return(output)
}

get_theta_hat.function <- function(test_mean, ..., ftheta, Y, Q, control){
  optimization_output <-
    calc_theta_hat(ftheta = ftheta, Y = Y, Q = Q,
                   optimization_fun = control[["optimization_fun"]],
                   optimization_args = control[["optimization_args"]],
                   fun_to_optimize_arg = control[["fun_to_optimize_arg"]])
  theta_hat <- optimization_output[[control[["theta_hat_name"]]]]
  if(is.null(theta_hat))
    stop("No element with name '", control[["theta_hat_name"]],
         "' found in optimization output")
  return(optimization_output)
}

validate_function_behavior <- function(f, X, n, theta){
  theta_char <- deparse1(substitute(theta))
  f_char <- deparse1(substitute(f))
  f_out <- tryCatch(f2ftheta(f = f, X = X[,, drop = FALSE], n = n)(theta),
                    error = function(e) stop("Unable to evaluate ", f_char, "(X, ",
                    theta_char, "): ", e))
  validate_numeric(x = f_out, len = n, message = "Function output failed numeric validation: ")
  
  if(!missing(theta) && !is.null(theta)){
    ftheta <- f2ftheta(f = f, X = X[1,, drop = FALSE], n = n)
    for(i in seq_along(theta)){
      theta_na <- theta
      theta_na[i] <- NA
      f_out <- tryCatch(ftheta(theta_na), error = function(e) return(e))
      if(!is(f_out, "error") && all(!is.na(f_out)))
        stop("Element ", i, " of ", theta_char, " unnecessary for function evaluation. ",
             "Recode ", f_char, ", or shorten ", theta_char, ", and retry.")
    }
  }
}

natural_order <- function(data, cols){
  # data should be only a matrix or a data frame at this point
  stopifnot(is.matrix(data) || is.data.frame(data))
  if(is.list(cols)) cols <- unlist(cols)
  df <- as.data.frame(data)
  cols <- paste0("df[['", cols, "']]", collapse = ", ")
  eval(str2expression(paste0("order(", cols, ")")))
}

validate_covariance_list <- function(covariance_list, n, sym_tol, sym_tol1){
  list_name <- deparse1(substitute(covariance_list))
  validate_named_list(covariance_list, valid_names = c("Sigma", "SqrtSigma", "P", "Q"))
  if(length(covariance_list) == 0) stop(list_name, " must have at least one element")
  
  cov_val <- function(x){
    if(!is.null(covariance_list[[x]])){
      if(!is.numeric(covariance_list[[x]]) && !is.list(covariance_list[[x]]))
        stop("All specified elements of ", list_name, " element ", x, " must be numeric or a list of matrices")
      if(is.list(covariance_list[[x]])){
        if(!all(sapply(covariance_list[[x]], is.matrix)) || !all(sapply(covariance_list[[x]], is.numeric)))
          stop("All elements of list specification must be matrices")
      } else {
        if(is.matrix(covariance_list[[x]])){
          validate_sqmat(covariance_list[[x]], n,
                         message = c("Error in validating ", list_name, " element ", x, " (length>1): "))
        } else {
          if(length(covariance_list[[x]]) != 1 && length(covariance_list[[x]]) != n){
            stop("Invalid covariance vector length")
          } else {
            if(any(covariance_list[[x]] <= 0)) stop("Elements of a vector specification for covariance must be positive")
          }
        }
      }
    } 
  }

  lapply(names(covariance_list), cov_val)
  
  if(!is.null(covariance_list[["Sigma"]])){
    validate_symmetric(covariance_list[["Sigma"]], name = "Sigma",
                       sym_tol = sym_tol, sym_tol1 = sym_tol1)
  }
  if(!is.null(covariance_list[["P"]])){
    validate_symmetric(covariance_list[["P"]], name = "P",
                       sym_tol = sym_tol, sym_tol1 = sym_tol1)
  }
}

validate_args_compare <-
  function(true_mean,
           true_method = true_method,
           true_method_args = true_method_args,
           true_covariance,
           true_X,
           true_data,
           theta,
           n,
           reps,
           prog,
           err_dist_fun,
           err_dist_args,
           keep,
           manual,
           update_args,
           arg_list){
    if(is.function(true_mean) && !is.null(true_data)){
      stop("true_data must not be supplied when true_mean is a function")
    }
    
    if(is(true_mean, "formula")){
      strict_match(arg = true_method, choices = valid_model_methods())
      if(!is.null(true_method_args) && !is.list(true_method_args))
        stop("true_method_args must be NULL or a list")
      if(is.null(true_data)) stop("true_data must be supplied when true_mean is a formula")
      if(!is.null(theta)) stop("theta must be NULL when true_mean is 'formula'")
    } else {
      if(!is.null(true_method)) stop("true_method must be NULL when true_mean is not a formula")
      if(!is.null(true_method_args)) stop("true_method_args must be NULL when true_mean is not a formula")
      if(!is.function(true_mean)){
        if(!is.null(true_data)) stop("true_data must be NULL when true_mean is a model object")
      }
    }
    
    if(!is.null(theta) && isModel(true_mean))
      stop("theta must be NULL when true_mean is a model object")

    test_mean <- arg_list[["test_mean"]]
    if(is.null(test_mean)) stop("test_mean must be specified")
    data <- arg_list[["data"]]
    X <- arg_list[["X"]]
    
    if(!is.null(arg_list[["Y"]])){
      warning("Y supplied to compare() is ignored")
      arg_list[["Y"]] <- NULL
    }
    
    
    if(!is.null(data)){
      if(is.matrix(data)){
        data <- as.data.frame(data)
      } else {
        if(!is.data.frame(data)) stop("data must be a data frame or a matrix")
      }
      if(nrow(data) == 0) stop("data cannot be an empty matrix or data frame")
    }
    
    # Check for appropriate test covariate specification.
    if(isModel(test_mean) && !is.null(data)){
      stop("data must be NULL when test_mean is a model object")
    }
    if(is.function(test_mean)){
      if(isTRUE("x" %in% names(formals(test_mean))) && is.null(X))
        stop("X must be specified when test_mean is a function with an 'x' argument")
    }
    
    test_mean_formula <- tryCatch(formula(test_mean), error = function(e) NULL)
    
    if(is(test_mean_formula, "formula")){
      if(!is.null(X)) stop("X must be NULL when test_mean has a formula method")
      if(is(test_mean, "formula") && is.null(data))
        stop("data must be supplied when test_mean is a formula")
      test_Y <- tryCatch(get_response(test_mean),
                         error = function(e) warning("Error in extracting response variable name: ", e))
      stopifnot("Response variable name not found in test_mean object's formula" = !is.null(test_Y),
                "Response variable name found in test_mean object is not a character!" = is.character(test_Y))
      if(length(test_Y) != 1) stop("Response in test_mean object must be exactly one term")
      data_for_verifying_y <- if(is.null(data)) get_data(test_mean, generation = 3) else data
      if(!(test_Y %in% colnames(data_for_verifying_y))) stop("Response variable '", test_Y, "' not found in data")
    } else {
      if(!is.null(data)) stop("data must be NULL when test_mean has no formula method")
      test_Y <- NULL
    }
    
    if(!is.null(X)){
      validate_numeric(x = X)
      X <- as.matrix(X)
    }
    
    if(is.function(true_mean)){
      if(is.null(true_X)){
        if(any(c("x", "X") %in% names(formals(true_mean))))
          stop("true_mean must have neither 'x' nor 'X' as an argument when true_X is NULL")
      } else {
        if(!(any(c("x", "X") %in% names(formals(true_mean)))))
          stop("true_mean must have 'x' or 'X' as an argument when true_X is not NULL")
        validate_numeric(x = true_X)
        true_X <- as.matrix(true_X)
      }
    } else {
      if(!is.null(true_X)) stop("true_X must be NULL if true_mean is not a function")
    }
    
    # Determine n    
    if(!is.null(n)){
      validate_numeric(x = n, pos_int = TRUE, len = 1)
      n <- as.integer(n)
    }
    n <- get_n(n = n, true_X = true_X, X = X, true_data = true_data,
               data = data, true_mean = true_mean, test_mean = test_mean)
    
    # Define default sym_tol value if NULL.
    if(is.null(arg_list[["control"]][["sym_tol"]])){
      sym_tol <- default_distfreereg_tol()[["sym_tol"]]
    } else {
      sym_tol <- arg_list[["control"]][["sym_tol"]]
    }
    validate_numeric(sym_tol, positive = TRUE, len = 1)
    
    # Define default sym_tol1 value if NULL.
    if(is.null(arg_list[["control"]][["sym_tol1"]])){
      sym_tol1 <- default_distfreereg_tol()[["sym_tol1"]]
    } else {
      sym_tol1 <- arg_list[["control"]][["sym_tol1"]]
    }
    validate_numeric(sym_tol1, positive = TRUE, len = 1)
    
    # Validate format of covariance lists.
    if(is.function(true_mean) || is(true_mean, "nls") || identical(true_method, "nls")){
      if(is.null(true_covariance))
        stop("'true_covariance' must be specified when 'true_mean' is a function or nls object, ",
             "or 'true_method' is 'nls'")
      validate_covariance_list(true_covariance, n = n, sym_tol = sym_tol,
                               sym_tol1 = sym_tol1)
    } else {
      if(!is.null(true_covariance))
        stop("'true_covariance' must be NULL when 'true_mean' is neither a function nor nls object, ",
             "and 'true_method' is not 'nls'")
    }
    if(!is.null(arg_list[["covariance"]])){
      validate_covariance_list(arg_list[["covariance"]], n = n, sym_tol = sym_tol,
                               sym_tol1 = sym_tol1)
    }
    
    # Validate true mean function.
    if(is.function(true_mean)){
      validate_mean_function_arg_names(f = true_mean)
      validate_numeric(x = theta, min_len = 1)
      validate_function_behavior(f = true_mean, X = true_X, n = n, theta = theta)
    }
    
    
    # Validate other arguments.
    validate_numeric(x = reps, min_val = 0, len = 1)
    reps <- as.integer(reps)
    
    # Note that the following call to validate_numeric() prog does not use
    # pos_int, since the default value reps/10 is 0 for reps < 10, so only
    # require positive and then convert 0 to 1.
    validate_numeric(x = prog, finite = FALSE, positive = TRUE, len = 1)
    if(is.finite(prog)) prog <- as.integer(prog)
    if(prog == 0) prog <- 1
    
    # Check for compatibility of err_dist_fun and err_dist_args.
    if(is(true_mean, "function") || is(true_mean, "nls") || identical(true_method, "nls")){
      if(!is.null(err_dist_fun)){
        if(!is.character(err_dist_fun) || length(err_dist_fun) != 1){
          stop("'err_dist_fun' must be a character string of length 1")
        }
        err_dist_fun <- get(err_dist_fun)
        if(!isTRUE(is.function(err_dist_fun)))
          stop("'err_dist_fun' must specify a function")
      } else {
        err_dist_fun <- rmvnorm
      }
      if(!is.null(err_dist_args) && !is.list(err_dist_args))
        stop("'err_dist_args' must be a list")
    } else {
      if(!is.null(err_dist_fun)){
        stop("'err_dist_fun' must be NULL when class(true_mean) is ", paste(class(true_mean), collapse = ", "),
             if(is(true_mean, "formula")) " (unless 'true_method' is 'nls')")
      }
      if(!is.null(err_dist_args)){
        stop("'err_dist_args' must be NULL when class(true_mean) is ", paste(class(true_mean), collapse = ", "),
             if(is(true_mean, "formula")) " (unless 'true_method' is 'nls')")
      }
    }
    
    if(!is.null(keep) && !identical("all", keep)){
      if(!is.numeric(keep)){
        stop("keep_all must be NULL, 'all', or a vector of positive integers")
      } else {
        validate_numeric(keep, pos_int = TRUE, min_len = 1, max_len = reps,
                         min_val = 1, max_val = reps)
        keep <- sort(unique(keep))
      }
    }
    
    if(!is.null(manual) && !is.function(manual))
      stop("'manual' must be NULL or a function")
    
    # Define default matsqrt_tol value if NULL.
    if(is.null(arg_list[["control"]][["matsqrt_tol"]])){
      matsqrt_tol <- default_distfreereg_tol()[["matsqrt_tol"]]
    } else {
      matsqrt_tol <- arg_list[["control"]][["matsqrt_tol"]]
    }
    validate_numeric(matsqrt_tol, max_val = 0, len = 1)
    
    # Define default solve_tol value if NULL.
    if(is.null(arg_list[["control"]][["solve_tol"]])){
      solve_tol <- default_distfreereg_tol()[["solve_tol"]]
    } else {
      solve_tol <- arg_list[["control"]][["solve_tol"]]
    }
    validate_numeric(solve_tol, positive = TRUE, len = 1)
    
    return(list(reps = reps, prog = prog, true_X = true_X, test_mean = test_mean,
                X = X, test_Y = test_Y, data = data, n = n, err_dist_fun = err_dist_fun,
                keep = keep, arg_list = arg_list, solve_tol = solve_tol,
                matsqrt_tol = matsqrt_tol))
  }

strict_match <-
function(arg, choices){
  # This is intended to be used in place of match.arg() when partial matching is
  # not desired/allowed.
  if(is(choices, "distfreereg") || is(choices, "compare")){
    # When choices is a distfreereg or compare object, automatically set
	# choices to the vector of stat names from object.
    element <- if(is(choices, "distfreereg")) "observed_stats" else
        "empirical_stats"
    choices_internal <- names(choices[[element]])
  } else {
    choices_internal <- choices
  }
  match_ind <- sapply(choices_internal, function(x) identical(arg, x))
  if(isFALSE(length(which(match_ind)) == 1)){
    if(is(choices, "compare")){
      stop("'", arg, "' not found among statistics in compare object")
    } else {
      stop(deparse1(substitute(arg)), " must be exactly one of ",
           paste0("'", choices_internal, "'", collapse = ", "),
           "; it cannot be '", arg, "'")
    }
  }
}

calc_mu <-
function(J, solve_tol, orth_tol){
  stopifnot(is.matrix(J), is.numeric(J), is.numeric(solve_tol),
            length(solve_tol) == 1, solve_tol > 0)
  nR <- crossprod(J)
  nR_sqrt <- tryCatch(t(chol(nR)),
                      error = function(e) stop("Unable to calculate Cholesky decomposition of t(J) %*% J: ", e))
  mu <- tryCatch(t(solve(nR_sqrt, t(J), tol = solve_tol)),
                 error = function(e) stop("Unable to solve system to calculate mu: ", e))
  stopifnot("crossprod(mu) is not equal to the identity matrix" =
              isTRUE(all.equal(crossprod(mu), diag(ncol(mu)),
                               check.attributes = FALSE,
                               tolerance = orth_tol)))
  return(mu)
}

calc_jacobian <-
function(ftheta, theta_hat, jacobian_args = NULL){
  stopifnot(is.function(ftheta), is.numeric(theta_hat), is.vector(theta_hat),
            is.null(jacobian_args) || is.list(jacobian_args))
  func <- function(x) ftheta(x)
  arg_list <- c(list(f = func, var = unname(theta_hat)), jacobian_args)
  J <- tryCatch(as.matrix(drop(do.call(calculus::jacobian, args = arg_list))),
                error = function(e) stop("Unable to compute Jacobian: ", e))
  validate_numeric(J)
  stopifnot(identical(length(dim(J)), 2L))
  return(J)
}

calc_theta_hat <-
  function(ftheta, Y, Q, optimization_fun, optimization_args, fun_to_optimize_arg){
    # optimization_args should already be formatted by
    # validate_args_distfreereg_...().
    stopifnot(is.function(ftheta), is.function(optimization_fun), is.numeric(Y),
              is.vector(Y), is.numeric(Q) || is.list(Q))
    target <- function(theta) crossprod(covprod(Q, (Y - ftheta(theta))))
    # Add function to optimize to argument list.
    optimization_args[[fun_to_optimize_arg]] <- target
    optimization_output <-
      tryCatch(do.call(optimization_fun, args = optimization_args),
               error = function(e) stop("Unable to calculate theta_hat: ", e))
    return(optimization_output)
  }

get_jacobian.glm <- function(test_mean, ...){
  stopifnot(is(test_mean, "glm"), !is.null(test_mean[["x"]]))
  f <- function(theta){
    test_mean[["family"]][["linkinv"]](colSums(theta * t(test_mean[["x"]])))
  }
  output <- as.matrix(drop(calculus::jacobian(f = f, var = unname(test_mean[["coefficients"]]))))
  return(output)
}

generate_error_matrix <- function(err_dist_fun, err_dist_args, err_dist_fun_formals,
                                  true_covariance, cov_args, solve_tol, reps, n, ...){
  # Define error-generating function and argument list, then generate errors.
  # message("covariance list names: ", names(true_covariance))
  if(reps > 0){
    err_dist_args_default <-
      create_err_dist_args_default(err_dist_fun_formals = err_dist_fun_formals,
                                   cov_args = cov_args, reps = reps, n = n,
                                   true_covariance = true_covariance)
    
    message("Generating errors...")
    error_matrix <- tryCatch(do.call(err_dist_fun,
                                     combine_lists(err_dist_args, err_dist_args_default)),
                             error = function(e) stop("Unable to generate error matrix: ", e))
    
    # Check error matrix.
    if(!is.matrix(error_matrix)) stop("class of error_matrix is '", class(error_matrix),
                                      "'; should be 'matrix'")
    validate_numeric(error_matrix, message = "Error matrix failed numeric validation: ")
    if(any(dim(error_matrix) != c(n, reps))) stop("dimension of error_matrix is ",
                                                  paste(dim(error_matrix), collapse = ", "),
                                                  "; should be ", n, ", ", reps)
  } else {
    error_matrix <- NULL
  }
  
  return(error_matrix)
}

validate_numeric <- function(x, empty = FALSE, missing = FALSE, nan = FALSE,
                             finite = TRUE, pos_int = FALSE, positive = FALSE,
                             len = NULL, min_len = NULL, max_len = NULL,
                             min_val = NULL, max_val = NULL,
                             min_val_strict = NULL, max_val_strict = NULL,
                             func = stop, message = NULL){
  var_name <- deparse1(substitute(x))
  if(!is.numeric(x)) func(message, var_name, " must be numeric; supplied value has class '", class(x), "'")
  if(isFALSE(empty) && length(x) == 0) func(message, var_name, " cannot be empty")
  if(isFALSE(nan) && any(is.nan(x))) func(message, var_name, " cannot have NaN values")
  if(isFALSE(missing) && any(is.na(x))) func(message, var_name, " cannot have NA values")
  if(isTRUE(finite) && any(!is.finite(x))) func(message, var_name, " must be finite")
  # Some values are converted to integer via as.integer(), and the result must
  # be positive. Require finite values, so prog = Inf does not cause a problem.
  if(isTRUE(pos_int) && all(is.finite(x)) && any(as.integer(x) <= 0))
    func(message, "as.integer(", var_name, ") must be positive")
  if(isTRUE(positive) && any(x <= 0)) func(message, var_name, " must be positive")
  if(!is.null(len) && length(x) != len) func(message, var_name, " must have length ", len, ", not ", length(x))
  if(!is.null(min_len) && length(x) < min_len)
    func(message, var_name, " must have length at least ", min_len)
  if(!is.null(max_len) && length(x) > max_len)
    func(message, var_name, " must have length at most ", max_len)
  if(!is.null(min_val) && any(x < min_val))
    func(message, var_name, " must have value(s) greater than or equal to ", min_val)
  if(!is.null(max_val) && any(x > max_val))
    func(message, var_name, " must have value(s) less than or equal to ", max_val)
  if(!is.null(min_val_strict) && any(x <= min_val_strict))
    func(message, var_name, " must have value(s) greater than ", min_val_strict)
  if(!is.null(max_val_strict) && any(x >= max_val_strict))
    func(message, var_name, " must have value(s) less than ", max_val_strict)
}

# This is intended only for symmetric matrices, until possibly replaced by
# Cholesky factorization.
matsqrt <-
  function(mat, tol = 0, symmetric = TRUE){
    stopifnot(is.numeric(mat), is.numeric(tol), length(tol) == 1, tol <= 0)
    if(is.matrix(mat)){
      if(all(dim(mat) == 1)){
        # Note: this is needed when p = 1, in which case J^tJ is 1x1.
        output <- sqrt(mat)
      } else{
        stopifnot(identical(diff(dim(mat)), 0L))
        eig <- tryCatch(eigen(mat, symmetric = symmetric),
                        error = function(e) stop("Unable to compute eigendecomposition of ",
                                                 deparse1(substitute(mat)), ": ", e))
        stopifnot(is.double(eig[["values"]]), is.double(eig[["vectors"]]),
                  all(eig[["values"]] >= tol))
        e_vals <- pmax(0, eig[["values"]])
        output <- eig[["vectors"]] %*% (sqrt(e_vals) * t(eig[["vectors"]]))
      }
    } else {
      # This allows covariance specification in the form of a single numeric
      # value.
      stopifnot(all(mat > 0))
      output <- sqrt(mat)
    }
    return(output)
  }

confband <- function(x, w, func, m, batch_len, N, conf.level, q_probs,
                     matsqrt_tol){
  n <- length(x); b <- batch_len; a <- n/b
  if(!isInteger(a)) warning("Batch length, ", b,
                            ", does not evenly divide data length, ", n)
  w_range <- quantile(x, probs = q_probs)
  if(is.null(w)) w <- seq(from = w_range[1], to = w_range[2],
                          length.out = m)
  fnw <- func(x = x)(w)
  Sigma_hat <-
      tryCatch(confband_Sigma_hat(x, fnw = fnw, func = func, w = w,
                                  a = a, b = b),
               error = function(e) stop("An confband error occurred when ",
                                        "trying to calculate Sigma_hat: ", e))
  radii <-
      tryCatch(confband_radii(N = N, Sigma_hat = Sigma_hat, df = a - 1,
                              conf.level = conf.level, n = n,
                              matsqrt_tol = matsqrt_tol),
               error = function(e) stop("An confband error occurred when ",
                                        "trying to calculate radii: ", e))
  output <- list(call = match.call(), Sigma_hat = Sigma_hat, radii = radii,
                 w = w, fnw = fnw, cb_lower = fnw - radii,
                 cb_upper = fnw + radii)
  return(output)
}

validate_sqmat <- function(x, n, message = NULL){
  validate_numeric(x)
  stopifnot(is.matrix(x))
  if(any(dim(x) != n)) stop(message, "The dimensions of ", deparse1(substitute(x)),
                            " must be ", n, " by ", n)
}

calc_mcse_binom <- function(p, B){
  stopifnot(is.numeric(unlist(p)), all(unlist(p) >= 0), all(unlist(p) <= 1),
            is.integer(B), length(B) == 1)
  lapply(p, function(x) if(any(x == 0:1)) NA_real_ else sqrt(x*(1-x)/B))
}

validate_args_distfreereg <-
  function(test_mean,
           ordering,
           group,
           stat,
           B,
           control,
           override,
           verbose,
           method,
           method_args,
           theta_init,
           Y,
           X,
           data,
           covariance,
           J,
           fitted_values){
    
    output <- list()
    
    if(!is.null(test_mean) && !is(test_mean, "formula") &&
       !is.function(test_mean) && !isModel(test_mean))
      stop("'test_mean' must be NULL, a function, a formula, or an acceptable model (see documentation).")

    if(!is.null(control)){
      control_names <- c("matsqrt_tol", "solve_tol", "qr_tol", "orth_tol",
                         "trans_tol", "sym_tol", "sym_tol1", "return_on_error")
      if(is.function(test_mean)){
        control_names <- c(control_names, "jacobian_args", "optimization_fun",
                           "fun_to_optimize_arg", "optimization_args",
                           "theta_hat_name", "theta_init_arg")
      }
      validate_named_list(control, valid_names = control_names)
    }
    
    control <- combine_lists(control,
                             list(return_on_error = TRUE),
                             default_distfreereg_tol())
    validate_control_tols(control)
    
    output[["control"]] <- control
    
    return_on_error <- control[["return_on_error"]]
    if(!isTRUE(return_on_error) && !isFALSE(return_on_error))
      stop("control[['return_on_error']] must be TRUE or FALSE")
    

    # weights validated in distfreereg() after converting formula to model.
        
    if(is(test_mean, "formula")){
      validate_single_response_term(form = test_mean)
      strict_match(arg = method, choices = valid_model_methods())
      
      if(!is.data.frame(data)) stop("'data' must be a data frame")
      n <- nrow(data)
      if(n == 0) stop("'data' must have at least one row")
      
      if(!is.null(method_args) && !is.list(method_args))
        stop("method_args must be a list")
    } else {
      if(!is.null(data))
        stop("'data' argument must be NULL when 'test_mean' is not a formula.")
      if(!is.null(method) || !is.null(method_args))
        stop("'method' and 'method_args' must be NULL when 'test_mean' is not a formula.")
    }
    
    if(isModel(test_mean)){
      n <- length(fitted(test_mean))
      validate_single_response_term(form = test_mean)
    }
    
    if(is(test_mean, "formula") || isModel(test_mean)){
      if(!is.null(X) || !is.null(Y)){
        stop("'X' and 'Y' must be NULL when 'test_mean' is a formula or a model")
      }
    } else {
      Y <- validate_Y(Y = Y, verbose = verbose)
      X <- validate_X(X = X, verbose = verbose)
      n <- length(Y)
      # Verify that X and Y dimensions match.
      if(!is.null(X) && nrow(X) != n)
        stop("The number of rows in X must be the length of Y: nrow(X) is ",
             nrow(X), ", but length(Y) is ", n)
      output[["X"]] <- X
      output[["Y"]] <- Y
    }
    
    output[["n"]] <- n
    
    # Verify override.
    if(!is.null(override)){
      override_names <- c("res_order", "r", "theoretical_stats", "J", "fitted_values",
                          "theta_hat")
      validate_named_list(override, valid_names = override_names)
      
      if(!is.null(override[["theta_hat"]]))
        validate_numeric(override[["theta_hat"]],
                         message = "override[['theta_hat']] failed numeric validation: ")
      
      if(!is.null(override[["J"]])){
        validate_numeric(override[["J"]], message = "override[['J']] failed numeric validation: ")
        override[["J"]] <- tryCatch(as.matrix(override[["J"]]),
                                    error = function(e) stop("Error converting override[['J']] to a matrix: ", e))
        if(nrow(as.matrix(override[["J"]])) != n) stop("nrow(as.matrix(override[['J']])) must be equal to length(Y)") 
      }
      
      if(!is.null(override[["J"]]) && !is.null(override[["r"]]) &&
         !all(dim(as.matrix(override[["J"]])) == dim(as.matrix(override[["r"]]))))
        stop("as.matrix(override[['J']]) and as.matrix(override[['r']]) must have the same dimensions")
      
      if(!is.null(override[["res_order"]])){
        if(!isOrdering(override[["res_order"]], len = n)) stop("override[['res_order']] must be an ordering")
      }
      
      if(!is.null(override[["r"]])){
        validate_numeric(override[["r"]], message = "override[['r']] failed numeric validation: ")
        if(nrow(as.matrix(override[["r"]])) != n) stop("nrow(as.matrix(override[['r']])) must be equal to length(Y)") 
      }
      
      if(!is.null(override[["fitted_values"]])){
        validate_numeric(override[["fitted_values"]], len = n,
                         message = "override[['fitted_values']] failed numeric validation: ")
      }
    }
    
    output[["override"]] <- override
    
    if(is.function(test_mean)){
      if(is.null(override[["theta_hat"]])){
        validate_numeric(x = theta_init, min_len = 1, message = "theta_init failed numeric validation: ")
      } else {
        # This is just so function output is verified at theta_hat below.
        theta_init <- override[["theta_hat"]]
      }
    } else {
      if(!is.null(theta_init))
        stop("'theta_init' must be NULL when 'theta_hat' is not a function.")
    }
    
    
    
    # Verify covariance list.
    if(is.function(test_mean) || is.null(test_mean)){
      validate_covariance_list(covariance, n = n, sym_tol = control[["sym_tol"]],
                               sym_tol1 = control[["sym_tol1"]])
    } else {
        if(!is.null(covariance))
            stop("'covariance' must be NULL when 'test_mean' is a model object")
    }
    
    
    # Deal with optimization arguments.
    if(is.function(test_mean)){
      optimization_args <- control[["optimization_args"]]
      if(!is.null(optimization_args) && !is.list(optimization_args))
        stop("control[['optimization_args']] must be a list.")
      if(is.null(control[["optimization_fun"]])){
        if(any(!is.null(control[["fun_to_optimize_arg"]]),
               !is.null(control[["theta_init_arg"]]),
               !is.null(control[["theta_hat_name"]])))
          warning("No alternative optimization function specified; using optim()",
                  " by default, ignoring 'fun_to_optimize_arg', 'theta_init_arg', and 'theta_hat_name'.")
        control[["optimization_fun"]] <- optim
        control[["optimization_args"]] <-
          combine_lists(control[["optimization_args"]], list(method = "BFGS"))
        control[["theta_init_arg"]] <- "par"
        control[["theta_hat_name"]] <- "par"
        control[["fun_to_optimize_arg"]] <- "fn"
      }
      if(!is.function(control[["optimization_fun"]]))
        stop("control[['optimization_fun']] must be a function")
      if(is.character(control[["theta_init_arg"]])){
        if(length(control[["theta_init_arg"]]) != 1)
          stop("control[['theta_init_arg']] must have length 1.")
        if(!(control[["theta_init_arg"]] %in% names(formals(control[["optimization_fun"]]))))
          stop("control[['theta_init_arg']] must be the name of an argument of",
               " control[['optimization_fun']].")
      } else {
        stop("control[['theta_init_arg']] must be a character string.")
      }
      if(is.character(control[["fun_to_optimize_arg"]])){
        if(length(control[["fun_to_optimize_arg"]]) != 1)
          stop("control[['fun_to_optimize_arg']] must have length 1")
        if(!(control[["fun_to_optimize_arg"]] %in% names(formals(control[["optimization_fun"]]))))
          stop("control[['fun_to_optimize_arg']] must be the name of an",
               " argument of control[['optimization_fun']].")
      } else {
        stop("control[['fun_to_optimize_arg']] must be a character string.")
      }
      if(!is.character(control[["theta_hat_name"]])){
        stop("control[['theta_hat_name']] must be a character vector of length 1.")
      } else {
        if(length(control[["theta_hat_name"]]) != 1)
          stop("control[['theta_hat_name']] must have length 1.")
      }
      
      control[["optimization_args"]][[control[["theta_init_arg"]]]] <- theta_init
      
      # Verification Jacobian arguments.
      jacobian_args <- control[["jacobian_args"]]
      if(!is.null(jacobian_args) && !is.list(jacobian_args))
        stop("jacobian_args must be a list")
    }
    
    output[["control"]] <- control
    
    if(is.function(test_mean)){
      if(is.null(names(theta_init))) names(theta_init) <- paste0("theta", seq_along(theta_init))
      
      validate_mean_function_arg_names(f = test_mean)
      if(is.null(X)){
        if(any(c("x", "X") %in% names(formals(test_mean))))
          stop("X must be supplied if 'x' or 'X' is an argument of test_mean")
        if(length(theta_init) != 1) stop("theta_init must have length 1 when X is NULL")
      } else {
        if(!any(c("x", "X") %in% names(formals(test_mean))))
          stop("'x' or 'X' must be an argument of test_mean when X is not NULL")
      }
      
      validate_function_behavior(f = test_mean, X = X, n = n, theta = theta_init)
      
      if(!is.null(override[["theta_hat"]])) theta_init <- NULL
    }
    
    
    # Validate B
    validate_numeric(x = B, pos_int = TRUE, len = 1)
    output[["B"]] <- as.integer(B)
    
    
    # Validate the observation ordering specification.
    if(is.null(override[["res_order"]])){
      if(is.list(ordering)){
        if(isModel(test_mean)) data <- get_data(test_mean)
        if(!is.null(data)){
          validate_order_columns(data, ordering)
        } else {
          validate_order_columns(X, ordering)
        }
      } else {
        if(length(ordering) > 1) stop("ordering method specification must have length one, ",
                                      "or be a list of column specifications")
        strict_match(ordering, c("asis", "optimal", "simplex", "natural"))
      }
    }
    
    # Validate 'group'
    if(!isTRUE(group) && !isFALSE(group)) stop("'group' must be TRUE or FALSE")
    
    # Check stat functions for egregious errors.
    v <- vector(mode = "list", length = length(stat))
    for(i in seq_along(stat)){
      v[[i]] <- tryCatch(get(stat[i])(rnorm(100)),
                         error = function(e) stop("Unable to evaluate ",
                                                  deparse1(substitute(stat[i])),
                                                  "(rnorm(100)): ", e))
    }
    bad_ind <- which(sapply(v, function(x) isFALSE(is.numeric(x)) ||
                              isFALSE(length(x) == 1)))
    if(length(bad_ind) > 0){
      stop("The following stat functions are invalid: ",
           paste(stat[bad_ind], collapse = ", "), ".\nEach failed either ",
           "because it did not return a (non-NA) numeric value or because its ",
           "output is not a single number.")
    }
    
    return(output)
  }

compare <-
  function(true_mean,
           true_method = NULL,
           true_method_args = NULL,
           true_covariance = NULL,
           true_X = NULL,
           true_data = NULL,
           theta = NULL,
           n = NULL,
           reps = 1e3,
           prog = reps/10,
           simulate_args = NULL,
           err_dist_fun = NULL,
           err_dist_args = NULL,
           keep = NULL,
           manual = NULL,
           update_args = NULL,
           ...){
    cl <- match.call()
    supplied_dfr_arg_list <- as.list(match.call(expand.dots = FALSE)$...)
    arg_list <- list(...)
    
    vargs <- validate_args_compare(true_mean = true_mean,
                                   true_method = true_method,
                                   true_method_args = true_method_args,
                                   true_covariance = true_covariance,
                                   true_X = true_X,
                                   true_data = true_data,
                                   theta = theta,
                                   n = n,
                                   reps = reps,
                                   prog = prog,
                                   err_dist_fun = err_dist_fun,
                                   err_dist_args = err_dist_args,
                                   keep = keep,
                                   manual = manual,
                                   update_args = update_args,
                                   arg_list = arg_list)
    
    test_mean <- vargs[["test_mean"]]; err_dist_fun <- vargs[["err_dist_fun"]]
    true_X <- vargs[["true_X"]];  X <- vargs[["X"]];
    test_Y <- vargs[["test_Y"]]; data <- vargs[["data"]];
    n <- vargs[["n"]]; reps <- vargs[["reps"]]; prog <- vargs[["prog"]]
    keep <- vargs[["keep"]]
    matsqrt_tol <- vargs[["matsqrt_tol"]]; solve_tol <- vargs[["solve_tol"]]
    covariance <- supplied_dfr_arg_list[["covariance"]]
    
    if(isModel(test_mean)){
      data <- get_data(test_mean)
    }
    
    if(is.function(true_mean) || is(true_mean, "nls") || identical(true_method, "nls")){
      message("Calculating required true covariance specification matrices...")
      err_dist_fun_formals <- names(formals(err_dist_fun))
      possible_cov_args <- c("Sigma", "SqrtSigma", "P", "Q")
      cov_args <- possible_cov_args[which(possible_cov_args %in% err_dist_fun_formals)]
      true_covariance <- fill_covariance_list(need = intersect(cov_args, possible_cov_args),
                                              covariance_list = true_covariance,
                                              matsqrt_tol = matsqrt_tol,
                                              solve_tol = solve_tol)
    } else {
      err_dist_fun_formals <- NULL
      cov_args <- NULL
      true_covariance <- NULL
    }
    
    Y <- simulate_response(true_mean = true_mean, true_method = true_method,
                           true_method_args = true_method_args,
                           true_covariance = true_covariance, true_X = true_X,
                           true_data = true_data, simulate_args = simulate_args,
                           err_dist_fun = err_dist_fun, err_dist_args = err_dist_args,
                           err_dist_fun_formals = err_dist_fun_formals,
                           cov_args = cov_args, solve_tol = solve_tol,
                           reps = reps, n = n, theta = theta)
    
    if(!is.null(manual)) manual_output <- vector(mode = "list", length = reps)
    if(identical(keep, "all")) keep <- 1:reps
    if(!is.null(keep)) dfrs <- vector("list", length = 0L)
    
    output_covariance <- theoretical_stats <- obs_stats <- p_values <- NULL
    
    message("Running simulation...")
    
    if(!is.null(data)) new_data <- data
    
    for(i in seq_len(reps)){
      if(i %% prog == 0) message("Repetition ", i, " of ", reps)
      new_Y <- Y[,i]
      new_call <- list(quote(distfreereg))
      if(isModel(test_mean)){
        new_data[rownames(Y), test_Y] <- new_Y
        if(all(c("x", "y") %in% names(formals(get(as.list(getCall(test_mean))[[1]]))))){
          new_test_mean <- update(test_mean, data = new_data, x = TRUE, y = TRUE)
        } else {
          new_test_mean <- update(test_mean, data = new_data)
        }
        stopifnot(identical(length(residuals(new_test_mean)), n))
        new_call[["test_mean"]] <- quote(new_test_mean)
      } else {
        if(is.null(data)){
          new_call[["X"]] <- quote(X)
          new_call[["Y"]] <- quote(new_Y)
        } else {
          new_data[rownames(Y), test_Y] <- new_Y
          new_call[["data"]] <- quote(new_data)
        }
      }
      
      # if(!is.null(global_override)) dfr_args[["override"]] <- global_override
      
      if(i == 1){
        new_call <- combine_lists(new_call, supplied_dfr_arg_list, 
                                  list(verbose = FALSE))
        dfr <- eval(as.call(new_call))
        output_covariance <- dfr[["covariance"]]
        theoretical_stats <- dfr[["theoretical_stats"]]
        
        # On first repetition, create some output data frames with the correct
        # dimensions. Doing it here guarantees that the data frames have the
        # correct dimensions and, in particular, the correct column names when no
        # value of "stat" is specified in "...".
        obs_stats <- p_values <- as.data.frame(matrix(NA, nrow = reps,
                                                      ncol = length(dfr[["observed_stats"]])))
        colnames(obs_stats) <- colnames(p_values) <- names(dfr[["observed_stats"]])
      } else {
        # Include res_order, r, and theoretical_stats in override list to prevent
        # updating those because "X" or "data" is always in dfr_args.
        new_call[[1]] <- quote(update.distfreereg)
        new_call[["object"]] <- quote(dfr)
        new_call[["override"]] <-
          combine_lists(as.list(supplied_dfr_arg_list[["override"]])[-1],
                        list(res_order = quote(dfr[["res_order"]]),
                             r = quote(dfr[["r"]]),
                             theoretical_stats = quote(dfr[["theoretical_stats"]]))
          )
        dfr <- eval(as.call(new_call))
      }
      if(isTRUE(i %in% keep)) dfrs[[paste0("dfr_", i)]] <- dfr
      if(is(dfr[[length(dfr)]], "error")){
        warning("Error in repetition ", i, "; exiting loop")
        break
      }
      obs_stats[i,] <- unlist(dfr[["observed_stats"]])
      p_values[i,] <- unlist(dfr[["p"]][["value"]])
      if(!is.null(manual)) manual_output[[i]] <- manual(dfr)
    }
    
    
    output <- list(call = cl,
                   Y = Y,
                   theta = theta,
                   true_mean = true_mean,
                   true_covariance = true_covariance,
                   true_X = true_X,
                   true_data = true_data,
                   test_mean = test_mean,
                   covariance = output_covariance,
                   X = X,
                   data = data,
                   empirical_stats = obs_stats,
                   theoretical_stats = theoretical_stats,
                   p = p_values)
    
    if(!is.null(keep)) output[["dfrs"]] <- dfrs
    if(!is.null(manual)) output[["manual"]] <- manual_output
    class(output) <- "compare"
    return(output)
  }

library(distfreereg)

all.equal.distfreereg <- distfreereg:::all.equal.distfreereg
test_dfr_functions <- distfreereg:::test_dfr_functions

fcl <- function(need, covlist)
  distfreereg:::fill_covariance_list(need = need, covariance_list = covlist,
                                     matsqrt_tol = distfreereg:::default_distfreereg_tol()[["matsqrt_tol"]],
                                     solve_tol = distfreereg:::default_distfreereg_tol()[["solve_tol"]])

n <- 1e2
func <- function(X, theta) theta[1] + theta[2]*X[,1] + theta[3]*X[,2]
set.seed(20250516)
Sig_list <- lapply(seq_len(n/2), function(i) rWishart(n/2, df = 2, Sigma = diag(2))[,,i])

set.seed(20250516)
Sig <- rWishart(1, df = n, Sigma = diag(n))[,,1]
P <- fcl(need = "P", list(Sigma = Sig))[["P"]]
SqrtSigma <- fcl(need = "SqrtSigma", list(P = P))[["SqrtSigma"]]
Q <- fcl(need = "Q", list(SqrtSigma = SqrtSigma))[["Q"]]

X <- matrix(rexp(2*n, rate = 1), nrow = n)
colnames(X) <- c("a", "b")

theta <- c(2,5,1)
Y <- distfreereg:::f2ftheta(f = func, X)(theta) +
  as.vector(distfreereg:::rmvnorm(n = n, reps = 1, mean = rep(0,n), SqrtSigma = distfreereg:::matsqrt(Sig)))

set.seed(20250516)
dfr_func <- distfreereg(Y = Y, X = X, test_mean = func,
                        covariance = list(Sigma = Sig),
                        theta_init = c(1,1,1), verbose = FALSE,
                        control = list(return_on_error = FALSE),
                        stat = c("KS", "CvM", "KSmax", "KSmin"))
dfr_func
print(dfr_func, show_params = FALSE)
update(dfr_func, override = list(theta_hat = rep(1, length(dfr_func[["theta_hat"]]))))

dfr_func_mod <- dfr_func
dfr_func_mod[["p"]][["value"]] <- c(0,1,0,1)
names(dfr_func_mod[["p"]][["value"]]) <- names(dfr_func[["p"]][["value"]])
dfr_func_mod

set.seed(20250516)
dfr_func_ungrouped <- distfreereg(Y = Y, X = X, test_mean = func, group = FALSE,
                                  covariance = list(Sigma = Sig),
                                  theta_init = c(1,1,1), verbose = FALSE,
                                  control = list(return_on_error = FALSE),
                                  stat = c("KS", "CvM", "KSmax", "KSmin"))

set.seed(20250516)
dfr_func_no_X <- distfreereg(Y = Y, test_mean = function(theta) theta,
                             covariance = list(Sigma = Sig),
                             theta_init = 1, verbose = FALSE,
                             control = list(return_on_error = FALSE))

set.seed(20250516)
dfr_func_verbose <- distfreereg(Y = matrix(Y), X = rnorm(n),
                                test_mean = function(X, theta) theta*X[,1],
                                covariance = list(Sigma = Sig),
                                theta_init = 1,
                                override = list(J = dfr_func[["J"]],
                                                fitted_values = dfr_func[["fitted_values"]],
                                                theoretical_stats = dfr_func[["theoretical_stats"]]),
                                control = list(return_on_error = FALSE,
                                               theta_hat_name = "hello"),
                                stat = c("KS", "CvM", "KSmax", "KSmin"))

set.seed(20250516)
dfr_func_J_for_r <- distfreereg(Y = Y, X = X, test_mean = func,
                                covariance = list(Sigma =Sig),
                                theta_init = c(1,1,1), verbose = FALSE,
                                override = list(r = dfr_func[["mu"]][order(order(dfr_func[["res_order"]])), ]),
                                control = list(return_on_error = FALSE))

set.seed(20250516)
dfr_func_list <- distfreereg(Y = Y, X = X, test_mean = func,
                             covariance = list(Sigma = Sig_list),
                             theta_init = c(1,1,1), verbose = FALSE,
                             control = list(return_on_error = FALSE),
                             stat = c("KS", "CvM", "KSmax", "KSmin"))

set.seed(20250516)
dfr_func_vec <- distfreereg(Y = Y, X = X, test_mean = func,
                            covariance = list(Sigma = runif(n, min = 1, max = 2)),
                            theta_init = c(1,1,1), verbose = FALSE,
                            control = list(return_on_error = FALSE),
                            stat = c("KS", "CvM", "KSmax", "KSmin"))

set.seed(20250516)
dfr_func_P <- distfreereg(Y = Y, X = X, test_mean = func,
                          covariance = list(P = P),
                          theta_init = c(1,1,1), verbose = FALSE,
                          control = list(return_on_error = FALSE),
                          stat = c("KS", "CvM", "KSmax", "KSmin"))
set.seed(20250516)
dfr_func_SqrtSigma <- distfreereg(Y = Y, X = X, test_mean = func,
                                  covariance = list(SqrtSigma = SqrtSigma),
                                  theta_init = c(1,1,1), verbose = FALSE,
                                  control = list(return_on_error = FALSE),
                                  stat = c("KS", "CvM", "KSmax", "KSmin"))
set.seed(20250516)
dfr_func_Q <- distfreereg(Y = Y, X = X, test_mean = func,
                          covariance = list(Q = Q),
                          theta_init = c(1,1,1), verbose = FALSE,
                          control = list(return_on_error = FALSE),
                          stat = c("KS", "CvM", "KSmax", "KSmin"))

stopifnot(all.equal(dfr_func, dfr_func_P))
stopifnot(all.equal(dfr_func, dfr_func_SqrtSigma))
stopifnot(all.equal(dfr_func, dfr_func_Q))

newdata_function <- matrix(rexp(2*10), nrow = 10)
test_dfr_functions(dfr_func, newdata = newdata_function)

plot(dfr_func, confband_args = NULL)
plot(dfr_func, which = "ecdf", confband_args = NULL)
plot(dfr_func, confband_args = FALSE)
plot(dfr_func, confband_args = list(m = 50,
                                    batch_len = 19,
                                    N = 1e3,
                                    conf.level = 0.9,
                                    q_probs = c(0.05, 0.95)))
tryCatch(plot(dfr_func, confband_args = list(m = 50,
                                             batch_len = 19,
                                             N = 1e3,
                                             conf.level = 0.9,
                                             q_probs = c(0.5, 0.3))),
         error = function(e) warning(e))

plot(dfr_func, which = "residuals")
plot(dfr_func, which = "epsp")


cdfr_func <- asymptotics(dfr_func, reps = 5)
signif(rejection(cdfr_func, alpha = c(0.1, 0.5))[,2:3], digits = 3)

tryCatch(rejection(cdfr_func, alpha = c(0.1, "a")), error = function(e) warning(e))
tryCatch(rejection(cdfr_func, stat = "k"), error = function(e) warning(e))


predict(dfr_func, newdata = 1:3)





# Update, smart versus dumb

set.seed(20250924)
dfr_1 <- distfreereg(Y = Y, X = X, test_mean = func,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE, B = 1e2)

set.seed(20250924)
dfr_update_smart <- update(dfr_1, theta_init = c(1,2,3))
set.seed(20250924)
dfr_update_dumb <- update(dfr_1, theta_init = c(1,2,3), smart = FALSE)

stopifnot(all.equal.distfreereg(dfr_update_smart, dfr_update_dumb))


# Orderings

dfr_func_asis <- update(dfr_func, ordering = "asis", verbose = TRUE)
dfr_func_optimal <- update(dfr_func, ordering = "optimal", verbose = TRUE)
dfr_func_natural <- update(dfr_func, ordering = "natural", verbose = TRUE)
dfr_func_colname <- update(dfr_func, ordering = list("a"), verbose = TRUE)
dfr_func_res_order <- update(dfr_func,
                             override = list(res_order = dfr_func[["res_order"]],
                                             r = dfr_func[["r"]]),
                             verbose = TRUE)


# Verify that ordering single column X with simplex results in natural ordering.

dfr_one_covariate_simplex <- distfreereg(test_mean = function(X, theta) theta*X[,1],
                                         Y = Y, X = X[,1,drop = FALSE],
                                         covariance = list(Sigma = 1),
                                         ordering = "simplex", B = 10,
                                         theta_init = 1, verbose = FALSE)
dfr_one_covariate_natural <- distfreereg(test_mean = function(X, theta) theta*X[,1],
                                         Y = Y, X = X[,1,drop = FALSE],
                                         covariance = list(Sigma = 1),
                                         ordering = "natural", B = 10,
                                         theta_init = 1, verbose = FALSE)

stopifnot(identical(dfr_one_covariate_simplex[["res_order"]],
                    dfr_one_covariate_natural[["res_order"]]))




### default

set.seed(20250516)
dfr_default <- distfreereg(Y = Y, X = X, test_mean = NULL,
                           covariance = list(Sigma = Sig),
                           verbose = FALSE,
                           override = list(fitted_values = dfr_func[["fitted_values"]],
                                           J = dfr_func[["J"]]),
                           control = list(return_on_error = FALSE),
                           stat = c("KS", "CvM", "KSmax", "KSmin"))
dfr_default

dfr_default_verbose <- distfreereg(Y = as.matrix(Y), X = X[,1], test_mean = NULL,
                                   covariance = list(Sigma = Sig),
                                   override = list(fitted_values = dfr_func[["fitted_values"]],
                                                   J = dfr_func[["J"]]),
                                   control = list(return_on_error = FALSE),
                                   stat = c("KS", "CvM", "KSmax", "KSmin"))

test_dfr_functions(dfr_default, newdata = newdata_function)

stopifnot(all.equal(dfr_func, dfr_default))


# Orderings

dfr_default_asis <- update(dfr_default, ordering = "asis")
dfr_default_optimal <- update(dfr_default, ordering = "optimal")
dfr_default_natural <- update(dfr_default, ordering = "natural")
dfr_default_colname <- update(dfr_default, ordering = list("a"))





### Warnings


# Partial Outputs

dfr_func_r_error <- distfreereg(Y = Y, X = X, test_mean = func,
                                covariance = list(Sigma = Sig),
                                theta_init = c(1,1,1),
                                control = list(orth_tol = 1e-100))
names(dfr_func_r_error)




# fitted_values

set.seed(20250603)
func_fv <- function(X, theta) theta[1]*X
Sig_fv <- runif(n, min = 1, max = 3)
theta_fv <- 2
X_fv <- matrix(runif(n, min = 1, max = 5), nrow = n)
Y_fv <- theta_fv[1]*X_fv[,1] + rnorm(n, sd = sqrt(Sig_fv))
dfr_fv <- distfreereg(Y = Y_fv, X = X_fv, test_mean = func_fv,
                      covariance = list(Sigma = Sig_fv),
                      theta_init = 1, B = 1e3)






### Failures

tryCatch(distfreereg(Y = Y, X = X, test_mean = NULL, group = FALSE,
                     covariance = list(Sigma = matrix(1:10, nrow = 5)), verbose = FALSE,
                     override = list(J = dfr_func[["J"]],
                                     fitted_values = dfr_func[["fitted_values"]]),
                     control = list(return_on_error = FALSE)),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X, test_mean = NULL, group = FALSE,
                     covariance = list(Sigma = 1:10), verbose = FALSE,
                     override = list(J = dfr_func[["J"]],
                                     fitted_values = dfr_func[["fitted_values"]]),
                     control = list(return_on_error = FALSE)),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X, test_mean = NULL, group = FALSE,
                     covariance = list(Sigma = 1:10), verbose = FALSE,
                     override = list(J = dfr_func[["J"]],
                                     fitted_values = NA_real_),
                     control = list(return_on_error = FALSE)),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X, test_mean = NULL, group = FALSE,
                     covariance = list(Sigma = 1:10), verbose = FALSE,
                     override = list(J = dfr_func[["J"]],
                                     fitted_values = numeric()),
                     control = list(return_on_error = FALSE)),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X, test_mean = NULL, group = FALSE,
                     covariance = list(Sigma = 1:10), verbose = FALSE,
                     override = list(J = dfr_func[["J"]],
                                     fitted_values = c(1, NaN)),
                     control = list(return_on_error = FALSE)),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X, test_mean = NULL, group = FALSE,
                     covariance = list(Sigma = 1:10), verbose = FALSE,
                     override = list(J = dfr_func[["J"]],
                                     fitted_values = c(1, Inf)),
                     control = list(return_on_error = FALSE)),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X, test_mean = NULL, group = FALSE,
                     covariance = list(Sigma = 1:10), verbose = FALSE,
                     override = list(J = dfr_func[["J"]],
                                     fitted_values = c(1, 2)),
                     control = list(return_on_error = FALSE)),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X, test_mean = NULL, group = FALSE,
                     covariance = list(Sigma = 1:10), verbose = FALSE,
                     override = list(J = dfr_func[["J"]],
                                     fitted_values = dfr_func[["fitted_values"]]),
                     control = list(return_on_error = FALSE,
                                    solve_tol = -1)),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X, test_mean = function(a) a^2,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE,
                     control = list(return_on_error = FALSE),
                     stat = c("KS", "CvM", "KSmax", "KSmin")),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X, test_mean = func,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE,
                     control = list(return_on_error = FALSE),
                     stat = c("KS", "CvM", "KSmax", "KSmin"),
                     override = list(res_order = 1:4)),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X, test_mean = func,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE,
                     control = list(return_on_error = FALSE),
                     stat = c("KS", "CvM", "KSmax", "KSmin"),
                     override = list(r = 1:5)),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X, test_mean = func,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE,
                     control = list(return_on_error = FALSE),
                     stat = c("KS", "CvM", "KSmax", "KSmin"),
                     override = list(J = 1:5)),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X, test_mean = func,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE,
                     control = list(return_on_error = FALSE),
                     stat = c("KS", "CvM", "KSmax", "KSmin"),
                     override = list(J = 1:5, r = 1:7)),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X, test_mean = func,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE,
                     control = list(return_on_error = FALSE),
                     stat = c("KS", "CvM", "KSmax", "KSmin"),
                     override = list(fitted_values = 1:5)),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X, test_mean = func,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE,
                     control = list(return_on_error = FALSE),
                     stat = c("KS", "CvM", "KSmax", "KSmin"),
                     override = list(J = c(1,2,NA))),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X, test_mean = func,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE,
                     control = list(return_on_error = FALSE),
                     stat = c("KS", "CvM", "KSmax", "KSmin"),
                     override = list(J = 1:5)),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X, test_mean = function(x) x,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE,
                     control = list(return_on_error = FALSE),
                     stat = c("KS", "CvM", "KSmax", "KSmin")),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, test_mean = function(x) x,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE,
                     control = list(return_on_error = FALSE),
                     stat = c("KS", "CvM", "KSmax", "KSmin")),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X, test_mean = function(theta) theta,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE,
                     control = list(return_on_error = FALSE),
                     stat = c("KS", "CvM", "KSmax", "KSmin")),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X[1:5,], test_mean = function(theta) theta,
                     covariance = list(Sigma = Sig),
                     theta_init = 1, verbose = FALSE,
                     control = list(return_on_error = FALSE),
                     stat = c("KS", "CvM", "KSmax", "KSmin")),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = NULL, test_mean = function(theta) theta,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE,
                     control = list(return_on_error = FALSE),
                     stat = c("KS", "CvM", "KSmax", "KSmin")),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X, test_mean = func,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE,
                     control = list(return_on_error = FALSE,
                                    optimization_args = 4),
                     stat = c("KS", "CvM", "KSmax", "KSmin")),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X, test_mean = func,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE,
                     control = list(return_on_error = FALSE,
                                    optimization_fun = "hello"),
                     stat = c("KS", "CvM", "KSmax", "KSmin")),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X, test_mean = func,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE,
                     control = list(return_on_error = FALSE),
                     stat = c("KS", "CvM", "KSmax", "KSmin"),
                     ordering = c(1,2)),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X, test_mean = NULL,
                     covariance = list(Sigma = Sig),
                     verbose = FALSE,
                     override = list(J = dfr_func[["J"]], dfr = "a",
                                     fitted_values = dfr_func[["fitted_values"]]),
                     control = list(return_on_error = FALSE),
                     stat = c("KS", "CvM", "KSmax", "KSmin")),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X, test_mean = NULL,
                     covariance = list(Sigma = Sig),
                     verbose = FALSE,
                     override = list(J = dfr_func[["J"]], res_order = 1:4,
                                     fitted_values = dfr_func[["fitted_values"]]),
                     control = list(return_on_error = FALSE),
                     stat = c("KS", "CvM", "KSmax", "KSmin")),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X, test_mean = NULL,
                     covariance = list(Sigma = Sig),
                     verbose = FALSE,
                     override = list(J = dfr_func[["J"]], r = 1:5,
                                     fitted_values = dfr_func[["fitted_values"]]),
                     control = list(return_on_error = FALSE),
                     stat = c("KS", "CvM", "KSmax", "KSmin")),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X, test_mean = NULL,
                     covariance = list(Sigma = Sig),
                     verbose = FALSE,
                     override = list(J = dfr_func[["J"]], K = 1:5,
                                     fitted_values = dfr_func[["fitted_values"]]),
                     control = list(return_on_error = FALSE),
                     stat = c("KS", "CvM", "KSmax", "KSmin")),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X[1:5,], test_mean = NULL,
                     covariance = list(Sigma = Sig),
                     verbose = FALSE,
                     override = list(J = dfr_func[["J"]],
                                     fitted_values = dfr_func[["fitted_values"]]),
                     control = list(return_on_error = FALSE),
                     stat = c("KS", "CvM", "KSmax", "KSmin")),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X, test_mean = NULL,
                     covariance = list(Sigma = Sig),
                     verbose = FALSE,
                     override = list(J = dfr_func[["J"]],
                                     fitted_values = dfr_func[["fitted_values"]]),
                     control = list(return_on_error = FALSE),
                     stat = c("KS", "CvM", "KSmax", "KSmin"),
                     ordering = c(1,4)),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X, test_mean = NULL,
                     covariance = list(Sigma = Sig),
                     verbose = FALSE,
                     override = list(J = dfr_func[["J"]],
                                     fitted_values = dfr_func[["fitted_values"]]),
                     control = list(return_on_error = FALSE),
                     stat = c("K")),
         error = function(e) warning(e))

tryCatch(plot(dfr_func, confband_args = ""), error = function(e) warning(e))
tryCatch(plot(dfr_func, density_args = ""), error = function(e) warning(e))
tryCatch(plot(dfr_func, polygon_args = ""), error = function(e) warning(e))
tryCatch(plot(dfr_func, abline_args = ""), error = function(e) warning(e))
tryCatch(plot(dfr_func, text_args = ""), error = function(e) warning(e))

tryCatch(asymptotics("dfr_default", reps = 5), error = function(e) warning(e))
tryCatch(asymptotics(dfr_default, reps = 5), error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X, test_mean = NULL,
                     covariance = list(Sigma = matrix(rnorm(n^2), nrow = n)),
                     verbose = FALSE,
                     override = list(J = dfr_func[["J"]],
                                     fitted_values = dfr_func[["fitted_values"]]),
                     control = list(return_on_error = FALSE)),
         error = function(e) warning(e))

# P_complex_eigen <- kronecker(diag(n/2), matrix(c(cos(0.5), -sin(0.5), sin(0.5), cos(0.5)), nrow = 2))
# tryCatch(distfreereg(Y = Y, X = X, test_mean = NULL,
#                      covariance = list(P = P_complex_eigen),
#                      verbose = FALSE, J = dfr_func[["J"]],
#                      fitted_values = dfr_func[["fitted_values"]],
#                      control = list(return_on_error = FALSE)),
#          error = function(e) warning(e))

tryCatch(dfr_func <- distfreereg(Y = Y, X = X, test_mean = func,
                                 covariance = list(),
                                 theta_init = c(1,1,1), verbose = FALSE,
                                 control = list(return_on_error = FALSE)),
         error = function(e) warning(e)
)

tryCatch(dfr_func <- distfreereg(Y = Y, X = X, test_mean = func,
                                 covariance = list(Sigma = "hello"),
                                 theta_init = c(1,1,1), verbose = FALSE,
                                 control = list(return_on_error = FALSE)),
         error = function(e) warning(e)
)

tryCatch(dfr_func <- distfreereg(Y = Y, X = X, test_mean = func,
                                 covariance = list(Sigma = list("hello")),
                                 theta_init = c(1,1,1), verbose = FALSE,
                                 control = list(return_on_error = FALSE)),
         error = function(e) warning(e)
)

tryCatch(dfr_func <- distfreereg(Y = Y, X = X, test_mean = func,
                                 covariance = list(Sigma = rep(-1, n)),
                                 theta_init = c(1,1,1), verbose = FALSE,
                                 control = list(return_on_error = FALSE)),
         error = function(e) warning(e)
)

tryCatch(plot(dfr_func, confband_args = list(curve_args = 7)),
         error = function(e) warning(e))

tryCatch(plot(dfr_func, confband_args = list(polygon_args = 7)),
         error = function(e) warning(e))

tryCatch(plot(dfr_func, confband_args = list(w = "a")),
         error = function(e) warning(e))

tryCatch(confint(dfr_func, level = 1), error = function(e) warning(e))
tryCatch(confint(dfr_func, level = 0), error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X, test_mean = func,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE,
                     override = list(r = matrix(1:n)),
                     control = list(return_on_error = FALSE),
                     stat = c("KS", "CvM", "KSmax", "KSmin")),
         error = function(e) warning(e)
)

tryCatch(predict(dfr_func, newdata = "hello"), error = function(e) warning(e))

tryCatch(distfreereg(Y = Y[1], X = X[1,], test_mean = func,
                     covariance = list(Sigma = Sig[1,1]),
                     theta_init = c(1,1,1), verbose = FALSE,
                     control = list(return_on_error = FALSE),
                     stat = c("KS", "CvM", "KSmax", "KSmin")),
         error = function(e) warning(e)
)

tryCatch(distfreereg(Y = Y, X = X, test_mean = func,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE,
                     group = TRUE,
                     control = list(return_on_error = "hello"),
                     stat = c("KS", "CvM", "KSmax", "KSmin")),
         error = function(e) warning(e)
)

tryCatch(distfreereg(Y = Y, test_mean = function(X, theta) X * theta,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE,
                     control = list(return_on_error = FALSE),
                     stat = c("KS", "CvM", "KSmax", "KSmin")),
         error = function(e) warning(e)
)

tryCatch(distfreereg(Y = Y[1:5], X = X, test_mean = func,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE,
                     control = list(return_on_error = FALSE),
                     stat = c("KS", "CvM", "KSmax", "KSmin")),
         error = function(e) warning(e)
)

tryCatch(distfreereg(Y = Y, X = X, test_mean = func,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE,
                     control = list(return_on_error = FALSE,
                                    optimization_fun = optim,
                                    theta_init_arg = 1)),
         error = function(e) warning(e)
)

tryCatch(distfreereg(Y = Y, X = X, test_mean = func,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE,
                     control = list(return_on_error = FALSE,
                                    optimization_fun = optim,
                                    theta_init_arg = c("hello", "goodbye"))),
         error = function(e) warning(e)
)

tryCatch(distfreereg(Y = Y, X = X, test_mean = func,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE,
                     control = list(return_on_error = FALSE,
                                    optimization_fun = optim,
                                    theta_init_arg = "hello")),
         error = function(e) warning(e)
)

tryCatch(distfreereg(Y = Y, X = X, test_mean = func,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE,
                     control = list(return_on_error = FALSE,
                                    optimization_fun = optim,
                                    theta_init_arg = "par",
                                    fun_to_optimize_arg = 1)),
         error = function(e) warning(e)
)

tryCatch(distfreereg(Y = Y, X = X, test_mean = func,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE,
                     control = list(return_on_error = FALSE,
                                    optimization_fun = optim,
                                    theta_init_arg = "par",
                                    fun_to_optimize_arg = c("hello", "goodbye"))),
         error = function(e) warning(e)
)

tryCatch(distfreereg(Y = Y, X = X, test_mean = func,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE,
                     control = list(return_on_error = FALSE,
                                    optimization_fun = optim,
                                    theta_init_arg = "par",
                                    fun_to_optimize_arg = "hello")),
         error = function(e) warning(e)
)

tryCatch(distfreereg(Y = Y, X = X, test_mean = func,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE,
                     control = list(return_on_error = FALSE,
                                    optimization_fun = optim,
                                    theta_init_arg = "par",
                                    theta_hat_name = 1,
                                    fun_to_optimize_arg = "fn")),
         error = function(e) warning(e)
)

tryCatch(distfreereg(Y = Y, X = X, test_mean = func,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE,
                     control = list(return_on_error = FALSE,
                                    optimization_fun = optim,
                                    theta_init_arg = "par",
                                    theta_hat_name = c("hello", "goodbye"),
                                    fun_to_optimize_arg = "fn")),
         error = function(e) warning(e)
)

tryCatch(distfreereg(Y = Y, X = X, test_mean = func,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE,
                     control = list(return_on_error = FALSE,
                                    optimization_fun = optim,
                                    theta_init_arg = "par",
                                    theta_hat_name = "hello",
                                    fun_to_optimize_arg = "fn")),
         error = function(e) warning(e)
)

tryCatch(distfreereg(Y = Y, X = X, test_mean = func,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE,
                     control = list(return_on_error = FALSE,
                                    jacobian_args = "hello")),
         error = function(e) warning(e)
)

tryCatch(update(dfr_func, Y = 1:5, X = matrix(rnorm(10*3), nrow = 10)),
         error = function(e) warning(e)
)

tryCatch(update(dfr_func, override = list(J = dfr_func[["J"]]),
                theta_init = c(1,1)),
         error = function(e) warning(e)
)

tryCatch(distfreereg(Y = Y, X = X, test_mean = func,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE,
                     control = "hello"),
         error = function(e) warning(e)
)

tryCatch(distfreereg(Y = Y, X = X, test_mean = NULL,
                     covariance = list(Sigma = Sig),
                     verbose = FALSE,
                     override = list(J = dfr_func[["J"]][1:5,],
                                     fitted_values = dfr_func[["fitted_values"]]),
                     control = list(return_on_error = FALSE),
                     stat = c("KS", "CvM", "KSmax", "KSmin")),
         error = function(e) warning(e)
)

bad_stat <- function(x) x

tryCatch(distfreereg(Y = Y, X = X, test_mean = NULL,
                     covariance = list(Sigma = Sig),
                     verbose = FALSE,
                     override = list(J = dfr_func[["J"]],
                                     fitted_values = dfr_func[["fitted_values"]]),
                     control = list(return_on_error = FALSE),
                     stat = c("bad_stat")),
         error = function(e) warning(e)
)

tryCatch(distfreereg(Y = Y, X = X, test_mean = func,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1,1), verbose = FALSE,
                     control = list(return_on_error = FALSE)),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = matrix(Y), X = rep(2, n),
                     test_mean = function(X, theta) theta*X[,1],
                     covariance = list(Sigma = Sig),
                     theta_init = 1,
                     control = list(return_on_error = FALSE)),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X, test_mean = func,
                     data = data.frame(),
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE,
                     control = list(return_on_error = FALSE)),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X, test_mean = func,
                     method = "nls",
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1,1), verbose = FALSE,
                     control = list(return_on_error = FALSE)),
         error = function(e) warning(e))

tryCatch(distfreereg(Y = Y, X = X, test_mean = func,
                     covariance = list(Sigma = Sig),
                     override = list(J = matrix(1, nrow = n+1, ncol = 3)),
                     theta_init = c(1,1,1), verbose = FALSE,
                     control = list(return_on_error = FALSE)),
         error = function(e) warning(e))

calc_epsp <-
  function(transformed_residuals, res_order, grouping_matrix){
    stopifnot(is.numeric(transformed_residuals), is.vector(transformed_residuals),
              is.numeric(res_order), is.vector(res_order),
              identical(length(transformed_residuals), length(res_order)),
              is.null(grouping_matrix) || is.matrix(grouping_matrix))
    if(is.null(grouping_matrix)){
      output <- cumsum(transformed_residuals[res_order])/sqrt(length(transformed_residuals))
    } else {
      output <- cumsum(as.vector(transformed_residuals %*% grouping_matrix))/sqrt(ncol(grouping_matrix))
    }
    return(output)
  }

# Verify that Y is numeric and has at least two entries.
validate_Y <- function(Y, verbose){
  if(!is.vector(Y)){
    if(isTRUE(verbose)) message("Coercing Y to vector...")
    Y <- tryCatch(as.vector(Y), error = function(e) stop("Error converting Y to a vector: ", e))
  }
  validate_numeric(x = Y, min_len = 2, message = "Y failed numeric validation: ")
  return(Y)
}

matinv <-
  function(mat, tol){
    # This function should be able to take a single number as input and output
    # the multiplicative inverse.
    stopifnot(is.numeric(mat), is.numeric(tol), tol > 0)
    if(is.matrix(mat)){
      output <-tryCatch(solve(mat, tol = tol),
                        error = function(e) stop("Unable to invert ",
                                                 deparse1(substitute(mat)), ": ", e))
    } else {
      stopifnot(all(mat != 0))
      output <- 1/mat
      stopifnot(all(is.finite(output)))
    }
    return(output)
  }

asymptotics <- function(object, ...){
  if(!is(object, "distfreereg")) stop("'object' must be an object of class 'distfreereg'")
  test_mean <- object[["test_mean"]]
  if(is.null(test_mean)) stop("'object' must have a non-NULL 'test_mean' value")
  distfreereg_args <- as.list(getCall(object))[-1]
  distfreereg_args[["Y"]] <- NULL
  compare_args <- list(true_mean = test_mean)
  if(is.function(test_mean)) compare_args[["theta"]] <- object[["theta_hat"]]
  if(is.function(test_mean) || is(test_mean, "nls") || is(object[["model"]], "nls"))
    compare_args[["true_covariance"]] <- object[["covariance"]]
  true_method <- get_method(object)
  if(!is.null(true_method)){
    compare_args[["true_method"]] <- true_method
    compare_args[["true_method_args"]] <- distfreereg_args[["method_args"]]
  }
  if(!is.null(distfreereg_args[["X"]]))
    compare_args[["true_X"]] <- object[["data"]][["X"]]
  if(!is.null(distfreereg_args[["data"]]))
    compare_args[["true_data"]] <- object[["data"]][["data"]]
  compare_args <- c(distfreereg_args, compare_args, list(...))
  cdfr <- do.call(compare, args = compare_args)
  return(cdfr)
}

# This function rather clumsily extracts the covariate data from an nls object.
get_data.nls <- function(m, ...){
  elements <- as.list(m[["m"]][["getEnv"]]())
  elements[["(weights)"]] <- NULL
  elements <- elements[unlist(lapply(elements, function(x) if(length(x) == 1) FALSE else TRUE))]
  return(as.data.frame(elements))
}

library(distfreereg)

all.equal.distfreereg <- distfreereg:::all.equal.distfreereg
test_dfr_functions <- distfreereg:::test_dfr_functions

n <- 1e2
func <- function(X, theta) theta[1] + theta[2]*X[,1] + theta[3]*X[,2]
set.seed(20250516)
Sig <- rWishart(1, df = n, Sigma = diag(n))[,,1]
X <- matrix(rexp(2*n, rate = 1), nrow = n)
theta <- c(2,5,1)
Y <- distfreereg:::f2ftheta(f = func, X)(theta) +
  as.vector(distfreereg:::rmvnorm(n = n, reps = 1, mean = rep(0,n), SqrtSigma = distfreereg:::matsqrt(Sig)))

df_nls <- as.data.frame(cbind(Y, X, rep(1:10, 10)))
names(df_nls) <- c("z", "x", "y", "g")
wt <- rexp(n) + 1
form_nls <- z ~ d + e*x + f*y

set.seed(20250516)
dfr_form_nls <- distfreereg(test_mean = form_nls, data = df_nls,
                            method_args = list(weights = wt),
                            method = "nls", verbose = FALSE,
                            control = list(return_on_error = FALSE))

dfr_form_nls


set.seed(20250516)
dfr_form_nls_no_weights <- distfreereg(test_mean = form_nls, data = df_nls,
                                       method = "nls",
                                       control = list(return_on_error = FALSE))

newdata_nls <- data.frame(a = rnorm(10), b = rnorm(10))
test_dfr_functions(dfr_form_nls, newdata = newdata_nls)

m_nls <- nls(form_nls, data = df_nls, weights = wt)
m_nls_na_omit <- update(m_nls, na.action = na.omit)

set.seed(20250516)
dfr_nls <- distfreereg(test_mean = m_nls, verbose = FALSE,
                       control = list(return_on_error = FALSE))

dfr_nls

test_dfr_functions(dfr_nls, newdata = newdata_nls)

stopifnot(all.equal(dfr_nls, dfr_form_nls))

set.seed(20250516)
dfr_nls_override <- distfreereg(test_mean = m_nls_na_omit,
                                override = list(J = dfr_nls[["J"]],
                                                fitted_values = dfr_nls[["fitted_values"]]),
                                control = list(return_on_error = FALSE))


cdfr_form_nls <- asymptotics(dfr_form_nls, reps = 5)
cdfr_nls <- asymptotics(dfr_nls, reps = 5)

signif(rejection(cdfr_form_nls, alpha = c(0.1, 0.5))[,2:3], digits = 3)
signif(rejection(cdfr_nls, alpha = c(0.1, 0.5))[,2:3], digits = 3)




# Orderings

set.seed(20250516)
dfr_nls_asis <- update(dfr_nls, ordering = "asis")
set.seed(20250516)
dfr_form_nls_asis <- update(dfr_form_nls, ordering = "asis")
stopifnot(all.equal(dfr_nls_asis, dfr_form_nls_asis))

set.seed(20250516)
dfr_nls_optimal <- update(dfr_nls, ordering = "optimal")
set.seed(20250516)
dfr_form_nls_optimal <- update(dfr_form_nls, ordering = "optimal")
stopifnot(all.equal(dfr_nls_optimal, dfr_form_nls_optimal))

set.seed(20250516)
dfr_nls_natural <- update(dfr_nls, ordering = "natural")
set.seed(20250516)
dfr_form_nls_natural <- update(dfr_form_nls, ordering = "natural")
stopifnot(all.equal(dfr_nls_natural, dfr_form_nls_natural))

set.seed(20250516)
form_nls_g <- z ~ d + e*x + f*y + I(0*is.numeric(g))
m_nls_g <- nls(form_nls_g, data = df_nls, weights = wt)
dfr_nls_g <- update(dfr_nls, ordering = list("g"), test_mean = m_nls_g)
set.seed(20250516)
dfr_form_nls_g <- update(dfr_form_nls, ordering = list("g"), test_mean = form_nls_g)
stopifnot(all.equal(dfr_nls_g, dfr_form_nls_g))

df_nls[dfr_nls_g[["res_order"]],][["g"]]

set.seed(20250516)
dfr_nls_g_grouped <- update(dfr_nls_g, group = TRUE)
set.seed(20250516)
dfr_form_nls_g_grouped <- update(dfr_form_nls_g, group = TRUE)
stopifnot(all.equal(dfr_nls_g_grouped, dfr_form_nls_g_grouped))





### Partial output

dfr_nls_partial <- distfreereg(test_mean = m_nls, verbose = FALSE,
                               control = list(orth_tol = 1e-100))
names(dfr_nls_partial)



### Failures

tryCatch(distfreereg(test_mean = form_nls, data = df_nls,
                     theta_init = 1:3,
                     method_args = list(weights = wt),
                     method = "nls", verbose = FALSE,
                     control = list(return_on_error = FALSE)),
         error = function(e) warning(e)
)

tryCatch(distfreereg(test_mean = form_nls, data = as.matrix(df_nls),
                     method_args = list(weights = wt),
                     method = "nls", verbose = FALSE,
                     control = list(return_on_error = FALSE)),
         error = function(e) warning(e)
)

tryCatch(distfreereg(test_mean = form_nls, data = data.frame(),
                     method_args = list(weights = wt),
                     method = "nls", verbose = FALSE,
                     control = list(return_on_error = FALSE)),
         error = function(e) warning(e)
)

tryCatch(distfreereg(test_mean = form_nls, data = df_nls,
                     method_args = "weights = wt",
                     method = "nls", verbose = FALSE,
                     control = list(return_on_error = FALSE)),
         error = function(e) warning(e)
)

tryCatch(distfreereg(test_mean = form_nls, data = df_nls,
                     method_args = list(weights = wt),
                     method = "nls", verbose = FALSE,
                     control = list(return_on_error = FALSE),
                     override = list(1:3)),
         error = function(e) warning(e)
)

tryCatch(distfreereg(test_mean = form_nls, data = df_nls,
                     method_args = list(weights = wt),
                     method = "nls", verbose = FALSE,
                     control = list(return_on_error = FALSE),
                     override = list(a = 1:3)),
         error = function(e) warning(e)
)

tryCatch(distfreereg(test_mean = m_nls, control = "hello"),
         error = function(e) warning(e)
)

set.seed(20250522)

fcl <- function(need, covlist)
  distfreereg:::fill_covariance_list(need = need, covariance_list = covlist,
                                     matsqrt_tol = distfreereg:::default_distfreereg_tol()[["matsqrt_tol"]],
                                     solve_tol = distfreereg:::default_distfreereg_tol()[["solve_tol"]])

n <- 1e2
Sig <- rWishart(1, df = n, Sigma = diag(n))[,,1]
P <- fcl(need = "P", list(Sigma = Sig))[["P"]]
SqrtSigma <- fcl(need = "SqrtSigma", list(P = P))[["SqrtSigma"]]
Q <- fcl(need = "Q", list(SqrtSigma = SqrtSigma))[["Q"]]
Sig_2 <- fcl(need = "Sigma", list(SqrtSigma = SqrtSigma))[["Sigma"]]
Sig_3 <- fcl(need = "Sigma", list(Q = Q))[["Sigma"]]
Sig_4 <- fcl(need = "Sigma", list(P = P))[["Sigma"]]
P_2 <- fcl(need = "P", list(Q = Q))[["P"]]
P_3 <- fcl(need = "P", list(SqrtSigma = SqrtSigma))[["P"]]
SqrtSigma_2 <- fcl(need = "SqrtSigma", list(Q = Q))[["SqrtSigma"]]



tryCatch(distfreereg:::model2method("hello"), error = function(e) warning(e))

tryCatch(distfreereg::rejection("hello"), error = function(e) warning(e))

mats <- rWishart(n = 2, df = 5, Sigma = diag(5))
mat_list <- list(mats[,,1], mats[,,2])
mat_list_sq <- distfreereg:::covsquare(mat_list)
stopifnot(isTRUE(all.equal(mat_list_sq, lapply(mat_list, crossprod))))

vec <- rexp(5)
vec_sq <- distfreereg:::matsquare(vec)
stopifnot(isTRUE(all.equal(vec_sq, vec * vec)))


err_dist_args <- distfreereg:::create_err_dist_args_default(
  err_dist_fun_formals = c("Sigma", "SqrtSigma", "P", "Q"),
  cov_args = c("Sigma", "SqrtSigma", "P", "Q"),
  reps = 10, n = 1e2, true_covariance = list(Sigma = Sig, P = P, Q = Q,
                                             SqrtSigma = SqrtSigma))
names(err_dist_args)


distfreereg:::matsqrt(matrix(4))

tryCatch(distfreereg:::covprod(cov = array(4), mat = matrix(1)),
         error = function(e) warning(e))

tryCatch(distfreereg:::scale_to_01(matrix(rep(1, 5)), one_value = "fail"),
         error = function(e) warning(e))

get_n <- function(n = NULL, ...){
  all_n <- c(n = n, lapply(list(...), get_n_single))
  all_n <- unlist(all_n[!vapply(all_n, is.null, TRUE)])
  unique_n <- unique(all_n)
  stopifnot("get_n(): No covariates supplied" = length(unique_n) > 0)
  if(length(unique_n) > 1){
    stop("All sets of covariates must have the same number of observations:\n- ",
         paste(names(all_n), all_n, collapse = "\n- ", sep = ": "))
  }
  return(unique_n)
}

get_cov.nls <- function(m, covariance, verbose){
  w <- weights(m)
  if(isTRUE(verbose)) message("Extracting covariance structure from nls object...")
  if(is.null(w)){
    covariance <- list(SqrtSigma = sigma(m))
  } else {
    covariance <- list(SqrtSigma = sigma(m)/sqrt(w))
  }
  return(covariance)
}

get_fitted_values.function <- function(test_mean, ..., ftheta, theta_hat){
  fitted_values <- ftheta(theta_hat)
  if(!is.vector(fitted_values)){
    warning("fitted_values being coerced to a vector...")
    fitted_values <- as.vector(fitted_values)
  }
  return(fitted_values)
}

determine_order <-
  function(X, ordering, n, verbose){
    if(is.null(X)){
      output <- seq_len(n)
    } else{
      if(identical(ordering, "asis")){
        if(isTRUE(verbose)) message("Leaving observation order as is...")
        output <- 1:nrow(X)
      } else {
        # If 'ordering' is a list, remove unused columns
        if(is.list(ordering)) X <- X[,unlist(ordering), drop = FALSE]
        # If 'ordering' is 'natural', overwrite column names now.
        if(identical(ordering, "natural")) colnames(X) <- LETTERS[seq_len(ncol(X))]
        # Create a data frame with an initial indexing column.
        df <- cbind(data.frame(seq_len(nrow(X))), X)
        # Now remove duplicate rows from X.
        X <- X[!duplicated(X),, drop = FALSE]
        if(identical(ordering, "optimal")){
          stopifnot(is.matrix(X))
          if(isTRUE(verbose)) message("Ordering observations using optimal transport...")
          m <- nrow(X)
          mbyn <- length(X)
          stopifnot(mbyn > 0)
          reference <- matrix((1:mbyn)/mbyn, nrow = m, byrow = TRUE)
          if(isTRUE(verbose)) message("- Creating distance matrix...")
          dist_mat <- matrix(NA, nrow = m, ncol = m)
          for(i in 1:m){
            for(j in 1:m){
              dist_mat[i,j] <- sqrt(sum((reference[i,] - X[j,])^2))
            }
          }
          if(isTRUE(verbose)) message("- Solving linear sum assignment problem using the Hungarian method...")
          output <- tryCatch(as.vector(clue::solve_LSAP(dist_mat)),
                             error = function(e) stop("Hungarian method failure: ", e))
          stopifnot(isPermutation(output))
        } else {
          if(identical(ordering, "simplex")){
            stopifnot(is.matrix(X))
            if(isTRUE(verbose)) message("Ordering observations by simplex method...")
            X_scaled <- scale_to_01(X)
            output <- order(rowSums(X_scaled))
          } else {
            # By this point, the only option left should be the natural ordering
            # or an ordering specified by columns; both are handled by
            # natural_order().
            if(identical(ordering, "natural")){
              if(isTRUE(verbose)) message("Ordering observations by natural order...")
              ordering <- colnames(X)
            } else {
              stopifnot(is.list(ordering))
              if(isTRUE(verbose)) message("Ordering observations by specified columns...")
              # Unlist, since if ordering specifies columns, it must be a list.
              # ordering <- unlist(ordering)
            }
            output <- natural_order(data = X, cols = ordering)
          }
        }
        ordered <- as.data.frame(cbind(X, data.frame(ordered = order(output))))
        df_merged <- merge(x = df, y = ordered, sort = FALSE)
        output <- df_merged$ordered[order(df_merged[["seq_len.nrow.X.."]])]
      }
    }
    # if(exists("df_merged"))
    #   print(list(output = output, X = X, df = df,
    #              df_merged = df_merged, ordered = ordered))
    return(output)
    # return(df_merged)
  }

# This function takes a character matrix and "buffers" each entry with blank
# spaces so each column's entries have the same length, equal to the length of
# the longest string in that column.
buffer_matrix <- function(mat){
  stopifnot(is.matrix(mat), is.character(mat))
  min_width <- apply(mat, MARGIN = 2, FUN = function(x) max(nchar(x)))
  output <- vector(mode = "list", length = 4)
  for(i in seq_len(ncol(mat))){
      mat[,i] <- sprintf(paste0("%", min_width[i], "s"), mat[,i])
  }
  return(mat)
}

get_n_single <- function(x){
  if(!is.null(x)){
    if(is.numeric(x)){
      output <- nrow(as.matrix(x))
    } else {
      if(is.data.frame(x)){
        output <- nrow(x)
      } else {
        if(isModel(x)){
          output <- length(fitted(x))
        } else {
          if(is.function(x) || is(x, "formula")){
            output <- NULL
          } else {
            stop("Invalid input to get_n_single(): ", class(x))
          }
        }
      }
    }
  } else {
    output <- NULL
  }
  return(output)
}

get_cov.glm <- function(m, covariance, verbose){
  if(isTRUE(verbose)) message("Extracting covariance structure from glm object...")
  covariance <- list(Sigma = m[["weights"]])
  return(covariance)
}

tryReturn <- function(expr, output, return_on_error, env = parent.frame()){
  if(isTRUE(return_on_error)){
    tryCatch(expr, error = function(e) error2return(e = e, output = output, env = env))
  } else {
    expr
  }
}

# This function takes a model as input and outputs the appropriate "method" used
# to create it, in the sense of the "method" argument of distfreereg(). It is
# used by asymptotics() to extract the correct value for true_method from the
# distfreereg object.
get_method <- function(object){
  stopifnot(is(object, "distfreereg"))
  if(isTRUE(is(object[["test_mean"]], "formula") & !is.null(object[["model"]]))){
    output <- model2method(object[["model"]])
  } else {
    output <- NULL
  }
  return(output)
}

confband_radii <- function(N, Sigma_hat, df, conf.level, n, matsqrt_tol){
  rho_hat <- cov2cor(Sigma_hat)
  t <- rmvt(n = N,
            SqrtSigma = matsqrt(rho_hat, tol = matsqrt_tol,
                                symmetric = TRUE),
            df = df)
  stopifnot(identical(ncol(t), as.integer(N)))
  t_max <- apply(t, MARGIN = 2, FUN = function(x) max(abs(x)))
  t_alpha_rho_hat <- quantile(t_max, probs = conf.level)
  radii <- t_alpha_rho_hat*sqrt(diag(Sigma_hat)/n)
  return(radii)
}

validate_weights.glm <- function(m, method_args = NULL){
  if((is.null(method_args) && !is.null(getCall(m)[["weights"]])) || !is.null(method_args[["weights"]]))
    stop("'weights' argument for glm() not supported")
}

# This function defines a matrix that is used to add transformed residuals.
# Grouping is determined by using res_order.
define_grouping_matrix <- function(X, res_order){
  if(length(unique(res_order)) == 1)
    stop("Grouping is unavailable when only one unique combination of covariate values is supplied.")
  mm <- unname(model.matrix(~ as.factor(res_order) - 1))
  matrix_scales <- sqrt(colSums(mm))
  grouping_matrix <- t(t(mm)/matrix_scales)
  
  return(grouping_matrix)
}

# Verify that X is numeric, and convert to a matrix in case it is just a
# vector.
validate_X <- function(X, verbose){
  if(!is.null(X)){
    if(!is.matrix(X)){
      if(isTRUE(verbose)) message("Coercing X to matrix...")
      X <- tryCatch(as.matrix(X), error = function(e) stop("Error converting X to a matrix: ", e))
    }
    validate_numeric(x = X, message = "X failed numeric validation: ")
    column_value_counts <- apply(X, MARGIN = 2, function(x) length(unique(x)))
    if(!identical(colnames(X)[1], "(Intercept)") && isTRUE(any(column_value_counts == 1)))
      warning("At least one column of X contains only one value. Verify that the model is identifiable.")
    return(X)
  }
}

confband_Sigma_hat <- function(x, fnw, func, w, a, b, ...){
  n <- length(x)
  fkw <- sapply(seq_len(a), function(k)
      func(x[seq(from = (k - 1)*b + 1, to = k*b)], ...)(w))
  Sigma_hat <- tcrossprod(fkw - fnw)*b/(a - 1)
  return(Sigma_hat)
}

library(distfreereg)

### Grouping

set.seed(20250610)
# Define a data frame with repeated covariate values.
df <- as.data.frame(rbind(
  c(1,3),# 1:  1, 1
  c(2,3),# 2:  3, 2
  c(2,4),# 3:  4, 2
  c(1,4),# 4:  2, 1
  c(1,4),# 5:  2, 1
  c(1,4),# 6:  2, 1
  c(1,3),# 7:  1, 1
  c(2,4),# 8:  4, 2
  c(2,4),# 9:  4, 2
  c(2,3) # 10: 3, 2
))

n <- nrow(df)
df <- cbind(data.frame(Y = rnorm(n)), df)
names(df) <- c("y", "x1", "x2")

form <- y ~ x1 + x2
m <- lm(formula = form, data = df)

dfr_form_lm_natural <- distfreereg(test_mean = m, ordering = "natural",
                                   group = TRUE, B = 10)

dfr_form_lm_natural
stopifnot(identical(dfr_form_lm_natural[["res_order"]],
                    as.integer(c(1, 3, 4, 2, 2, 2, 1, 4, 4, 3))))

# Verify that epsp from grouped dfr is correctly obtained from transformed residuals
groupit_natural_1 <- ((sum(dfr_form_lm_natural$residuals$transformed[c(1,7)])/sqrt(2))/sqrt(4))
stopifnot(all.equal(groupit_natural_1, dfr_form_lm_natural$epsp[1]))

groupit_natural_2 <- groupit_natural_1 + ((sum(dfr_form_lm_natural$residuals$transformed[4:6])/sqrt(3))/sqrt(4))
stopifnot(all.equal(groupit_natural_2, dfr_form_lm_natural$epsp[2]))

groupit_natural_3 <- groupit_natural_2 + ((sum(dfr_form_lm_natural$residuals$transformed[c(2,10)])/sqrt(2))/sqrt(4))
stopifnot(all.equal(groupit_natural_3, dfr_form_lm_natural$epsp[3]))

groupit_natural_4 <- groupit_natural_3 + ((sum(dfr_form_lm_natural$residuals$transformed[c(3,8,9)])/sqrt(3))/sqrt(4))
stopifnot(all.equal(groupit_natural_4, dfr_form_lm_natural$epsp[4]))


# Now test it when grouping by a subset of columns.

dfr_form_lm_x1 <- distfreereg(test_mean = m, ordering = list("x1"),
                              group = TRUE, B = 10)

dfr_form_lm_x1
stopifnot(identical(dfr_form_lm_x1[["res_order"]],
                    as.integer(c(1, 2, 2, 1, 1, 1, 1, 2, 2, 2))))

groupit_x1_1 <- ((sum(dfr_form_lm_x1$residuals$transformed[c(1,4,5,6,7)])/sqrt(5))/sqrt(2))
stopifnot(all.equal(groupit_x1_1, dfr_form_lm_x1$epsp[1]))

groupit_x1_2 <- groupit_x1_1 + ((sum(dfr_form_lm_x1$residuals$transformed[c(2,3,8,9,10)])/sqrt(5))/sqrt(2))
stopifnot(all.equal(groupit_x1_2, dfr_form_lm_x1$epsp[2]))




### Failures

tryCatch(distfreereg(test_mean = "hello"),
         error = function(e) warning(e))

ks.test.compare <-
  function(x, ..., stat = NULL){
    stopifnot(!is.null(names(x[["empirical_stats"]])))
    if(is.null(stat)) stat <- names(x[["empirical_stats"]])[[1]]
    strict_match(stat, choices = names(x[["empirical_stats"]]))
    stopifnot(!is.null(x[["empirical_stats"]][[stat]]), !is.null(x[["theoretical_stats"]][[stat]]))
    output <- ks.test(x[["empirical_stats"]][[stat]], x[["theoretical_stats"]][[stat]], ...)
    stat_text <- deparse1(substitute(stat))
    stat_text <- gsub("\"", replacement = "", x = stat_text)
    stat_text <- gsub("\'", replacement = "", x = stat_text)
    output[["data.name"]] <- paste0(deparse1(substitute(x)), "$empirical_stats$",
                                    stat_text, " and ",
                                    deparse1(substitute(x)), "$theoretical_stats$",
                                    stat_text)
    return(output)
  }

plot.distfreereg <-
    function(x, which = "dens", stat = NULL, density_args = NULL,
             polygon_args = NULL, confband_args = NULL, abline_args = NULL,
             shade_col = rgb(1,0,0,0.5), text_args = NULL, ...){
        strict_match(arg = which, choices = c("dens", "ecdf",
                                              "residuals", "epsp"))
        if(which %in% c("dens", "ecdf")){
            stopifnot(!is.null(names(x[["observed_stats"]])))
            if(is.null(stat)) stat <- names(x[["observed_stats"]])[1]
            validate_args_plot.distfreereg(object = x, stat = stat,
                                           density_args = density_args,
                                           polygon_args = polygon_args,
                                           confband_args = confband_args,
                                           abline_args = abline_args,
                                           text_args = text_args)
            stopifnot(!is.null(x[["theoretical_stats"]][[stat]]),
                      !is.null(x[["observed_stats"]][[stat]]))
            default_x <- x[["theoretical_stats"]][[stat]]
            if(identical(which, "dens")){
                default_density_args <- list(x = default_x, n = 2^11)
                density_args <- combine_lists(density_args,
                                              default_density_args)
                curve_data <- do.call(density, args = density_args)
            } else {
                curve_data <- list(x = sort(default_x),
                                   y = seq_along(default_x)/length(default_x))
            }
            # If plotting confidence band, then create band here to get range
            # for vertical axis.
            if(!isFALSE(confband_args)){
                confband_args <- combine_lists(confband_args,
                                               list(density_args =
                                                        density_args))
                cb <- create_confband(x = default_x, func = which,
                                      confband_args = confband_args)
                if(identical(which, "dens")){
                    y_max <- max(curve_data[["y"]], cb[["cb_upper"]])
                } else {
                    y_max <- 1
                }
            }

            plot_args <- list()

            if(!isFALSE(confband_args)) plot_args[["ylim"]] <- c(0, y_max)

            if(identical(which, "ecdf")){
                plot_args[["ylab"]] <- bquote(expression(P(.(stat)<=s)))
                plot_args[["main"]] <- paste("Empirical", stat,
                                                "Cumulative Distribution")
            } else {
                plot_args[["main"]] <- paste("Estimated", stat,
                                                "Statistic Density")
            }
            
            plot_args <-
                combine_lists(list(...), plot_args,
                              list(x = curve_data,
                                   xlab = stat, type = "l"))

            do.call(plot, plot_args)
            if(!isFALSE(confband_args)){
                plot_confband(cb = cb, confband_args = confband_args)
            }
            x_vals <- curve_data[["x"]]
            y_vals <- curve_data[["y"]]
            s <- x[["observed_stats"]][[stat]]
            if(!isFALSE(polygon_args) && identical(which, "dens")){
                default_polygon_args <- list(x = c(x_vals[x_vals >= s],
                                                   max(x_vals), s),
                                             y = c(y_vals[x_vals >= s], 0, 0),
                                             col = shade_col, border = NA)
                polygon_args <- combine_lists(polygon_args,
                                              default_polygon_args)
                do.call(polygon, args = polygon_args)
            }
            if(!isFALSE(abline_args)){
                default_line_args <- list(v = s)
                line_args <- combine_lists(abline_args, default_line_args)
                do.call(abline, args = line_args)
            }
            if(!isFALSE(text_args)){
                default_text_args <-
                    list(x = s, y = max(y_vals)/2, adj = c(1,0.5),
                         labels = paste(" p =", x[["p"]][["value"]][[stat]],
                                        " "))
                text_args <- combine_lists(text_args, default_text_args)
                do.call(text, args = text_args)
            }
            output <- list(x = x_vals, y = y_vals)
            if(!isFALSE(confband_args)){
                output[["confband"]] <- cb
            }
            invisible(output)
        } else {
            n <- length(x[["data"]][["Y"]])
            default_args <- list(type = "p", col = "blue", ylab = "Value",
                                 xlab = "Observation (Ordered)")
            if(identical(which, "residuals")){
                default_args[["x"]] <- seq_len(n)
                default_args[["y"]] <-
                    residuals(x, type = "transformed")[x[["res_order"]]]
                default_args[["main"]] <- "Ordered Transformed Residuals"
                plot_args <- combine_lists(list(...), default_args)
                do.call(plot, plot_args)
            } else {
                if(identical(which, "epsp")){
                    default_args[["type"]] <- "l"
                    default_args[["x"]] <- seq_along(x[["epsp"]])
                    default_args[["y"]] <- x[["epsp"]]
                    default_args[["main"]] <- "Empirical Partial Sum Process"
                    plot_args <- combine_lists(list(...), default_args)
                    do.call(plot, plot_args)
                }
            }
        }
    }

covprod <- function(cov, mat){
  mat <- as.matrix(mat)
  if(is.list(cov)){
    stopifnot(identical(sum(sapply(cov, NROW)), NROW(mat)))
    idx <- rep(seq_along(cov), times = sapply(cov, nrow))
    block_prods <- lapply(seq_along(cov),
                          function(i) cov[[i]] %*% as.matrix(mat)[which(idx == i), ])
    output <- do.call(rbind, block_prods)
  } else {
    if(is.matrix(cov)) {
      output <- cov %*% mat
    } else {
      if(is.vector(cov)) {
        output <- cov * mat
      } else {
        stop("invalid covariance type")
      }
    }
  }
  return(output)
}

confint.distfreereg <-
  function(object, parm, level = 0.95, ...)
  {
    stopifnot(is(object, "distfreereg"))
    if(is.null(object[["test_mean"]]))
      stop("'test_mean' element of distfreereg object must be non-NULL.")
    validate_numeric(x = level, max_val_strict = 1, min_val_strict = 0)
    test_mean <- object[["test_mean"]]
    if(isModel(test_mean) || is(test_mean, "formula")){
      if(is(test_mean, "formula")){
        ci_object <- object[["model"]]
        stopifnot(!is.null(ci_object))
      } else {
        ci_object <- object[["test_mean"]]
      }
      confint(object = ci_object, parm = parm, level = level, ...)
    } else {
      confint.default(object = object, parm = parm, level = level, ...)
    }
  }

rmvt <- function(n, SqrtSigma, df){
  stopifnot(is.numeric(df), length(df) == 1, df > 0,
            isInteger(n), n > 0, is.numeric(SqrtSigma), is.matrix(SqrtSigma))
  t(t(rmvnorm(reps = n, SqrtSigma = SqrtSigma))/sqrt(rchisq(n, df)/df))
}

validate_args_plot.compare <-
function(object, which, stat, hlines, curve_args, density_args, confband_args,
         poly, legend, qqline){
  strict_match(which, choices = c("cdf", "dens","qq", "qqp"))
  strict_match(arg = stat, choices = object)
  if(!is.null(hlines) && !isFALSE(hlines) && !is.list(hlines))
    stop("'hlines' must be NULL, FALSE, or a list")
  if(!is.null(curve_args) && !is.list(curve_args))
    stop("'curve_args' must be NULL or a list")
  if(!is.null(density_args) && !is.list(density_args))
    stop("'density_args' must be NULL or a list")
  if(!is.null(confband_args) && !isFALSE(confband_args) && !is.list(confband_args))
    stop("confband_args must be NULL, FALSE, or a list")
  if(is.list(confband_args))
    validate_named_list(confband_args,
                        valid_names = c("w", "m", "batch_len", "N", "conf.level",
                                        "q_probs", "lines_args", "polygon_args",
                                        "shade_col"))
  if(!is.null(poly) && !isFALSE(poly) && !is.list(poly))
    stop("'poly' must be NULL, FALSE, or a list")
  if(!is.null(legend) && !isFALSE(legend) && !is.list(legend))
    stop("'legend' must be NULL, FALSE, or a list")
  if(!is.null(qqline) && !isFALSE(qqline) && !is.list(qqline))
    stop("'qqline' must be NULL, FALSE, or a list")
}

error2return <- function(e, output, env){
  output <- c(output, list(e))
  warning("Error encountered, partial results returned: ", e, call. = FALSE)
  do.call("return", args = list(value = output), envir = env)
}

validate_args_plot.distfreereg <-
function(object, stat, density_args, polygon_args, confband_args, abline_args, text_args){
  strict_match(arg = stat, choices = object)
  if(!is.null(density_args) && !is.list(density_args)) stop("density_args must be a list")
  if(!is.null(polygon_args) && !isFALSE(polygon_args) && !is.list(polygon_args))
    stop("polygon_args must be NULL, FALSE, or a list")
  if(!is.null(confband_args) && !isFALSE(confband_args) && !is.list(confband_args))
    stop("confband_args must be NULL, FALSE, or a list")
  if(is.list(confband_args)){
    validate_named_list(confband_args,
                        valid_names = c("w", "m", "batch_len", "N", "conf.level",
                                        "q_probs", "curve_args", "polygon_args",
                                        "shade_col"))
    if(!is.null(confband_args[["w"]])) validate_numeric(x = confband_args[["w"]])
    if(!is.null(confband_args[["m"]])) validate_numeric(x = confband_args[["m"]], len = 1, pos_int = TRUE)
    if(!is.null(confband_args[["batch_len"]])) validate_numeric(x = confband_args[["batch_len"]], len = 1, pos_int = TRUE)
    if(!is.null(confband_args[["N"]])) validate_numeric(x = confband_args[["N"]], len = 1, pos_int = TRUE)
    if(!is.null(confband_args[["conf.level"]])) validate_numeric(x = confband_args[["conf.level"]], len = 1,
                                                                 min_val_strict = 0, max_val_strict = 1)
    if(!is.null(confband_args[["q_probs"]])){
        validate_numeric(x = confband_args[["q_probs"]], len = 2,
                min_val_strict = 0, max_val_strict = 1)
        if(confband_args[["q_probs"]][1] >= confband_args[["q_probs"]][2])
            stop("q_probs elements must be specified in increasing order.")
        }
    if(!is.null(confband_args[["curve_args"]]) && !is.list(confband_args[["curve_args"]]))
      stop("confband_args[['curve_args']] must be NULL or a list")
    if(!is.null(confband_args[["polygon_args"]]) &&
       !isFALSE(confband_args[["polygon_args"]]) &&
       !is.list(confband_args[["polygon_args"]]))
      stop("confband_args[['polygon_args']] must be NULL, FALSE, or a list")
  }
  if(!is.null(abline_args) && !isFALSE(abline_args) && !is.list(abline_args))
    stop("abline_args must be NULL, FALSE, or a list")
  if(!is.null(text_args) && !isFALSE(text_args) && !is.list(text_args))
    stop("text_args must be NULL, FALSE, or a list")
}

# Intended for package testing only.
all.equal.distfreereg <- function(target, current, tolerance = 1e-3, ...){
  epsp <- all.equal(target = target[["epsp"]],
                    current = current[["epsp"]],
                    tolerance = tolerance, ...)
  observed_stat <- all.equal(target = target[["observed_stat"]],
                             current = current[["observed_stat"]],
                             tolerance = tolerance, ...)
  p <- all.equal(target = target[["p"]],
                 current = current[["p"]],
                 tolerance = tolerance, ...)
  
  return(all(isTRUE(epsp), isTRUE(observed_stat), isTRUE(p)))
}

get_cov.lmerMod <- function(m, covariance, verbose){
  if(isTRUE(verbose)) message("Extracting covariance structure from lmerMod object...")
  re <- crossprod(as.matrix(getME(m,"Lambdat")))
  Z <- as.matrix(getME(m, "Z"))
  sig <- sigma(m)^2
  Sigma <- sig*(Z %*% re %*% t(Z)) + diag(sig/weights(m, type = "prior"), nrow(Z))
  return(list(Sigma = Sigma))
}

plot.compare <-
  function(x, y, ..., which = "cdf", stat = NULL, hlines = NULL, curve_args = NULL,
           confband_args = FALSE, density_args = NULL, poly = NULL,
           legend = NULL, qqline = NULL){
    # NOTE!!! "x" and "y" values used below DO NOT refer to horizontal and
    # vertical coordinates, but rather to values specified by the "x" and "x"
    # arguments.
    if(is.null(stat)) stat <- names(x[["empirical_stats"]])[1]
    validate_args_plot.compare(object = x, which = which, stat = stat,
                               hlines = hlines, curve_args = curve_args,
                               confband_args = confband_args,
                               density_args = density_args, poly = poly,
                               legend = legend, qqline = qqline)
    stopifnot(!is.null(x[["empirical_stats"]][[stat]]))
    xvals <- x[["empirical_stats"]][[stat]]
    xname <- deparse(substitute(x))
    x_length <- length(xvals)
    n <- nrow(x[["Y"]])
    if(missing(y)){
      stopifnot(!is.null(x[["theoretical_stats"]][[stat]]))
      yvals <- x[["theoretical_stats"]][[stat]]
    } else {
      if(!inherits(y, "compare")) stop("y must have class 'compare'")
      yname <- deparse(substitute(y))
      if(identical(xname, yname)) stop("The same object cannot be supplied to 'x' and 'y'")
      stopifnot(!is.null(y[["empirical_stats"]][[stat]]))
      strict_match(arg = stat, choices = y)
      if(which == "qqp") stop("Cannot create p-value Q-Q plot with two 'compare' objects")
      if(n != nrow(y[["Y"]])) warning("Length of Y different for ",
                                             xname, " and ", yname)
      yvals <- y[["empirical_stats"]][[stat]]
      if(x_length != length(yvals)) warning("Length of empirical stats different for ",
                                     xname, " and ", yname)
    }
    y_length <- length(yvals)
    if(isTRUE(which == "cdf")){
      xvals <- sort(xvals)
      yvals <- sort(yvals)
      main <- if(missing(y)){
        paste0("Empirical and theoretical ", stat, " CDFs (n=", n, ", reps=", x_length, ")")
      } else {
        paste0("Empirical ", stat, " CDFs")
      }
      # Define default arguments for creating plot region.
      default_args <- list(x = NULL, type = "n", ylim = c(0,1),
                           xlim = c(min(xvals, yvals), max(xvals, yvals)),
                           xlab = expression(s), ylab = bquote(expression(P(.(stat)<=s))),
                           main = main)
      plot_args <- combine_lists(list(...), default_args)
      do.call(plot, args = plot_args)
      
      # Deal with horizontal lines at 0 and 1.
      if(!isFALSE(hlines)){
        default_abline_args <- list(h = 0:1, lty = 2, col = "gray")
        abline_args <- combine_lists(hlines, default_abline_args)
        do.call(abline, args = abline_args)
      }
      
      # Create argument lists for creating cdf curves.
      default_obs_lines_args <- list(x = xvals,
                                     y = (1:length(xvals))/length(xvals),
                                     lty = 1, col = "blue", type = "l", 
                                     xlim = c(min(xvals), max(xvals)))
      default_mcsim_lines_args <- list(x = yvals,
                                       y = (1:length(yvals))/length(yvals),
                                       lty = 2, col = "red", type = "l",
                                       xlim = c(min(yvals), max(yvals)))
      # First add arguments in [["obs"]] and [["mcsim"]] elements to their respective lists
      obs_lines_args <- combine_lists(curve_args[["obs"]], default_obs_lines_args)
      mcsim_lines_args <- combine_lists(curve_args[["mcsim"]], default_mcsim_lines_args)
      
      # Now remove [["obs"]] and [["mcsim"]], and add what remains to both lists.
      curve_args[["obs"]] <- curve_args[["mcsim"]] <- NULL
      obs_lines_args <- combine_lists(curve_args, obs_lines_args)
      mcsim_lines_args <- combine_lists(curve_args, mcsim_lines_args)
      
      # Before plotting cdf curves, plot confidence bands, if called for.
      if(!isFALSE(confband_args)){
        cbx <- create_confband(x = sample(xvals), func = "ecdf", confband_args = confband_args)
        cby <- create_confband(x = sample(yvals), func = "ecdf", confband_args = confband_args)
        plot_confband(cb = cbx, confband_args = confband_args)
        plot_confband(cb = cby, confband_args = confband_args)
      }
      
      do.call(lines, obs_lines_args)
      do.call(lines, mcsim_lines_args)
      
      if(!isFALSE(legend)){
        default_legend_args <- list(x = "bottomright", bg = "white",
                                    col = c(obs_lines_args[["col"]], mcsim_lines_args[["col"]]),
                                    lty = c(obs_lines_args[["lty"]], mcsim_lines_args[["lty"]]))
        default_legend_args[["legend"]] <- if(missing(y)){
          c("empirical", "theoretical")
        } else {
          c(xname, yname)
        }
        legend_args <- combine_lists(legend, default_legend_args)
        do.call(graphics::legend, args = legend_args)
      }
      output <- list(list(x = xvals, y = 1:x_length/x_length),
                     list(x = yvals, y = 1:y_length/y_length))
      if(missing(y)){
        names(output)[1:2] <- c("empirical", "theoretical")
      } else {
        names(output)[1:2] <- c(xname, yname)
      }
      if(!isFALSE(confband_args)){
        output[["confband_x1"]] <- cbx
        output[["confband_x2"]] <- cby
        if(missing(y)){
          names(output)[3:4] <- c("confband_empirical", "confband_theoretical")
        } else {
          names(output)[3:4] <- c(paste("confband", xname, sep = "_"),
                                  paste("confband", yname, sep = "_"))
        }
      }
      invisible(output)
    } else {
      if(isTRUE(which == "dens")){
        obs_density_args <- combine_lists(density_args[["obs"]], list(x = xvals))
        mcsim_density_args <- combine_lists(density_args[["mcsim"]], list(x = yvals))
        density_args[["obs"]] <- density_args[["mcsim"]] <- NULL
        obs_density_args <- combine_lists(density_args, obs_density_args)
        mcsim_density_args <- combine_lists(density_args, mcsim_density_args)
        dens_x <- do.call(density, args = obs_density_args)
        dens_y <- do.call(density, args = mcsim_density_args)
        main <- if(missing(y)){
          paste0("Empirical and theoretical ", stat, " Densities (n=", n, ", reps=", x_length, ")")
        } else {
          paste0("Empirical ", stat, " Densities")
        }
        default_args <- list(x = NULL, type = "n",
                             xlim = c(min(dens_x[["x"]], dens_y[["x"]]), max(dens_x[["x"]], dens_y[["x"]])),
                             ylim = c(0, max(dens_x[["y"]], dens_y[["y"]])),
                             xlab = "Statistic", ylab = "Density",
                             main = main)
        plot_args <- combine_lists(list(...), default_args)
        do.call(plot, args = plot_args)
        
        if(!isFALSE(poly)){
          default_obs_polygon_args <- list(x = dens_x[["x"]], y = dens_x[["y"]],
                                           col = rgb(0,0,1,0.5))
          obs_polygon_args <- combine_lists(poly[["obs"]], default_obs_polygon_args)
          default_mcsim_polygon_args <- list(x = dens_y[["x"]], y = dens_y[["y"]],
                                             col = rgb(1,0,0,0.5))
          mcsim_polygon_args <- combine_lists(poly[["mcsim"]], default_mcsim_polygon_args)
          poly[["obs"]] <- poly[["mcsim"]] <- NULL
          obs_polygon_args <- combine_lists(poly, obs_polygon_args)
          mcsim_polygon_args <- combine_lists(poly, mcsim_polygon_args)
          do.call(polygon, args = obs_polygon_args)
          do.call(polygon, args = mcsim_polygon_args)
        }
        
        obs_lines_args <- combine_lists(curve_args[["obs"]],
                                        list(x = dens_x[["x"]], y = dens_x[["y"]]))
        mcsim_lines_args <- combine_lists(curve_args[["mcsim"]],
                                          list(x = dens_y[["x"]], y = dens_y[["y"]]))
        
        # Now remove [["obs"]] and [["mcsim"]], and add what remains to both lists.
        curve_args[["obs"]] <- curve_args[["mcsim"]] <- NULL
        obs_lines_args <- combine_lists(curve_args, obs_lines_args)
        mcsim_lines_args <- combine_lists(curve_args, mcsim_lines_args)
        
        # Before plotting density curves, plot confidence bands, if called for.
        if(!isFALSE(confband_args)){
          confband_args[["density_args"]] <- density_args
          cbx1 <- create_confband(x = sample(xvals), func = "dens",
                                  confband_args = combine_lists(confband_args,
                                                                list(density_args = obs_density_args)))
          cbx2 <- create_confband(x = sample(yvals), func = "dens",
                                  confband_args = combine_lists(confband_args,
                                                                list(density_args = mcsim_density_args)))
          plot_confband(cb = cbx1, confband_args = combine_lists(confband_args,
                                                                 list(density_args = mcsim_density_args)))
          plot_confband(cb = cbx2, confband_args = combine_lists(confband_args,
                                                                 list(density_args = mcsim_density_args)))
        }
        
        do.call(lines, obs_lines_args)
        do.call(lines, mcsim_lines_args)
        
        if(!isFALSE(legend)){
          default_legend_args <- list(x = "bottomright", bg = "white",
                                      fill = c(obs_polygon_args[["col"]], mcsim_polygon_args[["col"]]))
          default_legend_args[["legend"]] <- if(missing(y)){
            c("empirical", "theoretical")
          } else {
            c(xname, yname)
          }
          legend_args <- combine_lists(legend, default_legend_args)
          do.call(graphics::legend, args = legend_args)
        }
        output <- list(list(x = dens_x[["x"]], y = dens_x[["y"]]),
                       list(x = dens_y[["x"]], y = dens_y[["y"]]))
        if(missing(y)){
          names(output)[1:2] <- c("empirical", "theoretical")
        } else {
          names(output)[1:2] <- c(xname, yname)
        }
        if(!isFALSE(confband_args)){
          if(missing(y)){
            output[["confband_empirical"]] <- cbx1
            output[["confband_theoretical"]] <- cbx2
          } else {
            output[[paste("confband", xname, sep = "_")]] <- cbx1
            output[[paste("confband", yname, sep = "_")]] <- cbx2
          }
        }
        invisible(output)
      } else {
        if(isTRUE(which == "qq")){
          if(missing(y)){
            xlab <- "Empirical Statistics"
            ylab <- "Theoretical Statistics"
          } else {
            xlab <- paste("Empirical", xname)
            ylab <- paste("Empirical", yname)
          }
          default_args <- list(x = xvals, y = yvals, xlab = xlab,
                               ylab = ylab, main = paste0("Q-Q Plot for ", stat,
                                                          " (n=", n, ", reps=", x_length, ")"))
          qqplot_args <- combine_lists(list(...), default_args)
          output <- do.call(qqplot, args = qqplot_args)
          if(!isFALSE(qqline)){
            default_abline_args <- list(a = 0, b = 1, col = "red")
            abline_args <- combine_lists(qqline, default_abline_args)
            do.call(abline, args = abline_args)
          }
          invisible(output)
        } else {
          if(isTRUE(which == "qqp")){
            default_args <- list(x = punif(ppoints(length(x[["p"]][[stat]]))),
                                 y = x[["p"]][[stat]],
                                 xlab = "Uniform Quantiles",
                                 ylab = "P-Values",
                                 main = paste0("P-Values and Uniform Quantiles for ",
                                               stat, " (n=", n, ", reps=", x_length, ")"))
            qqplot_args <- combine_lists(list(...), default_args)
            output <- do.call(qqplot, args = qqplot_args)
            if(!isFALSE(qqline)){
              default_abline_args <- list(a = 0, b = 1, col = "red")
              abline_args <- combine_lists(qqline, default_abline_args)
              do.call(abline, args = abline_args)
            }
            invisible(output)
          }
        }
      }
    }
  }

# This function converts the r matrix into the "r tilde" matrix, as described in
# (23) on page 5 of the notes.
define_r_tilde <- function(r, mu, k2_tol){
  # Verify that there are as many columns in r as elements in theta_hat.
  stopifnot(is.matrix(r), is.matrix(mu), isTRUE(all(dim(r) == dim(mu))))
  # Define a matrix of the required size.
  r_tilde <- matrix(NA, nrow = nrow(r), ncol = ncol(r))
  # The first column is the same as that of r.
  r_tilde[,1] <- r[,1]
  # Now, iterate through the other columns of r to define the other columns of
  # r_tilde.
  for(i in seq_len(ncol(r) - 1) + 1){
    r_tilde[,i] <- r[,i]
    for(j in seq_len(i-1)){
      r_tilde[,i] <- k2(r_tilde[,i], mu[,j], r_tilde[,j], k2_tol = k2_tol)
    }
  }
  return(r_tilde)
}

rejection <- function(object, alpha = 0.05, stat = names(object[["empirical_stats"]]), ...){
  validate_args_rejection(object = object, alpha = alpha, stat = stat)
  output <- data.frame(stat = character(), rate = numeric(), mcse = numeric())
  for(i in seq_along(stat)){
    obs_stat <- object[["empirical_stats"]][[stat[i]]]
    mcsim_stat <- object[["theoretical_stats"]][[stat[i]]]
    func <- ecdf(obs_stat)
    q <- quantile(mcsim_stat, probs = 1 - alpha, ...)
    rejection_rate <- 1 - func(q)
    mcse <- unlist(calc_mcse_binom(p = rejection_rate,
                                   B = length(object[["theoretical_stats"]][[1]])))
    output <- rbind(output, data.frame(stat = stat[i],
                                       alpha = alpha,
                                       rate = rejection_rate,
                                       mcse = mcse))
  }
  return(output)
}

vcov_function <- function(f, Q, theta_hat, X, Y, jacobian_args, hessian_args){
  stopifnot(is.numeric(Y), is.vector(Y), is.numeric(X), is.matrix(X),
            is.numeric(Q) || is.list(Q), is.numeric(theta_hat),
            is.vector(theta_hat), is.function(f))
  n <- length(Y)
  p <- length(theta_hat)
  ftheta <- f2ftheta(f, X, n)
  g <- function(theta) (covprod(Q, (Y - ftheta(theta))))^2
  J <- as.matrix(drop(do.call(calculus::jacobian,
                              args = combine_lists(list(f = g, var = unname(theta_hat)),
                                                   jacobian_args))))
  
  psi <- simplify2array(apply(J, MARGIN = 1, FUN = tcrossprod, simplify = FALSE))
  P_psi <- if(is.vector(psi)) as.matrix(sum(psi)) else rowSums(psi, dims = 2)
  
  hess <- do.call(calculus::hessian,
                  args = combine_lists(list(f = g, var = unname(theta_hat), drop = FALSE),
                                       hessian_args))
  P_psi_dot <- apply(colSums(hess), MARGIN = 3, rbind)
  P_psi_dot_inv <- solve(P_psi_dot)
  
  return(P_psi_dot_inv %*% P_psi %*% t(P_psi_dot_inv))
}

scale_to_01 <- function(x, one_value = "omit"){
  stopifnot(is.numeric(x), is.matrix(x), one_value %in% c("omit", "fail"))
  X <- x
  unique_counts <- apply(X, MARGIN = 2, FUN = function(x) length(unique(x)))
  stopifnot(is.numeric(unique_counts))
  if(any(unique_counts == 1)){
    if(identical(one_value, "omit")){
      X <- X[,which(unique_counts != 1), drop = FALSE]
    } else {
      stop("At least one column of sorting matrix has only one value")
    }
  }
  col_mins <- apply(X, 2, min)
  X <- sweep(X, 2, col_mins)
  col_maxs <- apply(X, 2, max)
  X <- sweep(X, 2, col_maxs, FUN = "/")
  return(X)
}

calc_resid <-
function(Y, fitted_values){
  # Calculate raw residuals.
  # message("class(fitted_values: ", class(fitted_values))
  # message("dim(fitted_values): ", paste(dim(fitted_values), collapse = ", "))
  stopifnot(is.numeric(Y), is.numeric(fitted_values),
            is.vector(Y), is.vector(fitted_values),
            identical(length(Y), length(fitted_values)))
  return(as.vector(Y - fitted_values))
}

fit_model <- function(test_mean, data, method, method_args){
  default_method_args <- list(formula = test_mean, data = data,
                              na.action = na.fail)
  
  method_args <- combine_lists(method_args, default_method_args)
  
  if(identical(method, "lm") || identical(method, "glm")){
    method_args[["x"]] <- TRUE
    method_args[["y"]] <- TRUE
  }
  
  m <- do.call(get(method), args = method_args)
  
  return(m)
}

# This function assumes that true_covariance has already been filled as needed.
create_err_dist_args_default <- function(err_dist_fun_formals, cov_args, reps, n,
                                         true_covariance){
  err_dist_args_default <- true_covariance[cov_args]
  for(a in c("reps", "n")){
    if(a %in% err_dist_fun_formals) err_dist_args_default[[a]] <- get(a)
  }
  
  return(err_dist_args_default)
}

calc_sphered_resid <-
function(raw_residuals, Q){
  stopifnot(is.vector(raw_residuals), is.numeric(raw_residuals), is.numeric(Q) || is.list(Q))
  sphered_residuals <- as.vector(covprod(Q, raw_residuals))
  return(sphered_residuals)
}

validate_order_columns <- function(data, cols){
  stopifnot(is.list(cols))
  cols <- unlist(cols)
  if(!is.character(cols)){
    stop("Ordering column specification must be a list of character elements.")
  }
  # Verify no NAs.
  stopifnot("Columns for ordering has at least one NA" = all(!is.na(cols)),
            is.character(cols) || is.numeric(cols))
  # Verify that all elements are column names.
  bad_names <- cols[which(!(cols %in% colnames(data)))]
  if(length(bad_names) > 0)
    stop("Ordering columns not found in data: ",
         paste(bad_names, collapse = ", "))
}

validate_named_list <- function(named_list, valid_names = NULL){
  input_name <- deparse1(substitute(named_list))
  if(!is.list(named_list)){
    stop(input_name, " must be a named list")
  } else {
    if(length(named_list) > 0){
      element_names <- names(named_list)
      if(is.null(element_names) || any(nchar(element_names) == 0))
        stop("All elements of ", input_name, " must be named")
      if(!is.null(valid_names)){
        bad_names <- element_names[which(!(element_names %in% valid_names))]
        if(length(bad_names) > 0)
          stop(input_name, " has the following bad names: ",
               paste(bad_names, collapse = ", "))
      }
    }
  }
}

get_jacobian.function <- function(test_mean, ..., ftheta, theta_hat, control){
  J <- calc_jacobian(ftheta = ftheta, theta_hat = theta_hat,
                     jacobian_args = control[["jacobian_args"]])
  return(J)
}

residuals.distfreereg <-
  function(object, ..., type = "raw"){
    strict_match(arg = type, choices = c("raw", "sphered", "transformed"))
    stopifnot(!is.null(object[["residuals"]][[type]]))
    return(object[["residuals"]][[type]])
  }

update.distfreereg <- function(object, ..., smart = TRUE){
  override_supplied <- as.list(match.call(expand.dots = FALSE)$...$override)
  # Sometimes (only sometimes; why?), the first element is "list" as a language
  # object. This should be removed.
  if(length(override_supplied) >0 && identical(quote(list), override_supplied[[1]])){
    override_supplied <- override_supplied[-1]
  }
  override_supplied <- lapply(override_supplied, deparse)
  extras <- eval(match.call(expand.dots = FALSE)$...)
  if("override" %in% names(extras)) extras[["override"]] <- NULL
  
  old_arg_list <- as.list(getCall(object))[-1]
  
  if(!isTRUE(smart)){
    update.default(object = object, ...)
  } else {
    # First prepare for dealing with "override".
    
    n_old <- nrow(object[["J"]])
    p_old <- ncol(object[["J"]])
    
    dots <- list(...)
    n_all <- sapply(list(Y = dots[["Y"]], X = dots[["X"]],
                         data = dots[["data"]], J =
                             dots[["override"]][["J"]],
                         fitted_values =
                             dots[["override"]][["fitted_values"]]),
                    NROW)
    n_new <- setdiff(unique(n_all), 0)
    p_all <- as.numeric(c(sapply(dots[["override"]]["J"], NCOL),
                          sapply(c(dots["theta_init"],
                                   dots[["override"]][["theta_hat"]]),
                                 length)))
    p_new <- setdiff(unique(p_all), 0)
    
    if(length(n_new) == 0) n_new <- NULL
    if(length(p_new) == 0) p_new <- NULL
    if(length(n_new) > 1)
        stop("Sample size mismatch in update() arguments:\n",
             paste(names(n_all), n_all, sep = ": ", collapse = "\n"))
    if(length(p_new) > 1)
        stop("Parameter space dimension mismatch in arguments to update()")
    
    # Create a list of user inputs that should trigger recomputation of
    # values. For example, if user calls "update(object, X = X1)", then
    # object[["res_order"]] should not be used as an override. If none of X,
    # data, and ordering are specified in the call to update(), then it
    # should. The argument "verbose" is omitted from consideration here. Note
    # that the handling of "data" is not optimal, since "data" might change
    # without changing covariate values, which would not require updating
    # res_order.
    var_list <- list(
      theta_hat     = c("covariance", "data", "test_mean", "theta_init", "X",
                        "Y", "method", "method_args"),
      fitted_values = c("covariance", "data", "test_mean", "theta_init", "X",
                        "Y", "method", "method_args"),
      J             = c("covariance", "data", "test_mean", "theta_init", "X",
                        "Y", "method", "method_args"),
      res_order     = c("data", "X", "ordering", "group"),
      r             = c("data", "X", "ordering", "test_mean"),
      theoretical_stats   = c("B", "group", "ordering", "stat")
    )
    # Since some values are hidden within the control list, there is a
    # separate list analogous to var_list that contains the names of the
    # control list elements that should trigger recomputation.
    control_list <- list(
      theta_hat = c("solve_tol", "optimization_fun", "optimization_args"),
      J = c("jacobian_args"),
      r = c("qr_tol", "orth_tol")
    )
    # Next, since some values are hidden within the override list, there is a
    # separate list analogous to control_list. Note that it is not named
    # "override_supplied", since that name is used elsewhere.
    or_list <- list(
      J = c("theta_hat"),
      fitted_values = c("theta_hat"),
      r = c("res_order"),
      theoretical_stats = c("res_order", "r")
    )
    
    # Finally, define a list of custom tests for updating each variable. This
    # is important, for example, to avoid updating r when Y changes but
    # length(Y) does not. Each expression should evaluate to a single logical
    # value, and should be TRUE when var needs to be recalculated (that is,
    # not included in the override argument).
    custom_list <- list(
      res_order =
        isTRUE(!is.null(n_new) && n_new != n_old) ||
        isTRUE(isModel(eval(extras[["test_mean"]], envir = parent.frame()))),
      theta_hat = 
        isTRUE(is(object[["test_mean"]], "formula") || isModel(object[["test_mean"]])),
      r =
        isTRUE(!is.null(n_new) && n_new != n_old) ||
        isTRUE(!is.null(p_new) && p_new != p_old),
      theoretical_stats =
        isTRUE(!is.null(n_new) && n_new != n_old) ||
        isTRUE(!is.null(p_new) && p_new != p_old) ||
        isTRUE((isModel(object[["test_mean"]]) || is(object[["test_mean"]], "formula")) &&
                 !is.null(extras[["test_mean"]])) ||
        isTRUE(isTRUE(old_arg_list[["group"]]) && !is.null(extras[["ordering"]])) ||
        isTRUE(isTRUE(old_arg_list[["group"]]) && !is.null(extras[["X"]])) ||
        isTRUE(isTRUE(old_arg_list[["group"]]) && !is.null(extras[["data"]]))
    )
    
    # Now, for each variable in any of the lists above, set override_supplied[[var]]
    # to appropriate value from object unless a new argument should prevent it.
    override_candidates <- unique(c(names(var_list), names(control_list), names(or_list), names(custom_list)))
    include_in_override <- rep(FALSE, length(override_candidates))
    names(include_in_override) <- override_candidates
    for(var in names(include_in_override)){
      if(is.null(override_supplied[[var]]) &&
         !isTRUE(any(any(var_list[[var]] %in% names(extras)),
                     any(control_list[[var]] %in% names(extras[["control"]])),
                     any(or_list[[var]] %in% names(override_supplied)),
                     custom_list[[var]]))){
        # override_supplied[[var]] <- object[[var]]
        include_in_override[[var]] <- TRUE
      }
    }
    
    include_in_override <- names(include_in_override)[which(sapply(include_in_override, isTRUE))]
    
    if(length(include_in_override) > 0){
      override_smart <- paste0(deparse1(substitute(object)), "[['", include_in_override, "']]")
      names(override_smart) <- include_in_override
    } else {
      override_smart <- NULL
    }
    
    orig_call <- as.list(getCall(object))
    orig_func <- orig_call[[1]]
    
    if((is.null(orig_call[["test_mean"]]) && !("test_mean" %in% names(extras))) ||
       ("test_mean" %in% names(extras) && is.null(extras[["test_mean"]]))){
      # This accounts for the possibility that we need to reuse override values
      # when no test_mean value is provided.
      override_orig_list <- as.list(orig_call[["override"]])[c("fitted_values", "J")]
    } else {
      override_orig_list <- NULL
    }
    
    override_supplied_list <- lapply(override_supplied, str2lang)
    override_smart_list <- lapply(override_smart, str2lang)
    override_combined_list <- combine_lists(
      override_supplied_list,
      override_orig_list,
      override_smart_list
    )
    override_list <- as.call(c(quote(list), override_combined_list))
    override_list <- list(override = override_list)
    
    # Second, deal with covariance, which is not part of the "override"
    # paradigm.
    
    if(is.null(extras[["covariance"]]) &&
       (is(object[["test_mean"]], "function") || is.null(object[["test_mean"]]))){
      # extras[["covariance"]] <- object[["covariance"]]
      extras[["covariance"]] <- str2lang(paste0(deparse1(substitute(object)), "[['covariance']]"))
    }
    
    orig_call[[1]] <- orig_call[["override"]] <- orig_call[["covariance"]] <- NULL
    
    cl <- as.call(combine_lists(list(orig_func), extras, orig_call, override_list))
    
    eval(cl, envir = parent.frame())
  }
}

vcov.distfreereg <- function(object, ..., jacobian_args = NULL, hessian_args = NULL){
  stopifnot(is(object, "distfreereg"))
  test_mean <- object[["test_mean"]]
  theta_hat <- object[["theta_hat"]]
  Q <- object[["covariance"]][["Q"]]
  stopifnot(!is.null(test_mean), !is.null(theta_hat), !is.null(Q))
  if(is(test_mean, "function")){
    output <- vcov_function(f = test_mean,
                            Q = Q,
                            theta_hat = theta_hat,
                            X = object[["data"]][["X"]],
                            Y = object[["data"]][["Y"]],
                            jacobian_args = jacobian_args,
                            hessian_args = hessian_args)
    colnames(output) <- rownames(output) <- names(theta_hat)
  } else {
    if(!is.null(jacobian_args) || !is.null(jacobian_args))
      warning("jacobian_args and/or hessian_args ignored when 'test_mean' is not a function")
    if(is(test_mean, "formula")){
      m <- object[["model"]]
      stopifnot(!is.null(m))
    } else {
      m <- object[["test_mean"]]
    }
    output <- vcov(object = m, ...)
  }
  return(output)
}

simulate_response.function <- function(true_mean, ..., theta, true_X, n){
  Y_mean <- as.vector(f2ftheta(true_mean, true_X, n)(theta))
  errors <- generate_error_matrix(..., n = n)
  output <- as.matrix(Y_mean + errors)
  rownames(output) <- seq_len(nrow(output))
  return(output)
}

rmvnorm <- function(n, reps, mean = rep(0, nrow(SqrtSigma)), SqrtSigma = NULL){
  stopifnot(is.numeric(SqrtSigma) || is.list(SqrtSigma))
  if(is.numeric(SqrtSigma) && is.vector(SqrtSigma)){
    stopifnot(length(SqrtSigma) %in% c(1, n))
    output <- matrix(rnorm(reps * n, sd = SqrtSigma), ncol = reps)
  } else {
    nc <- if(is.matrix(SqrtSigma)){
      ncol(SqrtSigma)
    } else {
      stopifnot(is.list(SqrtSigma))
      sum(sapply(SqrtSigma, ncol))
    }
    output <- covprod(SqrtSigma, matrix(rnorm(reps * nc), ncol = reps))
  }
  return(output)
}

calc_k2_resid <-
  # This implements the function U_p^*.
  function(x, r_tilde, mu, k2_tol){
    stopifnot(is.numeric(x), is.vector(x),
              is.matrix(r_tilde), is.matrix(mu),
              isTRUE(all(dim(r_tilde) == dim(mu))),
              identical(dim(mu)[1], length(x)))
    output <- x
    for(i in rev(seq_len(ncol(r_tilde)))){
      output <- k2(output, r_tilde[,i], mu[,i], k2_tol = k2_tol)
    }
    return(output)
  }

---
title: "An Introduction to the distfreereg Package"
output:
    rmarkdown::html_vignette:
      number_sections: true
vignette: >
  %\VignetteIndexEntry{An Introduction to the distfreereg Package}
  %\VignetteEngine{knitr::rmarkdown_notangle}
  %\VignetteEncoding{UTF-8}
header-includes:
  - \def\eqdef{\mathrel{\raise0.4pt\hbox{$:$}\hskip-2pt=}}
  - \def\ev{{\rm E}}
  - \def\given{\mathrel|}
  - \def\proglang#1{#1}
  - \def\class#1{\textsf{#1}}
  - \def\pkg#1{\textbf{#1}}
linkcolor: blue
link-citations: true
csl: american-statistical-association.csl
bibliography: bibliography.bib
---





```{r setup, include = FALSE}
knitr::opts_chunk$set(echo = TRUE, message = FALSE, out.width = "49%",
                      fig.dim = c(6,5), fig.align = "center")
is_check <- ("CheckExEnv" %in% search()) ||
  any(c("_R_CHECK_TIMINGS_", "_R_CHECK_LICENSE_") %in% names(Sys.getenv()))
knitr::opts_chunk$set(eval = !is_check)

library(distfreereg)
```



# Introduction

The **distfreereg** package has two main functions: `distfreereg()` and
`compare()`. The former is introduced in this vignette; `compare()` is
introduced [here](../doc/v2_compare.html).

The function `distfreereg()` implements the distribution-free regression testing
procedure introduced by @Khmaladze2021. Its purpose is to test whether or not a
specified function $Y=f(X;\theta)$ is the conditional mean, $\ev(Y\given X)$.
Specifically, the test is of the following type:
\begin{equation}
  H_0{:}\ \exists\theta\in\Theta\,|\,\ev(Y\given X)=f(X;\theta)
  \quad\hbox{against}\quad
  H_1{:}\ \forall\theta\in\Theta\,|\,\ev(Y\given X)\neq f(X;\theta).
  \label{eqn:hypo1}
\end{equation}

This vignette starts by discussing the [testing of a linear
model](#testing-a-linear-model) created using `lm()`. Following this is an
analogous discussion of [testing a non-linear
model](#testing-a-non-linear-model) created using `nls()`. Both scenarios can be
replicated using the [formulas directly](#means-specified-by-formulas). In the
[most general implementation](#means-specified-by-functions), the mean function
is specified by an R function. To use `distfreereg()` with a mean
function implemented in software other than R, see the section on the
[default
method](#the-default-method-using-the-algorithm-with-external-functions).






# Testing a Linear Model

Suppose that a linear model object is created using `lm()`, and we want to test
whether or not a linear model is appropriate; specifically, whether or not the
true mean function follows a specified form that is linear in its parameters.

In some cases, a residual plot suffices to show that the specified form is
wrong. Below, the true mean function is $\ev(y\given x)=x^{2.1}$, but the
formula used to build the model is linear in $x$. The residuals plot clearly
indicates model mis-specification.

```{r lm_testing_with_plots}
set.seed(20240304)
n <- 3e2
form_lm <- y ~ x
data_lm <- data.frame(x = runif(n, min = 0, max = 3))
data_lm$y <- data_lm$x^2.1 + rnorm(n, sd = 0.3)
m_1 <- lm(form_lm, data = data_lm)
plot(m_1, which = 1)
```

If, however, we use a form that is quadratic in $x$, the residuals plot is
suggestive, but it leaves us without a clear conclusion. How certain can we be
that the specified form is wrong?

```{r create_lm}
form_lm_2 <- y ~ I(x^2)
m_2 <- lm(form_lm_2, data = data_lm)
plot(m_2, which = 1)
```

In this case, there appears to be evidence of a mis-specified mean, but we need
a formal test to quantify our (un)certainty. The function `distfreereg()`
implements such a test.

```{r lm_method}
set.seed(20240304)
(dfr_lm_2 <- distfreereg(test_mean = m_2))
```

In fact, tests based on two statistics are included by default: a
Kolmogorov--Smirnov (KS) statistic and a Cramér--von Mises (CvM) statistic,
defined as $\max_i|w_i|$ and ${1\over n}\sum_iw_i^2$, respectively, where
$w\eqdef(w_1,\ldots,w_n)$ is the partial sum process calculated by
`distfreereg()`. The preceding output shows a table summarizing the tests'
results, including p-values and Monte Carlo standard errors thereof. Both of the
p-values are marginal, supplying a measure of our intuitive conclusion based on
the residuals plot.






# Testing a Non-Linear Model

The procedure just illustrated applies to objects created with `nls()`, as well.
Here, the data generating function has the form being tested, so we should not
expect the null to be rejected. Note that the `weights` argument in `nls()` is
set equal to the inverse of the variance vector.

```{r create_nls}
set.seed(20240304)
n <- 3e2
sds <- runif(n, min = 0.5, max = 5)
data_nls <- data.frame(x = rnorm(n), y = rnorm(n))
data_nls$z <- exp(3*data_nls$x) - 2*data_nls$y^2 + rnorm(n, sd = sds)
form_nls <- z ~ exp(a*x) - b*y^2
m_2 <- nls(form_nls, data = data_nls, weights = 1/sds^2, start = c(a = 1, b = 1))
```

Testing this form of the mean structure is done just as with an `lm`
object:

```{r nls_method}
set.seed(20240304)
(dfr_nls <- distfreereg(test_mean = m_2))
```

Both tests correctly fail to reject the null.







# Means Specified by Formulas

If no `lm` or `nls` object has been created yet, the formula and data can be
supplied directly to `distfreereg()`. Below, we recreate the examples from above
without first creating the `lm` and `nls` objects.

The most notable practical difference in implementing the tests in this context
is that three arguments are required rather than just one:

1. `test_mean`: a `formula` object specifying the mean function;

1. `data`: a data frame containing the data;

1. `method`: specifies the function to use to create the model.



## Formulas with `lm()`

The following call conducts the same test as in `dfr_lm_2` above.

```{r formula_method_lm}
set.seed(20240304)
(dfr_form_lm_2 <- distfreereg(test_mean = form_lm_2, data = data_lm, method = "lm"))
```

The key pieces of the output are identical to those of `dfr_lm`:

```{r comparison_lm_verify, include = FALSE, echo = FALSE}
stopifnot(identical(dfr_lm_2$observed_stats, dfr_form_lm_2$observed_stats))
stopifnot(identical(dfr_lm_2$p, dfr_form_lm_2$p))
```

```{r comparison_lm}
all(identical(dfr_lm_2$observed_stats, dfr_form_lm_2$observed_stats),
    identical(dfr_lm_2$p, dfr_form_lm_2$p))
```


## Formulas with `nls()`

An additional element of `method_args` is used here to specify starting values
for parameter estimation. (This is optional, since `nls()` will use default
starting values with a warning if not supplied. See `nls()` documentation for
further details.) The following call conducts the same test as in `dfr_nls`.

```{r formula_method_nls}
set.seed(20240304)
(dfr_form_nls <- distfreereg(test_mean = form_nls, data = data_nls, method = "nls",
                             method_args = list(weights = 1/sds^2,
                                                start = c(a = 1, b = 1))))
```

The key pieces of the output are identical to those of `dfr_nls`:

```{r comparison_nls_verify, include = FALSE, echo = FALSE}
stopifnot(identical(dfr_nls$observed_stats, dfr_form_nls$observed_stats))
stopifnot(identical(dfr_nls$p, dfr_form_nls$p))
```

```{r comparison_nls}
all(identical(dfr_nls$observed_stats, dfr_form_nls$observed_stats),
    identical(dfr_nls$p, dfr_form_nls$p))
```






# Means Specified by Functions

The most general case that can be handled entirely within R is that
in which the mean regression function is specified by an R function.

Suppose that we want to replicate the situation in the `nls` examples
above, but we want to account for errors that have a non-diagonal covariance
matrix. The example below illustrates how to do this.

The function being tested is specified by `test_mean`, which is a function with
two arguments, `X` and `theta`. Here, `X` represents the entire matrix of
covariates. Therefore, to replicate the references to `x` and `y` in [this
example](#testing-a-non-linear-model), we use `X[,1]` and `X[,2]`, respectively.
The arguments `X` and `Y` supply the covariate matrix and the response vector,
respectively, in lieu of the `data` argument.

```{r dfr_1}
set.seed(20240304)
n <- 3e2
true_mean <- function(X, theta) exp(theta[1]*X[,1]) - theta[2]*X[,2]^2
test_mean <- true_mean
theta <- c(3,-2)
Sigma <- rWishart(1, df = n, Sigma = diag(n))[,,1]
X <- matrix(rnorm(2*n), nrow = n)
Y <- distfreereg:::f2ftheta(true_mean, X)(theta) +
  distfreereg:::rmvnorm(n = n, reps = 1, SqrtSigma = distfreereg:::matsqrt(Sigma))

(dfr_1 <- distfreereg(test_mean = test_mean, Y = Y, X = X,
                      covariance = list(Sigma = Sigma),
                      theta_init = rep(1, length(theta))))
```

As with the `nls` example, testing the mean function used to generate the
data produces test results that correctly fail to reject the null hypothesis.




# Using the Algorithm with External Functions

All of the examples above illustrate how `distfreereg()` can be used when the
mean function being tested is defined in R. If the mean function is
not defined in R, `distfreereg()` can still be used as long as
certain objects can be imported into R. Specifically, `distfreereg()`
needs the model's fitted values and the Jacobian of the mean function, evaluated
at the estimated parameter vector for each observation's covariate values.

To illustrate this, we extract certain elements from the example
[above](#means-specified-by-functions), and pretend that these were created
using external software, and then imported into R.

```{r default_inputs}
J <- dfr_1$J
fitted_values <- fitted(dfr_1)
```

These can be supplied to `distfreereg()` in the `override` argument to implement
the test. Note that `test_mean` is set to `NULL`, and since no parameter
estimation is done here, no value for `theta_init` is specified.

```{r default}
distfreereg(test_mean = NULL, Y = Y, X = X, covariance = list(Sigma = Sigma),
            override = list(fitted_values = fitted_values, J = J))
```






# Summary and Diagnostic Plots

Below is an introduction to the three types of plots available for
`distfreereg` objects. See the [plotting vignette](../doc/v3_plotting.html)
for more details regarding plot customization.

## A Summary Plot

A summary of the results of the test can be plotted easily.
```{r}
plot(dfr_1)
```

The density of the simulated statistic is plotted, as is a vertical line showing
the observed statistic. The upper tail is shaded, and the p-value (the area of
that shaded region) is shown. Finally, a 95% simultaneous confidence band of the
density function is plotted, and the region is shaded in grey.

## Diagnostic Plots

Two diagnostic plots are also available. The first produces a time-series-like
plot of the residuals in the order specified by `res_order` in the
`distfreereg` object.

```{r residuals_plot}
plot(dfr_1, which = "residuals")
```

The second produces a plot of the empirical partial sum process.

```{r epsp_plot}
plot(dfr_1, which = "epsp")
```





# Specifying the Covariance Structure

The covariance structure of the errors, conditional on the covariates, must be
specified for the testing algorithm. When `test_mean` is not a `function`,
the covariance structure is estimated using the supplied/created model object.
When `test_mean` is a `function`, this structure must be specified by the
user using the `covariance` argument. The value of `covariance` is a list with
at least one named element that specifies one of four matrices:

1. `Sigma`, the covariance matrix;

1. `SqrtSigma`, the square root of `Sigma`;

1. `P`, the precision matrix (that is, the inverse of `Sigma`); and

1. `Q`, the square root of `P`.

Internally, the algorithm only needs `Q`, so some efficiency can be gained by
supplying this directly if it is known. Supplying more than one of the four
matrices is not forbidden, but is strongly discouraged. In this case, no
verification is done that the supplied matrices are consistent with each other,
and `Q` will be calculated using the most convenient supplied element.

Each of the four named elements can be one of four types of object:

1. A positive-definite matrix, the most general way to specify a covariance
structure;

1. A list of positive-definite matrices;

1. a numeric vector of positive values whose length is the sample size; and

1. a length-1 numeric vector specifying a positive number.

Specifying a list of matrices is theoretically equivalent to specifying a block
diagonal matrix with the listed matrices as the blocks. Specifying a numeric
vector is theoretically equivalent to specifying a diagonal matrix with the
given vector along the diagonal. Specifying a single number is theoretically
equivalent to specifying a diagonal matrix with that value along the diagonal.
Using the simplest possible expression in each case is recommended for
conceptual and computational simplicity.

The following code shows that supplying `Q` produces observed statistics
identical to those in [the previous example](#means-specified-by-functions).

```{r dfr_2}
Q <- distfreereg:::matinv(distfreereg:::matsqrt(Sigma), tol = .Machine$double.eps)
dfr_2 <- distfreereg(Y = Y, X = X, test_mean = test_mean,
                     covariance = list(Q = Q),
                     theta_init = rep(1, length(theta)))
identical(dfr_1$observed_stats, dfr_2$observed_stats)
```

```{r dfr_2_verify, include = FALSE, echo = FALSE}
stopifnot(identical(dfr_1$observed_stats, dfr_2$observed_stats))
```






# Methods

Several common generic functions have `distfreereg` methods, including `coef()`,
`predict()`, and `update()`. See the documentation for the complete list.



# Adding New Statistics

The `stat` argument of `distfreereg()` allows the user to specify other
statistics by specifying any function whose input is a numeric vector and whose
output is a numeric vector of length one.

```{r new_stat}
new_stat <- function(x) sum(abs(x))
update(dfr_1, stat = "new_stat")
```

If necessary, the default statistics can be selected using "`KS`" and "`CvM`" as
explicit values in the character vector supplied to `stat`.

```{r three_stats}
update(dfr_1, stat = c("new_stat", "KS", "CvM"))
```

Checks are done to verify that the functions provided calculate legitimate
statistics; that is, that they take a numeric vector as input and output a
single number.

```{r stats_checks}
bad_stat <- function(x) x[1,2]
try(update(dfr_1, stat = c("KS", "CvM", "bad_stat")))
try(update(dfr_1, stat = c("KS", "CvM", "undefined_stat")))
```



# Non-Normal Errors

One of the strengths of the test implemented by `distfreereg()` is that it makes
only weak assumptions about the distribution of the errors. Specifically, it
only requires that the error covariance matrix be finite and that the error
distribution be strongly mixing (also known as $\alpha$-mixing). Below is an
example illustrating that we get the expected results when the error
distribution is the $t$ distribution. Note that the $t$ distribution with three
degrees of freedom has variance 3.

The first call to `distfreereg()` below tests the true mean function, and the
test correctly fails to reject the null hypothesis. The second call tests a
wrong mean function, and the test correctly rejects the null hypothesis.

```{r t_dist}
set.seed(20241003)
n <- 1e2
func <- function(X, theta) theta[1] + theta[2]*X[,1] + 0.5*X[,1]^2
theta <- c(2,5)
X <- matrix(rexp(n))
Y <- theta[1] + theta[2]*X[,1] + 0.5*X[,1]^2 + rt(n, df = 3)

dfr <- distfreereg(Y = Y, X = X, test_mean = func, theta_init = c(1,1),
                   covariance = list(Sigma = 3))
dfr

alt_func <- function(X, theta) theta[1] + theta[2]*X[,1]
update(dfr, test_mean = alt_func)
```







# References

---
title: "Parameter Estimation with distfreereg"
output:
  rmarkdown::html_vignette:
    number_sections: true
vignette: >
  %\VignetteIndexEntry{Parameter Estimation with distfreereg}
  %\VignetteEngine{knitr::rmarkdown_notangle}
  %\VignetteEncoding{UTF-8}
  linkcolor: blue
header-includes:
  - \def\eqdef{\mathrel{\raise0.4pt\hbox{$:$}\hskip-2pt=}}
  - \def\ev{{\rm E}}
  - \def\given{\mathrel|}
  - \def\proglang#1{\textsf{#1}}
  - \def\class#1{\textsf{#1}}
  - \def\pkg#1{\textbf{#1}}
linkcolor: blue
link-citations: true
csl: american-statistical-association.csl
bibliography: bibliography.bib
---


```{r setup, include = FALSE}
knitr::opts_chunk$set(echo = TRUE, message = FALSE, out.width = "49%",
                      fig.dim = c(6,5), fig.align = "center")
is_check <- ("CheckExEnv" %in% search()) ||
  any(c("_R_CHECK_TIMINGS_", "_R_CHECK_LICENSE_") %in% names(Sys.getenv()))
knitr::opts_chunk$set(eval = !is_check)

library(distfreereg)
```




# Introduction

An essential part of the testing procedure implemented by `distfreereg()` is the
estimation of the model's parameters. When using the `lm` or `nls` methods, or
the corresponding `formula` methods, parameter estimation is handled using the
modeling functions themselves. Below is the initial setup of the `distfreereg`
object.

```{r dfr_1}
set.seed(20240926)
n <- 3e2
true_mean <- function(X, theta) exp(theta[1]*X[,1]) - theta[2]*X[,2]^2
test_mean <- true_mean
theta <- c(3,-2)
Sigma <- rWishart(1, df = n, Sigma = diag(n))[,,1]
X <- matrix(rnorm(2*n), nrow = n)
Y <- distfreereg:::f2ftheta(true_mean, X)(theta) +
  distfreereg:::rmvnorm(n = n, reps = 1, SqrtSigma = distfreereg:::matsqrt(Sigma))

dfr_1 <- distfreereg(test_mean = test_mean, Y = Y, X = X,
                     covariance = list(Sigma = Sigma),
                     theta_init = rep(1, length(theta)))
```


# Modifying the Method

For the `function` method, parameters are estimated using the `optim()`
function by default. The default optimization method is `BFGS`, which works well
for many cases. The method can be changed using the `control` argument of
`distfreereg()`. For example, if we know beforehand that
$\theta=(\theta_1,\theta_2)$ satisfies $2\leq\theta_1\leq4$ and
$-3\leq\theta_2\leq-1$, we could use `L-BFGS-B` to account for bounds for the
parameter vector.

```{r dfr_2}
set.seed(20240926)
dfr_2 <- update(dfr_1, control = list(
  optimization_args = list(method = "L-BFGS-B", lower = c(2,-3), upper = c(4,-1))))
all.equal(dfr_1$theta_hat, dfr_2$theta_hat)
```

The results are not identical, but are nearly so.



# Estimating the Precision of Parameter Estimates

To estimate the precision with which the parameters have been estimated,
`vcov()` has a `distfreereg` method. When `test_mean` is a `formula`, an `lm`
object, or an `nls` object, the model is supplied to `vcov()` for appropriate
method dispatch. When `test_mean` is a `function`, then the method described in
@Vaart2007 is applied.

```{r vcov}
vcov(dfr_1)
```

An analogous method exists for `confint()`, which uses this covariance matrix
to calculate confidence intervals at a stated confidence level:

```{r confint}
confint(dfr_1, level = 0.9)
```





# Using a Different Optimization Function

By default, `distfreereg.function()` uses `optim()` to estimate the model's
parameters. Suppose, however, we wanted to use `nlminb()` instead. (This
function is selected to illustrate the use of another optimization function
only, and is not an implicit recommendation.) We can use `distfreereg()`'s
`control` argument to do this.

For a given optimization function, we need three things:

1. The name of the argument that specifies the function to be minimized.

1. The name of the argument that specifies the starting parameter values.

1. The name of the element in the returned value that contains the parameter
estimates.

In the case of `nlminb()`, a review of the help page indicates that these are
"`objective`", "`start`", and "`par`", respectively. We supply these three
names, along with the function `nlminb` itself, to the `control` argument as
follows.

```{r nlm, message = FALSE}
set.seed(20240926)
dfr_3 <- update(dfr_1, control = list(optimization_fun = nlminb,
                                      fun_to_optimize_arg = "objective",
                                      theta_init_arg = "start",
                                      theta_hat_name = "par"))
```

Once again, these are not identical, but are nearly so.

```{r}
all.equal(dfr_1$theta_hat, dfr_3$theta_hat)
```





# Detailed Output from the Optimization Process

Detailed output from `optim()`, helpful for diagnosing some parameter estimation
problems, is found in the `optimization_output` element of the output:
```{r}
dfr_3$optimization_output
```



# Checking Compatibility of Arguments

Before optimizing, `distfreereg.function()` verifies that the mean function can
be evaluated at `theta_init`.  This will detect some problems but does not
guarantee compatibility. This should be kept in mind when an opaque error
message arises. In the examples below, the starting vectors are either too short
or too long.

```{r optim_bad_theta_init}
try(update(dfr_1, theta_init = 1))# starting parameter vector too short!
try(update(dfr_1, theta_init = rep(1, 3)))# starting parameter vector too long!
```










# The Importance of Normality of Parameter Estimates

The theory behind the test implemented by `distfreereg()` requires that the
parameter estimates be approximately normally distributed around the true
parameter value. With regularity assumptions on the test mean function, this is
true in theory, but it must also be sufficiently close to true in practice. This
requires good optimization algorithms. In the example below, something goes
awry.

## The Setup

In this example, we use a different error generating function from the default,
specifically one that uses a block-diagonal covariance matrix and generates
errors corresponding to each block using a multivariate $t$ distribution.
Further, we use the default method for `optim()`, namely Nelder--Mead.

First, define the function and true parameter vector.


```{r block_diagonal_setup_func_pars}
true_func <- function(X, theta) theta[1] + theta[2]*X[,1] + X[,1]^2
theta <- c(2,5)
```

Next, define the error distribution function. This first specifies the dimension
`matdim` of the blocks. The function `block_t()` generates the errors by
repeated calls to `mvtnorm::rmvt()` with the appropriate scale matrix to obtain
the correct covariance structure. (Recall that the errors for each repetition
done by `compare()` are stored in the columns of the error matrix.)

```{r block_diagonal_setup_err_dist}
matdim <- 10
block_t <- function(n, reps, blocks, df){
  matsqrt_list <- distfreereg:::covsqrt(blocks, tol = -(.Machine[["double.eps"]])^0.25)
  output <- matrix(NA, nrow = n, ncol = reps)
  for(i in seq_len(reps))
    output[,i] <- as.vector(
      sapply(matsqrt_list,
             function(x) distfreereg:::rmvt(1, df = df, SqrtSigma = x*sqrt((df-2)/df))))
  return(output)
}
```


## The Simulation

Finally, we set a random seed, generate covariates and a covariance matrix, and
then run `compare()`.

```{r run_with_32}
set.seed(32)
n <- 500
X <- as.matrix(rexp(n, rate = 1))
Sig <- lapply(1:(n/matdim), function(x) rWishart(1, df = matdim, diag(matdim))[,,1])

cdfr_seed32 <- compare(theta = theta,
                       true_mean = true_func, test_mean = true_func,
                       true_X = X, X = X,
                       true_covariance = list(Sigma = Sig), covariance = list(Sigma = Sig),
                       theta_init = c(1,1),
                       err_dist_fun = "block_t",
                       err_dist_args = list(blocks = Sig, df = 4),
                       reps = 1e3,
                       manual = function(dfr) dfr$theta_hat,
                       control = list(optimization_args = list(method = "Nelder-Mead")))
```

Next, extract the saved parameter estimates.

```{r extract_parameter_estimates}
theta_seed32_1 <- sapply(cdfr_seed32$manual, function(x) x[[1]])
theta_seed32_2 <- sapply(cdfr_seed32$manual, function(x) x[[2]])
```

Since we are using the same function for `true_mean` and `test_mean`, the
cumulative distribution functions of the observed and simulated statistics
should be nearly the same. The following plot is therefore alarming.

```{r cdf_plot}
plot(cdfr_seed32)
```

We can see that there is a problem with the parameter estimates, namely that
their densities are not approximately normally distributed about the true
parameter values.

```{r density_plots}
plot(density(theta_seed32_1))
plot(density(theta_seed32_2))
```

## The Most Likely Source of the Problem

The problem seems to be the use of Nelder--Mead here, since the default method
selected by `distfreereg()`, BFGS, does not result in such distributions in
similar cases. (For the example above, this is left to the reader.) This is, of
course, no guarantee that BFGS will always be better, but it does appear to be a
more sensible default here.

## Two Possible Objections

The keen-eyed reader will note two possible objections to the analysis above:
perhaps the sample size is insufficient, and perhaps the input of `set.seed()`
was carefully selected to obtain these results.

The first of these objections can be addressed by showing the results with a
different seed, where the plots suggest no substantial problems.

```{r sample_size_not_the_problem}
set.seed(1)
X <- as.matrix(rexp(n, rate = 1))
Sig <- lapply(1:(n/matdim), function(x) rWishart(1, df = matdim, diag(matdim))[,,1])
cdfr_seed1 <- update(cdfr_seed32, X = X, true_covariance = list(Sigma = Sig),
                     covariance = list(Sigma = Sig))
theta_seed1_1 <- sapply(cdfr_seed1$manual, function(x) x[[1]])
theta_seed1_2 <- sapply(cdfr_seed1$manual, function(x) x[[2]])
plot(cdfr_seed1)
plot(density(theta_seed1_1))
plot(density(theta_seed1_2))
```

A formal test that the samples have come from the same distribution is also
reassuring.

```{r kstest}
ks.test(cdfr_seed1)
```

The second possible objection, namely that the seed was carefully selected to
produce these results, is only half true. It was selected, but it was not
difficult to find. Considering integers from 1 to 40 as inputs to `set.seed()`,
at least 16, 26, 27, and 32 produce poor results. The phenomenon is not
difficult to reproduce.









# References

---
title: "Plotting with the distfreereg Package"
output:
  rmarkdown::html_vignette:
      number_sections: true
vignette: >
  %\VignetteIndexEntry{Plotting with the distfreereg Package}
  %\VignetteEngine{knitr::rmarkdown_notangle}
  %\VignetteEncoding{UTF-8}
header-includes:
  - \def\eqdef{\mathrel{\raise0.4pt\hbox{$:$}\hskip-2pt=}}
  - \def\ev{{\rm E}}
  - \def\given{\mathrel|}
  - \def\proglang#1{\textsf{#1}}
  - \def\class#1{\textsf{#1}}
  - \def\pkg#1{\textbf{#1}}
linkcolor: blue
link-citations: true
csl: american-statistical-association.csl
bibliography: bibliography.bib
---

\def\eqdef{\mathrel{\raise0.4pt\hbox{$:$}\hskip-2pt=}}

```{r setup, include = FALSE}
knitr::opts_chunk$set(echo = TRUE, out.width = "49%", fig.dim = c(6,5),
                      fig.align = "center")
is_check <- ("CheckExEnv" %in% search()) ||
  any(c("_R_CHECK_TIMINGS_", "_R_CHECK_LICENSE_") %in% names(Sys.getenv()))
knitr::opts_chunk$set(eval = !is_check)

library(distfreereg)
```



# Introduction

Plotting methods for `distfreereg` and `compare` objects are intended to provide
sensible defaults, almost all of which can be modified easily as needed.  While
the examples below are separated into `distfreereg` and `compare` classes, the
way modifications are made in each case is similar.



# Plotting `distfreereg` Objects

The following code creates the `distfreereg` object with a fairly simple
setup that is used in most of the examples that follow.

```{r dfr, message = FALSE}
set.seed(20240214)
n <- 1e2
true_mean <- function(X, theta) theta[1] + theta[2]*X[,1]
theta <- c(2,5)

X <- matrix(runif(n, min = 1, max = 100))
Y <- true_mean(X, theta) + rnorm(n)

dfr <- distfreereg(Y = Y, X = X, test_mean = true_mean,
                   covariance = list(Sigma = 1),
                   theta_init = rep(1, length(theta)))
```


## Density Plots

The default plot displays the estimated density of the simulated statistics:
```{r}
plot(dfr)
```

The estimated density of the simulated statistic is plotted, as is a vertical
line showing the observed statistic. The upper tail is shaded, and the p-value
(the area of the shaded region) is shown. A 95% simultaneous confidence band for
the estimated density function is plotted, as well.




### Modifying the P-Value Label

The plot above is not ideal because the p-value label overlaps the density
curve. The specifications for this label can be modified using the `text_args`
argument, the elements of which are passed to `graphics::text()`.

The default placement is vertically centered on the left side of the vertical
line. By using the `text_args` argument, the text can be printed on the right
side of the line and shifted down a bit. Note that only the elements to change
need to be specified in the list supplied to `text_args`.

```{r}
plot(dfr, text_args = list(adj = c(0, 0.5), y = 0.5))
```

One special value, `text_args = FALSE`, prevents the label from being printed.

```{r}
plot(dfr, text_args = FALSE)
```





### Selecting the Statistic to Plot

The `stat` argument can be used to produce the corresponding plot for the CvM
statistic.
```{r}
plot(dfr, stat = "CvM", text_args = FALSE)
```




### Modifying the Density Calculation and Plotting

The calculation of the density curve can be modified using `density_args`, whose
elements are passed to `density()`.

To modify the appearance of the curve once the density curve has been estimated,
pass arguments to `plot.default()` via the `...` argument. The example below
shows how to modify the line type.

```{r}
plot(dfr, text_args = FALSE, lty = 2)
```



### Modifying the P-Value Line and Shading

The appearance of the vertical line can be modified using the `abline_args`
argument, whose elements are passed to `abline()`. As with the text label, the
line can be omitted by setting `abline_args` equal to `FALSE`.

```{r}
plot(dfr, text_args = FALSE, abline_args = FALSE)
```

The shading under the curve is produced by calling `polygon()`, and can be
modified by passing arguments to that function via `polygon_args`. It can also
be omitted by setting `polygon_args = FALSE`.

```{r}
plot(dfr, text_args = FALSE, polygon_args = list(density = 10))
```

As a convenience, the shading color can be changed using the `shade_col`
argument. This is equivalent to modifying the `col` argument of `polygon()`.
```{r}
plot(dfr, text_args = FALSE, shade_col = rgb(0.5, 0.5, 0.8, 0.5))
```



### Borders

The default behavior is to omit the border of the shaded region. As seen above,
this is notable when the vertical line is omitted. This can be changed by
setting `border = NULL`, its default value in `polygon()`, which (usually)
results in a border.

```{r}
plot(dfr, text_args = FALSE, abline_args = FALSE, polygon_args = list(border = NULL))
```





### Output

In case the values used to create a plot are useful for further calculation,
each call to `plot.distfreereg()` invisibly returns these values in a list with
either two or three elements. The first two elements, `x` and `y`, contain the
coordinates of the curve. If confidence bands are plotted, then a third element
named `confband` is also returned.

```{r distfreereg_returned_values}
output <- plot(dfr, confband_args = NULL, text_args = FALSE)
names(output)
```

The $y$-values of the curves that determine the confidence band are saved in the
`cb_lower` and `cb_upper` elements, while the $x$-values are saved in `w`.

```{r}
names(output$confband)
```




## Diagnostic Plots

Below are examples of two diagnostic plots available through
`plot.distfreereg()`.

### Residual Plots

A useful diagnostic plot displays the transformed residuals ordered according to
the `res_order` element of the `distfreereg` object.

```{r residual_plot}
plot(dfr, which = "residuals")
```

As with the density plot, all options can be modified if needed by including
additional arguments for `plot()`.

```{r modified_residual_plot}
plot(dfr, which = "residuals", main = "New Title", lty = "dashed")
```



### Empirical Partial Sum Process Plots

Another useful diagnostic plot displays the values of the empirical partial sum
process, which can be modified as expected.

```{r epsp_plot}
plot(dfr, which = "epsp")
plot(dfr, which = "epsp", xlab = "i", col = "red")
```








# Plotting `compare` Objects

Most of the examples of plot modifications apply to `compare` objects, as
well.  These are illustrated explicitly below.


## Setup

The following code creates the `compare` object with a fairly simple setup
that is used in all of the examples that follow.

```{r reference_object, message = FALSE}
set.seed(20240920)
n <- 100
func <- function(X, theta) theta[1] + theta[2]*X[,1] + theta[3]*X[,2]
theta <- c(2,5,-1)
X <- matrix(rexp(2*n), nrow = n)
cdfr <- compare(theta = theta, true_mean = func, test_mean = func,
                true_X = X, true_covariance = list(Sigma = 3), X = X,
                covariance = list(Sigma = 3), prog = Inf,
                theta_init = rep(1, length(theta)))
```




## CDF Plots

By default, `plot.compare()` displays the graphs of the estimated cumulative
distribution functions of the observed and simulated statistics.

```{r cdf}
plot(cdfr)
```

### Curves

The appearance of the function curves can be modified using the `curve_args`
argument. Note that this passes values to `lines()`, not `curve()`. The value
of `curve_args` must be a list. If an argument of `lines()` is an element of
this list, then its value is passed to the calls for both curves. For example,
the width of both curves can be changed as follows.

```{r modify_curves}
plot(cdfr, curve_args = list(lwd = 3))
```

To change a property of only one curve, two special (named) elements of this
list are available: "`obs`" and "`mcsim`". Each of these, if present, must be a
list. Their elements are passed to the `lines()` call of the corresponding
curve. The following example shows how to change the thickness of both curves
but the style of only the observed statistics curve.

```{r modify_curves_differently}
plot(cdfr, curve_args = list(lwd = 3, obs = list(lty = 4)))
```


### Legend

The argument `legend` can be used to modify the default behavior of the legend.

```{r modify_legend}
plot(cdfr, legend = list(title = "A Title", bg = "grey"))
```

While not recommended, it can be omitted by setting `legend` to "`FALSE`".

```{r omit_legend}
plot(cdfr, legend = FALSE)
```


### Horizontal Lines

The horizontal dashed lines are plotted by default to mimic the default behavior
of `plot.ecdf()`. These can be modified using the `hlines` argument, whose value
is a list of arguments to pass to `abline()`.

```{r modify_horizontal_lines}
plot(cdfr, hlines = list(lty = 1, lwd = 3))
```


### Confidence Bands

Confidence bands are not plotted by default, but they can be included with
default values as follows.

```{r confidence_bands}
plot(cdfr, confband_args = NULL)
```





## Density Plots

Estimated density curves can be plotted using the `which` argument.

```{r density}
plot(cdfr, which = "dens")
```

The area under each density curve is shaded by default. These can be modified,
either together or separately, as can be done with `curve_args`. The example
below shows how to change the density of the shading for both curves but the
color and angle only of the observed statistics curve.

```{r modify_shading}
plot(cdfr, which = "dens",
     poly = list(density = 20, obs = list(col = rgb(0.5,0.2,0.2,0.2), angle = -45)))
```

Other options, such as `curve_args`, operate here as described above in the
discussion of CDF plotting.



## Q--Q Plots

The other plots available are Q--Q plots: one compares observed and simulated
statistics, and the other compares p-values to uniform quantiles.

```{r qqplots}
plot(cdfr, which = "qq")
plot(cdfr, which = "qqp")
```

Both of these plots accept optional lists of arguments to pass to `qqplot()`

```{r modify_qqplots}
plot(cdfr, which = "qq", conf.level = 0.95)
```

The diagonal line can be modified using the `qqline` argument, whose elements
are passed to `abline()`.

```{r modify_diagonal_line}
plot(cdfr, which = "qqp", qqline = list(lwd = 3))
```





## Multiple `compare` Objects

It might be useful to compare the observed statistics in two `compare`
objects.  This can be done by supplying both objects to `compare()`. To
illustrate, we first create a second `compare` object:

```{r second_object, message = FALSE}
set.seed(20240920)
n <- 100
func <- function(X, theta) theta[1] + theta[2]*X[,1]
theta <- c(7,3)
X <- matrix(rexp(n), nrow = n)
cdfr2 <- compare(theta = theta, true_mean = func, test_mean = func,
                 true_X = X, true_covariance = list(Sigma = 3), X = X,
                 covariance = list(Sigma = 3), prog = Inf,
                 theta_init = rep(1, length(theta)))
```

The following call compares the observed statistics from the two `compare`
objects.

```{r compare_two_objects}
plot(cdfr, cdfr2)
```




## Output

In case the values used to create a plot are useful for further calculation,
each call to `plot.compare()` invisibly returns these values in a list. The
Q--Q plots both return the value returned by `qqplot()` itself. The other plots
return values corresponding to their curves, including confidence bands, if
plotted.

```{r compare_returned_values}
output <- plot(cdfr, confband_args = NULL)
names(output)
```
The $y$-values of the curves that determine the confidence band are saved in the
`cb_lower` and `cb_upper` elements, while the $x$-values are saved in `w`.

```{r}
names(output$confband_empirical)
```

---
title: "Advanced Options for the distfreereg Package"
output:
  rmarkdown::html_vignette:
      number_sections: true
vignette: >
  %\VignetteIndexEntry{Advanced Options for the distfreereg Package}
  %\VignetteEngine{knitr::rmarkdown_notangle}
  %\VignetteEncoding{UTF-8}
  linkcolor: blue
header-includes:
  - \usepackage{natbib}
  - \def\eqdef{\mathrel{\raise0.4pt\hbox{$:$}\hskip-2pt=}}
  - \def\ev{{\rm E}}
  - \def\given{\mathrel|}
  - \def\proglang#1{\textsf{#1}}
  - \def\class#1{\textsf{#1}}
  - \def\pkg#1{\textbf{#1}}
linkcolor: blue
link-citations: true
csl: american-statistical-association.csl
bibliography: bibliography.bib
---



\def\trans{\top}
\def\Var{{\rm Var}}

```{r setup, include = FALSE}
knitr::opts_chunk$set(echo = TRUE, out.width = "49%", fig.dim = c(6,5),
                      fig.align = "center")
is_check <- ("CheckExEnv" %in% search()) ||
  any(c("_R_CHECK_TIMINGS_", "_R_CHECK_LICENSE_") %in% names(Sys.getenv()))
knitr::opts_chunk$set(eval = !is_check)

library(distfreereg)
```




# Specifying the Test Mean Covariate Argument: Uppercase or Lowercase

When `test_mean` is an R function, it has an (optional) argument for
the covariates. This argument can be an uppercase `X` or a lowercase `x`.  Most
examples in other vignettes use the uppercase `X`, since this is the more
efficient choice, and suffices in most cases. The uppercase `X` requires,
though, that the expressions used in `test_mean` are vectorized.

In the case in which some part of the function is not vectorized, a lowercase
`x` can be used. Suppose, for example, that `integrate()` is required to define
`test_mean`, and that the covariate value is used as the upper limit of
integration. The example below illustrates this using the integral
$$
  \int_0^x2t\,dt,
$$
whose value is known to be $x^2$. We can therefore compare the results of the
lowercase example with those of the uppercase example.

```{r upperlower, message = FALSE}
set.seed(20240913)
n <- 1e2
func_upper <- function(X, theta) theta[1] + theta[2]*X[,1] + theta[3]*X[,2]^2
func_lower <- function(x, theta) theta[1] + theta[2]*x[1] +
  theta[3]*integrate(function(x) 2*x, lower = 0, upper = x[2])$value
theta <- c(2,5,1)
X <- matrix(rexp(2*n, rate = 1), ncol = 2)
Y <- distfreereg:::f2ftheta(f = func_upper, X)(theta) + rnorm(n)

set.seed(20240913)
dfr_upper <-  distfreereg(Y = Y, X = X, test_mean = func_upper,
                          covariance = list(Sigma = 1), theta_init = c(1,1,1))
set.seed(20240913)
dfr_lower <-  distfreereg(Y = Y, X = X, test_mean = func_lower,
                          covariance = list(Sigma = 1), theta_init = c(1,1,1))

all.equal(dfr_upper$observed_stats, dfr_lower$observed_stats)
all.equal(dfr_upper$p, dfr_lower$p)
```













# Ordering the Observations

The fundamental object involved in this testing procedure is the *empirical
partial sum process* (EPSP); that is, the scaled vector of cumulative sums of
transformed residuals. When the model includes only one covariate (that is, when
the matrix $X$ of covariates has only one column), the order in which the
residuals are added to form this process is determined by the natural linear
order of the covariates. When more than one covariate is present, though, no
single order is evidently the "correct" one.

## Under the Null Hypothesis

When the null hypothesis is true, the order does not affect the distribution of
the statistics of the EPSP. To illustrate this, consider the following example
with only one covariate.

```{r ordering, message = FALSE}
set.seed(20241001)
n <- 1e2
func <- function(X, theta) theta[1] + theta[2]*X[,1]
theta <- c(2,5)
X <- matrix(rexp(n))

cdfr_simplex <- compare(true_mean = func,
                        true_X = X,
                        true_covariance = list(Sigma = 1),
                        theta = theta,
                        test_mean = func,
                        covariance = list(Sigma = 1),
                        X = X,
                        theta_init = c(1,1),
                        keep = 1)
cdfr_asis <- update(cdfr_simplex, ordering = "asis")
```

The order in which the residuals are added is determined by the `ordering`
argument of `distfreereg()`. The default method of determining the order is the
"`simplex`" method. In the case of a single covariate, this method orders the
observations in increasing order of covariate values. Another built-in option is
to preserve to given order of the observations; that is, the order in which the
residuals are added is the order of the observations specified in the input to
`distfreereg()`. The CDF plots of each method are shown below, and both pairs of
curves are mostly overlapping.

```{r ordering_plots}
plot(cdfr_simplex)
plot(cdfr_asis)
```


## Under an Alternative Hypothesis

When the null hypothesis is false, however, the order of the residuals can have
a substantial effect on the power of the test. If the residuals are added in a
random order (which they are in the "`asis`" objects, because `X` is randomly
generated and not ordered), the test is generally less able to detect deviations
from the expected null-hypothesis behavior of the EPSP. Ordering the covariates
increases the power of the test to detect such deviations.

To see this in practice, suppose we now change the previous example so that the
true mean function has a quadratic term.

```{r change_mean, message = FALSE}
alt_func <- function(X, theta) theta[1] + theta[2]*X[,1] + 0.5*X[,1]^2
cdfr_simplex_alt <- update(cdfr_simplex, true_mean = alt_func)
cdfr_asis_alt <- update(cdfr_asis, true_mean = alt_func)
```

The plots below show that the CDFs for the `simplex` method are more separated
than those for the `asis` method.

```{r ordering_plots_alt}
plot(cdfr_simplex_alt)
plot(cdfr_asis_alt)
```

The estimated powers of these tests, as expected, highly favor the default
`simplex` method:

```{r powers}
rejection(cdfr_simplex_alt, stat = "KS")
rejection(cdfr_asis_alt, stat = "KS")
```


## Diagnostic Plots

The previous plots are instructive regarding the importance of ordering, but are
not useful when we want to explore model mis-specification. To do that,
examining the residuals can be helpful, as suggested by @Khmaladze2021. The plot
below uses the example above in which the null hypothesis is true.

```{r epsp_null}
plot(cdfr_simplex$dfrs[[1]], which = "epsp")
```

Now let us look at the plots when the alternative hypothesis is true. First, the
plot with residuals ordered in the order of `X`:

```{r epsp_alt_asis}
plot(cdfr_asis_alt$dfrs[[1]], which = "epsp")
```

This plot seems similar to the previous one. On the other hand, the plot with
the residuals ordered by covariate value show a clear pattern.

```{r epsp_alt_simplex}
plot(cdfr_simplex_alt$dfrs[[1]], which = "epsp")
```

The slopes of this plot indicate that the residuals are positive, then negative,
and then positive again as the covariate values increase. (This is a common
trend when a linear function is used to model data with a quadratic mean
function.) This patterns can also be seen by looking at the following plot.

```{r fitted_and_observed}
dfr_simplex <- cdfr_simplex_alt$dfrs[[1]]
X <- dfr_simplex$data$X
Y <- dfr_simplex$data$Y[order(X)]
Y_hat <- fitted(dfr_simplex)[order(X)]
X <- sort(X)
ml <- loess(Y ~ X)
plot(X, predict(ml), type = "l", col = "blue", ylab = "Y")
points(X, Y, col = rgb(0,0,1,0.4))
lines(X, Y_hat, col = "red", lty = "dashed")
legend(x = "bottomright", legend = c("smoothed", "fitted"), col = c("blue", "red"),
       lty = c("solid", "dashed"))
```


## Methods

The `asis` option is included primarily for exploratory purposes, and for cases
in which observations are given in the desired order. In general, the ordering
should arise from a principled mapping of the observations onto the time
domain. One such option is provided by the `simplex` method, which scales each
covariate to the unit interval, and then orders the observations in increasing
order of the sums of their scaled values.

Another option is `optimal`, which orders observations using optimal transport.
This is illustrated below. In this example, the power is comparable to the
`simplex` method, but the patterns in the EPSP is not as clear.

Note: this method is computationally expensive when the sample size is large.

```{r optimal, message = FALSE}
cdfr_optimal_alt <- update(cdfr_simplex_alt, ordering = "optimal")
plot(cdfr_optimal_alt)
rejection(cdfr_optimal_alt, stat = "KS")
plot(cdfr_optimal_alt$dfrs[[1]], which = "epsp")
```

Another option available for ordering observations is the "`natural`" ordering,
which can also be seen as analogous to a lexicographic ordering: the
observations are ordered by covariate values from left to right. That is,
observations are ordered in ascending order of the first covariate in the matrix
or data frame containing the covariates; ties are broken by ordering by the
second covariate; remaining ties are broken by the third; and so on.

In the following example, `Y` is generated using an expression with a quadratic
term, but the function being tested is linear in its covariates. We hope that
the tests reject the null.

```{r natural, message = FALSE}
set.seed(20241003)
n <- 1e2
theta <- c(2,5,-1)
X <- cbind(sample(1:5, size = n, replace = TRUE),
           sample(1:10, size = n, replace = TRUE))
Y <- theta[1] + theta[2]*X[,1] + theta[3]*X[,2] + (2e-1)*X[,2]^2 + rnorm(nrow(X))

func <- function(X, theta) theta[1] + theta[2]*X[,1] + theta[3]*X[,2]

dfr <- distfreereg(Y = Y, X = X, test_mean = func, theta_init = c(1,1,1),
                   covariance = list(Sigma = 1), ordering = "natural")
dfr
```

These results can be compared to the `simplex` method.

```{r compare_with_simplex, message = FALSE}
update(dfr, ordering = "simplex")
```

The results for these methods are comparable in this example.

More flexibility is possible by specifying a list of column names or numbers
that will result in the process above being used except only with the listed
columns in the listed order.




# Grouping Observations

The previous example's covariates contain many repeated value combinations.
These covariate observations are indistinguishable, and their order among
themselves is dependent on the order in which they happen to appear in the data.
To avoid this arbitrariness in the calculation of test statistics, a grouping
option is available. When the `group` argument is `TRUE`, the transformed
residuals for repeated covariate values are combined before forming the EPSP.
This avoids dependence on the order in which the observations enter the data.

Grouping is done by default. To prevent grouping, use `group = FALSE`, as in the
following example.

```{r group, message = FALSE}
dfr_not_grouped <- update(dfr, group = FALSE)
```

Note that the length of `epsp` in `dfr_not_grouped` is the sample size. In
`dfr`, it is not. It is instead the number of unique combinations of values in
the two columns of `X`.

```{r length_of_epsp}
length(dfr_not_grouped$epsp)
length(dfr$epsp)
length(which(table(X[,1], X[,2]) > 0))
```

The test results are similar, but not identical to, those in `dfr`.

```{r dfr_grouped_results}
dfr_not_grouped
```




# The `override` Argument

The `override` argument of `distfreereg()` is used by `update.distfreereg()` to
avoid unnecessary recalculation. This has particular benefits for the
performance of `compare()`, which uses `update.distfreereg()`.

This argument can be useful in other cases, too. For example, if the
observations should be ordered in a way that is not among the available options
for `ordering`, this order can be specified using `res_order`.

The example below illustrates a more involved application using `compare()` That
function uses `override` to avoid recalculating the `r`, `res_order`, and
`mc_stats` elements of the `distfreereg` objects it creates. Any values
specified in the `override` list supplied to `update()` take precedence over all
others, and are therefore used in every repetition of `compare()`. Let us use
this below to illustrate that using the true value of $\theta$ instead of
$\hat\theta$ does not produce good results.

```{r override, message = FALSE}
set.seed(20241003)
n <- 2e2
func <- function(X, theta) theta[1] + theta[2]*X[,1]
theta <- c(2,5)
X <- matrix(rexp(n))

cdfr <- compare(true_mean = func, test_mean = func, true_X = X, X = X,
                true_covariance = list(Sigma = 1), covariance = list(Sigma = 1),
                theta = theta, theta_init = c(1,1))
cdfr_theta <- update(cdfr, override = list(theta_hat = theta))
```

The first plot is what we expect, given that the true mean and test mean are the
same in `cdfr`:

```{r cdfr_plot}
plot(cdfr)
```

However, the next plot shows that using $\theta$ does not yield good results:

```{r cdfr_theta_plot}
plot(cdfr_theta)
```







# Matrix Square Roots and Covariance Specifications

An early step in the testing procedure is to sphere the residuals; that is, to
left multiply the residual vector $\hat\epsilon$ by a matrix to remove
covariances among the residuals. Let $\Sigma$ denote the covariance matrix of
$\hat\epsilon$, and let $Q$ denote any matrix satisfying $Q^\trans
Q=\Sigma^{-1}$. Then $Q\hat\epsilon$ has the identity covariance matrix. There
are infinitely many such matrices $Q$ that satisfy $Q^\trans Q=\Sigma^{-1}$.
(See @Kessy2018 for a discussion of sphering matrices.) Two considerations are
particularly important here: computational feasibility and performance within
the goodness-of-fit testing procedure. All of the discussion in this section
pertains to the case in which `test_mean` is a function, and therefore the
covariance structure must be specified by the user. In particular, it pertains
to the case in which the covariance structure must be specified by a matrix
rather than one of the simpler forms (e.g., a vector).

Matrix square roots calculated by **distfreereg** are calculated using
eigendecompositions. These are "true" square roots in the sense that they are
symmetric, and therefore satisfy, e.g., $QQ=\Sigma^{-1}$. Eigendecompositions
are not computationally feasible in all cases (specifically when the dimensions
are large), in which case it is necessary to specify either `SqrtSigma` or `Q`
instead of `Sigma`. Such a specification must result in a matrix $Q$ such that
$Q^\trans Q=\Sigma^{-1/2}$. One such option is to use a Cholesky factor.
For example, let $LL^\trans$ be a Cholesky decomposition of $\Sigma$, and let
$Q=L^{-1}$. Such a value of $Q$ is acceptable as the `Q` element of
`covariance`. Simulations indicate, however, that this choice of square root
leads to lower power than an eigendecomposition-based square root.

As an example, the following simulation compares the powers of the two default
tests (the KS- and CvM-based tests) using the eigendecomposition-based $Q$ and
the Cholesky-based $Q$. The basics of the simulation are set up below.

```{r Q_comparison, message = FALSE}
theta <- c(1,-1)
true_mean <- function(X, theta) exp(theta[1]*X[,1]) + theta[2]*X[,2]
test_mean <- function(X, theta) theta[1]*X[,1] + theta[2]*X[,2]

n <- 2e2
set.seed(20250903)
Sigma <- rWishart(1, df = n, Sigma = diag(n))[,,1]
SqrtSigma <- distfreereg:::matsqrt(Sigma)
Q_eigen <- solve(SqrtSigma)
Q_chol <- t(solve(chol(Sigma)))

X <- matrix(rnorm(2*n), nrow = n)

Y <- distfreereg:::f2ftheta(true_mean, X)(theta) +
  distfreereg:::rmvnorm(n = 1, reps = 1, SqrtSigma = SqrtSigma)
```

To verify that the sample size is adequate for asymptotic results,
`asymptotics()` is applied to `distfreereg` objects that are created using
`test_mean`, one of which uses `Q_eigen` and the other of which uses `Q_chol`.
The plots indicate that the sample size is adequate.

```{r Q_comp_dfr, message = FALSE}
dfr_eigen <- distfreereg(test_mean = test_mean, Y = Y,
                         X = X, theta_init = c(1,1),
                         covariance = list(Q = Q_eigen))
dfr_chol <- distfreereg(test_mean = test_mean, Y = Y,
                        X = X, theta_init = c(1,1),
                        covariance = list(Q = Q_chol))
cdfr_eigen <- asymptotics(dfr_eigen)
cdfr_chol <- asymptotics(dfr_chol)

plot(cdfr_eigen)
plot(cdfr_chol)
```

Finally, `compare` objects are created to estimate the tests' power. The
rejection rates indicate that the Cholesky decomposition leads to lower power
for both tests.

```{r Q_comp_rejection, message = FALSE}
cdfr_eigen_H1 <- update(cdfr_eigen, true_mean = true_mean, theta = theta)
cdfr_chol_H1 <- update(cdfr_chol, true_mean = true_mean, theta = theta)
rejection(cdfr_eigen_H1)
rejection(cdfr_chol_H1)
```








# References

---
title: "Comparing Distributions with the distfreereg Package"
output:
  rmarkdown::html_vignette:
    number_sections: true 
vignette: >
  %\VignetteIndexEntry{Comparing Distributions with the distfreereg Package}
  %\VignetteEngine{knitr::rmarkdown_notangle}
  %\VignetteEncoding{UTF-8}
header-includes:
  - \def\eqdef{\mathrel{\raise0.4pt\hbox{$:$}\hskip-2pt=}}
  - \def\ev{{\rm E}}
  - \def\given{\mathrel|}
  - \def\proglang#1{\textsf{#1}}
  - \def\class#1{\textsf{#1}}
  - \def\pkg#1{\textbf{#1}}
linkcolor: blue
link-citations: true
csl: american-statistical-association.csl
bibliography: bibliography.bib
---


\def\Secref#1{\hyperref[sec:#1]{Section~\ref{sec:#1}}}
\def\eqdef{\mathrel{\raise0.4pt\hbox{$:$}\hskip-2pt=}}

```{r setup, include = FALSE}
knitr::opts_chunk$set(echo = TRUE, out.width = "49%", fig.dim = c(6,5),
                      fig.align = "center")
is_check <- ("CheckExEnv" %in% search()) ||
  any(c("_R_CHECK_TIMINGS_", "_R_CHECK_LICENSE_") %in% names(Sys.getenv()))
knitr::opts_chunk$set(eval = !is_check)

library(distfreereg)
```







# Comparing Empirical and Theoretical Distributions of Statistics
\label{sec:compare}

Because the procedure implemented by `distfreereg()` is distribution-free only
asymptotically, it is important to explore how well the approximations perform
at finite sample sizes.  How large the sample size must be for the asymptotic
behavior to be sufficiently achieved is dependent on the details of each
application. To facilitate the exploration of sample size requirements,
**distfreereg** provides the function `compare()`.

In the following example, we explore the required sample size when the true mean
is $$f(X;\theta) = \theta_1 + \theta_2X_1 + \theta_3X_2,$$ $\theta=(2,5,-1)$,
and the errors are independent standard normal errors. The covariate matrix is
generated randomly. We start with a sample size of 10.

```{r comp_1, message = FALSE}
set.seed(20240913)
n <- 10
func <- function(X, theta) theta[1] + theta[2]*X[,1] + theta[3]*X[,2]
theta <- c(2, 5, -1)
X <- matrix(rexp(2*n, rate = 1), nrow = n)
comp_dfr <- compare(theta = theta, true_mean = func, test_mean = func,
                    true_X = X, true_covariance = list(Sigma = 3), X = X,
                    covariance = list(Sigma = 3),
                    theta_init = rep(1, length(theta)))
```

This function generates a response vector `Y` from `true_mean` using the given
values of `true_X`, `true_covariance` (by default, the errors are multivariate
normal), and `theta`. It then passes `Y` and most of the remaining arguments to
`distfreereg()`. The statistics and p-values corresponding to each simulated
data set are saved, and the process is repeated. The results are returned in an
object of class `compare`.




# Plotting the Results

These results can be explored graphically. Several options are available. The
default behavior of the `compare` method for `plot()` produces a
comparison of estimated cumulative distribution functions (CDFs) for the
observed and simulated statistics. If the sample size is sufficiently large,
these two curves should be nearly identical.

```{r cdf}
plot(comp_dfr)
```

This plot gives some cause for concern.

Another way of exploring this uses quantile--quantile (Q--Q) plots. Below is a
Q--Q plot that compares observed and simulated statistics. If the sample size is
sufficiently large, these points should lie on the line $y=x$.

```{r qq}
plot(comp_dfr, which = "qq")
```

The curvature indicates an insufficient sample size. Similar to this is the next
plot, which is a Q--Q plot comparing p-values to uniform quantiles. Once again,
if the sample size is sufficiently large, these points should lie on the line
$y=x$.

```{r qqp}
plot(comp_dfr, which = "qqp")
```

These plots all indicate that this sample size is insufficient for the
asymptotic behavior for the KS statistic. Similar plots can be made for the CvM
statistic. More details on this plot method are discussed [in the plotting
vignette](../doc/v3_plotting.html).


# Formal Comparison Testing

The `compare` method for `ks.test()`, which compares the observed and
simulated distributions, confirms that they are still different:

```{r ks.test}
ks.test(comp_dfr)
```

The default statistic is whatever is the first statistic appearing in the
supplied object:

```{r}
names(comp_dfr$obs)
ks.test(comp_dfr, stat = "CvM")
```




# Increasing the Sample Size

Let us repeat the simulation with a sample size of 100.
```{r comp_larger_n, message = FALSE}
n_2 <- 100
X_2 <- matrix(rexp(2*n_2, rate = 1), nrow = n_2)
comp_dfr_2 <- update(comp_dfr, true_X = X_2, X = X_2)
```

The following plots indicate that this sample size is sufficient.

```{r qqplots_2}
plot(comp_dfr_2)
plot(comp_dfr_2, which = "qq")
plot(comp_dfr_2, which = "qqp")
```

A formal test confirms this assessment.

```{r ks.test_2}
ks.test(comp_dfr_2)
```



# Using a `distfreereg` Object: `asymptotics()`

When a `distfreereg` object has already been created, the function
`asymptotics()` provides a convenient wrapper for `compare()` to explore sample
size adequacy. See the function's documentation for details.



# Power Considerations

The examples using `compare()` above all illustrate behavior of the function
when the null hypothesis is true. It is also important to investigate the
behavior when it is false to learn about the test's power against particular
alternative hypotheses. Using the example in \Secref{compare}, consider the case
in which the true mean function is quadratic in $X_2$: $f(x;\theta) = \theta_0 +
\theta_1x_1 + \theta_2x_2 + x_2^2$. Having verified above that a sample size of
100 is sufficient for the asymptotic behavior to be sufficiently approximated,
this can be investigated as follows.

```{r power, message = FALSE}
alt_func <- function(X, theta) theta[1] + theta[2]*X[,1] + theta[3]*X[,2] + 0.5*X[,2]^2
comp_dfr_3 <- update(comp_dfr_2, true_mean = alt_func, true_covariance = list(Sigma = 3))
```

```{r}
plot(comp_dfr_3)
```

As hoped, these curves are clearly different. Power estimates can be obtained
with `rejection()`.

```{r}
rejection(comp_dfr_3)
```

Power for different significance levels can be found by changing the `alpha`
argument.

```{r}
rejection(comp_dfr_3, alpha = 0.01)
```


# Warnings

The test implemented by `distfreereg()` is distribution-free, but simulating
values requires specifying an error distribution. The default behavior of
`compare()` is to use multivariate normal errors, or the default behavior of
`simulate()`. There is no way to avoid specifying a distribution to run these
simulations. Alternative error distributions can be specified, however, when
`true_mean` is a `function`, an `nls` object, or a `formula` with `method` equal
to "`nls`".

The safe interpretation of these simulations is limited to the error
distribution that is used.

\name{ks.test.compare}
\alias{ks.test.compare}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
	Formally Compare Empirical and Theoretical Statistics from a \code{compare} Object
}
\description{
    This is a \code{\link{ks.test}} method for objects of class \code{compare}.
    It performs a two-sample Kolmogorov--Smirnov test to compare the observed
    and simulated statistics in an object of class \code{compare}.
}
\usage{
	\method{ks.test}{compare}(x, ..., stat = NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
	Object of class \code{compare}.
}
  \item{...}{
	Additional parameters passed to \code{ks.test}.
}
  \item{stat}{
	Character string specifying the statistic on which to run the test.
}
}
\details{
    When \code{stat} is \code{NULL}, the default value is the first statistic
    appearing in the \code{observed_stats} element of \code{object}.
}
\value{
	A list of the form specified in \code{\link{ks.test}}.
}
\author{
	Jesse Miller
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
	\code{\link{compare}}, \code{\link{distfreereg}}, \code{\link{ks.test}}
}
\examples{
# In practice, set "reps" larger than 200.
set.seed(20240201)
n <- 100
func <- function(X, theta) theta[1] + theta[2]*X[,1]
Sig <- rWishart(1, df = n, Sigma = diag(n))[,,1]
theta <- c(2,5)
X <- matrix(rexp(n, rate = 1))
cdfr <- compare(true_mean = func, true_X = X, true_covariance = list(Sigma = Sig),
                test_mean = func, X = X, covariance = list(Sigma = Sig), reps = 200,
                prog = Inf, theta = theta, theta_init = rep(1, length(theta)))

ks.test(cdfr)
ks.test(cdfr, stat = "CvM")
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory (show via RShowDoc("KEYWORDS")):
% \keyword{ ~kwd1 }
% \keyword{ ~kwd2 }
% Use only one keyword per line.
% For non-standard keywords, use \concept instead of \keyword:
% \concept{ ~cpt1 }
% \concept{ ~cpt2 }
% Use only one concept per line.

\name{print.distfreereg}
\alias{print.distfreereg}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
	Printing \code{distfreereg} Objects
}
\description{
	This is a \code{\link{print}} method for objects of class \code{distfreereg}.
}
\usage{
	\method{print}{distfreereg}(x, ..., digits = 3, col_sep = 2, show_params = TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
	Object of class \code{distfreereg}.
}
  \item{\dots}{
	Additional parameters, currently ignored.
}
  \item{digits}{
	Integer; passed to \code{\link{signif}} to determine the number of significant digits to display.
}
  \item{col_sep}{
	Integer; specifies the padding (in units of spaces) between columns in the printed table of statistics.
}
  \item{show_params}{
	Logical; determines whether or not the parameter estimates, if present in \code{x}, are displayed.
}
}
\details{
	This function prints a useful summary of the \code{distfreereg} object \code{x}.
}
\value{
	No return value (\code{NULL}).
}
\author{
	Jesse Miller
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
	\code{\link{distfreereg}}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory (show via RShowDoc("KEYWORDS")):
% \keyword{ ~kwd1 }
% \keyword{ ~kwd2 }
% Use only one keyword per line.
% For non-standard keywords, use \concept instead of \keyword:
% \concept{ ~cpt1 }
% \concept{ ~cpt2 }
% Use only one concept per line.

\name{plot.compare}
\alias{plot.compare}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
	Summary and Diagnostic Plots for \code{compare} Objects
}
\description{
    This is a \code{\link{plot}} method for objects of class \code{compare}. It
    automates the creation of four summary and diagnostic plots for
    \code{compare} objects. See the \href{../doc/v3_plotting.html}{Plotting with
        the \code{distfreereg} Package} vignette for examples.
}
\usage{
	\method{plot}{compare}(x, y, ..., which = "cdf", stat = NULL, hlines = NULL, curve_args = NULL,
	confband_args = FALSE, density_args = NULL, poly = NULL, legend = NULL,
	qqline = NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
	Object of class \code{compare}.
}
  \item{y}{
	Optional object of class \code{compare}.
}
  \item{\dots}{
	Additional parameters passed to a plotting function depending on the
	value of \code{which}: to \code{\link{plot}} for "\code{cdf}" and
	"\code{dens}"; to \code{\link{qqplot}} for "\code{qq}" and "\code{qqp}".
}
  \item{which}{
    Character string. Acceptable values are "\code{cdf}", "\code{dens}",
    "\code{qq}", and "\code{qqp}":
    \itemize{
	  \item "\code{cdf}" produces a plot of the estimated cumulative distribution
	  functions of the two vectors of statistics being compared.
	  \item "\code{dens}" produces a plot of the estimated density functions of
	  the two vectors of statistics being compared.
	  \item "\code{qq}" produces a quantile--quantile plot comparing the two
	  vectors of statistics.
	  \item "\code{qqp}" produces a quantile--quantile plot comparing the p-values
	  with uniform quantiles. (This is not available when \code{y} is present.)
	}
}
  \item{stat}{
  Character string, specifies the statistic to plot.
}
  \item{hlines}{
    An optional list of arguments to pass to \code{\link{abline}}, used to
    create the horizontal dashed lines when \code{which} is "\code{cdf}".
    Setting equal to \code{FALSE} prevents the call, and no lines are drawn.
  }
  \item{curve_args}{
	An optional list used to pass arguments to \code{\link{lines}} (not
    \code{\link{curve}}!), used to create the curves when \code{which} is
    "\code{cdf}" or "\code{dens}". It can have two special named arguments,
    \code{obs} and \code{mcsim}, whose values must be lists. Those lists contain
    arguments passed to the calls to \code{\link{lines}} for plotting the curves
    for the empirical and theoretical statistics, respectively. Any other
    elements are passed to both calls.
  }
  \item{confband_args}{
    An optional list of values that control the calculation and plotting of
    confidence bands when \code{which} is "\code{cdf}" or "\code{dens}". Any of
    the following named elements are allowed.
    \itemize{
        \item \code{w}: Numeric; the sequence of points on which to evaluate the
            confidence band. By default, it is a sequence of \code{m}
            evenly-spaced numbers whose endpoints are determined by the
            quantiles of \code{x} calculated using the probabilities specified
            by \code{q_probs}, described below.	  
        \item \code{m}: Integer; the length of \code{w}, used only when \code{w}
            is \code{NULL}. The default value is 100.
        \item \code{batch_len}: Integer; the batch length for the algorithm. The
            default value is 50.
        \item \code{N}: Integer; the number of multivariate \eqn{t} samples to
            use in the simulation.
	  \item \code{conf.level}: Numeric; the desired confidence level.
	  \item \code{q_probs}: Numeric vector of length 2; the probabilities sent
            to \code{\link{quantile}} to calculate the bounds used to define
            \code{w}, used only when \code{w} is \code{NULL}.	  
      \item \code{curve_args}: An optional list of arguments passed to
            \code{\link{lines}} (again, not \code{\link{curve}}!), used to
            create the boundaries of the confidence band. It can have two
            special named arguments, \code{obs} and \code{mcsim}, which function
            in the same way as the corresponding elements of the
            \code{curve_args} argument described above.
      \item \code{polygon_args}: An optional list of arguments passed to
      \code{\link{polygon}}, used to shade the confidence region. Setting equal
      to \code{FALSE} prevents the call, and no shading is done.
      \item \code{shade_col}: This provides a shortcut to the \code{col}
      argument of \code{\link{polygon}} to change the color of the shaded
      region.
	}
	Setting equal to \code{FALSE} prevents calculation and plotting of the band.
}
  \item{density_args}{
    An optional list of arguments passed to \code{\link{density}} when
    \code{which} is "\code{dens}", which calculates the points used to plot the
    density curves. The list can have two special named elements, \code{obs} and
    \code{mcsim}, which function in the same way as the corresponding elements
    of the \code{curve_args} argument described above.
  }
  \item{poly}{
    An optional list of arguments passed to \code{\link{polygon}} when
    \code{which} is "\code{dens}", which shades the area under the density
    curves. The list can have two special named elements, \code{obs} and
    \code{mcsim}, which modify the shadings for their respective curves,
    analogous to their behavior in the \code{curve_args} argument. When
    \code{poly} is equal to \code{FALSE}, no call is made, and therefore no
    shading is done.
  }
  \item{legend}{
  An optional list of arguments passed to \code{\link{legend}} when \code{which}
  is "\code{cdf}" or "\code{dens}". When equal to \code{FALSE}, no call is made,
  and therefore no legend is created.
  }
  \item{qqline}{
  An optional list of arguments passed to \code{\link{abline}} when \code{which}
  is "\code{qq}" or "\code{qqp}". By default, this plots the line \eqn{y=x}.
  When equal to \code{FALSE}, no call is made, and therefore no line is plotted.
  }
}
\details{
	This function produces a plot of a type specified by \code{which}. The values
	plotted depend on whether or not \code{y} is present and the value of
	\code{which}. When \code{y} is present, the plots compare the empirical
	statistics in \code{x} and the empirical statistics in \code{y}. When \code{y}
	is missing, the plots compare the empirical and theoretical statistics in
	\code{x}. (The exception is when \code{which} is "\code{qqp}", which is only
	available when \code{y} is missing.)
	
	When \code{which} is "\code{cdf}" or "\code{dens}", the plotting region and
	associated labels, tick marks, etc., are created by an initial call to
	\code{\link{plot}}. The curves themselves are drawn with \code{\link{lines}}.
	The arguments specified in \code{...} are passed to the initial call to
	\code{\link{plot}}.
}
\value{
    The values used to create the curves (or points, in the case of a Q--Q plot)
    are returned invisibly. The details depend on the value of \code{which}:
    \itemize{
      \item \code{cdf}: A list with two or four elements, all lists. The first
      two sub-lists contain the \eqn{x}- and \eqn{y}-values cdf curves. If
      confidence bands are plotted, then two additional elements are included
      with output from the confidence band calculations, including elements
      \code{w}, \code{cb_lower}, and \code{cb_upper}, which contain,
      respectively, the \eqn{x}-coordinates for both the upper and lower bounds
      of the band, the \eqn{y}-coordinates for the lower band, and the
      \eqn{y}-coordinates for the upper band.
      \item \code{dens}: A list with two or four elements, all lists. The first
      two sub-lists contain \eqn{x}- and \eqn{y}-values for the density curves.
      If confidence bands are plotted, then two additional sub-lists are
      supplied, with contents identical to what is described for "\code{cdf}".
	  \item \code{qq}, \code{qqp}: The output of \code{\link{qqplot}}.
	}
  For "\code{cdf}" and "\code{dens}", the names of the elements of the returned
  list depend on whether or not a value for the argument \code{y} was supplied.
}
\author{
	Jesse Miller
}
\references{
Flegal, James M. et al. \strong{Simultaneous confidence bands for (Markov chain)
    Monte Carlo simulations}, forthcoming.
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
	\code{\link{distfreereg}}, \code{\link{compare}}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory (show via RShowDoc("KEYWORDS")):
% \keyword{ ~kwd1 }
% \keyword{ ~kwd2 }
% Use only one keyword per line.
% For non-standard keywords, use \concept instead of \keyword:
% \concept{ ~cpt1 }
% \concept{ ~cpt2 }
% Use only one concept per line.

\name{residuals.distfreereg}
\alias{residuals.distfreereg}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
	Extract Residuals from \code{distfreereg} Objects
}
\description{

    This is a \code{\link{residuals}} method for objects of class
    \code{distfreereg}. It can extract any of the three available types of
    residuals.

}
\usage{
	\method{residuals}{distfreereg}(object, ..., type = "raw")
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{object}{
	Object of class \code{distfreereg}.
}
  \item{\dots}{
	Additional parameters passed to or from other methods. Currently ignored.
}
  \item{type}{
	Character string specifying the type of residuals to return. Must be one of
	"\code{raw}", "\code{sphered}", and "\code{transformed}".
}
}
\value{
	Numeric vector of residuals.
}
\author{
	Jesse Miller
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
	\code{\link{distfreereg}}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory (show via RShowDoc("KEYWORDS")):
% \keyword{ ~kwd1 }
% \keyword{ ~kwd2 }
% Use only one keyword per line.
% For non-standard keywords, use \concept instead of \keyword:
% \concept{ ~cpt1 }
% \concept{ ~cpt2 }
% Use only one concept per line.

\name{asymptotics}
\alias{asymptotics}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
	Convenience Function for Exploring Asymptotic Behavior and Sample Size Adequacy
}
\description{
	This is a convenience function that calls \code{\link{compare}} using \code{object} to create the true and test model specifications.
}
\usage{
	asymptotics(object, ...)

}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{object}{
	Object of class \code{distfreereg}.
}
  \item{...}{
	Additional arguments to pass to \code{\link{compare}}.
}
}

\details{
	An important step in implementing \code{\link{distfreereg}} is determining the plausibility that the sample size of a data set is adequate to produce the desired asymptotic behavior of the simulated statistics. This can be done by calling \code{\link{compare}} with the appropriate argument values. Because of the importance of this step, this convenience function automates that call using a single argument, namely the object of class \code{distfreereg} in question.
}

\section{Warning}{
	The fitted model in \code{object} is assumed to be the true model, not an estimation of the true model. Further, the simulation of values requires assuming a particular error distribution. See the "Warnings" section in \code{\link{compare}}'s documentation for details.
}
\value{
	Object of class \code{compare}.
}
\author{
	Jesse Miller
}

%% ~Make other sections like Warning with \section{Warning }{....} ~
\seealso{
	\code{\link{distfreereg}}, \code{\link{compare}}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory (show via RShowDoc("KEYWORDS")):
% \keyword{ ~kwd1 }
% \keyword{ ~kwd2 }
% Use only one keyword per line.
% For non-standard keywords, use \concept instead of \keyword:
% \concept{ ~cpt1 }
% \concept{ ~cpt2 }
% Use only one concept per line.

\name{update.distfreereg}
\alias{update.distfreereg}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
	Update \code{distfreereg} Objects
}
\description{
	This is a \code{distfreereg} method for \code{\link{update}}. The method takes
	advantage of the \code{override} argument of \code{\link{distfreereg}} to
	prevent unnecessary recalculation of potentially computationally expensive
	objects.
}
\usage{
	\method{update}{distfreereg}(object, ..., smart = TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{object}{
	Object of class \code{distfreereg}.
}
  \item{\dots}{
	Additional named parameters to pass to \code{\link{distfreereg}}.
}
  \item{smart}{
	Logical. If \code{TRUE}, then saved values from \code{object} are passed to
	\code{\link{distfreereg}} using the \code{override} argument, when they need
	not themselves be updated. See details.
}
}
\details{
	This function updates an object of class \code{distfreereg}. By default, it
	does so "intelligently" in the sense that it does not unnecessarily recompute
	elements that are already saved in \code{object}. For example, if a new value
	for \code{covariance} is not included in \code{...}, then the value of
	\code{covariance} saved in \code{object} is automatically passed to the new
	call, preventing recalculating \code{Q}. If a new value of \code{covariance}
	is specified, then all objects dependent on that (e.g., \eqn{\hat\theta}) are
	recomputed by default.
	
	In particular, the simulated samples depend on the data and function only
	through the number of observations, the covariates (if any), and the dimension
	of the parameter space of the function. If none of these change, then the
	updated object reuses the simulated samples from the supplied \code{object}.
	
	This function uses the `override` argument of \code{\link{distfreereg}}, and
	therefore it must be handled carefully when it itself is being updated. To
	create the value of `override` for the updated call, the following three named
	lists are combined. When a name appears in more than one of these lists,
	priority is given in the order in which the lists are shown:
	\itemize{
	  \item The value of `override` supplied to \code{\link{update}}.
	  \item Values of `fitted_values` and `J` supplied to the `override` argument
	  in the `call` element of `object`, if the value of `test_mean` in the
	  updated call is `NULL`.
	  \item The list of "intelligently chosen" override values determined by
	  \code{update.distfreereg}.
	}
}
\value{
	An updated object of class \code{distfreereg}.
}
\author{
	Jesse Miller
}
\note{
  The usual behavior of \code{\link{update}} is to create an updated call and
  then evaluate that call. This is what \code{update.distfreereg} does, as well,
  but some of the updated elements are drawn from \code{object} itself for use
  as override values. In general, an object created by \code{update.distfreereg}
  is not \code{\link{identical}} to the object created by
  \code{\link{distfreereg}} using corresponding arguments, because the
  \code{call} values will differ.
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
	\code{\link{distfreereg}}
}
\examples{
set.seed(20240218)
n <- 1e2
func <- function(X, theta) X[,1]^theta[1] + theta[2]*X[,2]
Sig <- runif(n, min = 1, max = 3)
theta <- c(2,5)
X <- matrix(runif(2*n, min = 1, max = 5), nrow = n)
Y <- X[,1]^theta[1] + theta[2]*X[,2] + rnorm(n, sd = sqrt(Sig))
dfr_1 <- distfreereg(Y = Y, X = X, test_mean = func,
                     covariance = list(Sigma = Sig),
                     theta_init = c(1,1))

func_updated <- function(X, theta) X[,1]^theta[1] + theta[2]*X[,2]^2
dfr_2 <- update(dfr_1, test_mean = func_updated)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory (show via RShowDoc("KEYWORDS")):
% \keyword{ ~kwd1 }
% \keyword{ ~kwd2 }
% Use only one keyword per line.
% For non-standard keywords, use \concept instead of \keyword:
% \concept{ ~cpt1 }
% \concept{ ~cpt2 }
% Use only one concept per line.

\name{plot.distfreereg}
\alias{plot.distfreereg}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
	Summary and Diagnostic Plots for \code{distfreereg} Objects
}
\description{
	This is a \code{\link{plot}} method for objects of class \code{distfreereg}.
	It automates the creation of summary and diagnostic plots for
	\code{distfreereg} objects. See the \href{../doc/v3_plotting.html}{Plotting
	with the \code{distfreereg} Package} vignette for examples.
}
\usage{
	\method{plot}{distfreereg}(x, which = "dens", stat = NULL, density_args = NULL,
	polygon_args = NULL, confband_args = NULL, abline_args = NULL,
	shade_col = rgb(1,0,0,0.5), text_args = NULL, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{

    \item{x}{
        Object of class \code{distfreereg}.
    }

    \item{which}{
        Character string. Acceptable values are "\code{dens}", "\code{ecdf}",
        "\code{residuals}", and "\code{epsp}":
        \itemize{
            \item "\code{dens}" produces a plot of the estimated density curve
                of the specified statistic.

            \item "\code{ecdf}" produces a plot of the empirical cumulative
                distribution function of the specified statistic.

            \item "\code{residuals}" produces a plot of the transformed
                residuals in the order specified by \code{x$res_order}.

            \item "\code{epsp}" produces a plot of the empirical partial sum
                process of the (ordered) transformed residuals.
        }
    }

    \item{stat}{
        Character vector of length one specifying the name of the statistic to
        plot when \code{which} is "\code{dens}" or "\code{ecdf}". By default,
        the first statistic in \code{x$observed_stats} is used.
    }

    \item{density_args}{
        An optional list of arguments to pass to \code{\link{density}} when
        \code{which} is "\code{dens}".
    }

    \item{polygon_args}{ An optional list of arguments to pass to
        \code{\link{polygon}}, used when \code{which} is "\code{dens}" to shade
        under the density curve to the right of the value of the observed
        statistic. Setting equal to \code{FALSE} prevents the call, and no
        shading is done.
    }

  \item{confband_args}{
	An optional list of values that control the calculation and plotting of
	confidence bands. Any of the following named elements are allowed.
    \itemize{
        \item \code{w}: Numeric; the sequence of points on which to evaluate the
            confidence band. By default, it is a sequence of \code{m}
            evenly-spaced numbers whose endpoints are determined by the
            quantiles of \code{x} calculated using the probabilities specified
            by \code{q_probs}, described below.
	  \item \code{m}: Integer; the length of \code{w}, used only when \code{w}
            is \code{NULL}. The default value is 100.
	  \item \code{batch_len}: Integer; the batch length for the algorithm. The
            default value is 50.
	  \item \code{N}: Integer; the number of multivariate \eqn{t} samples to use
            in the simulation.
	  \item \code{conf.level}: Numeric; the desired confidence level.
	  \item \code{q_probs}: Numeric vector of length 2; the probabilities sent
            to \code{\link{quantile}} to calculate the bounds used to define
            \code{w}, used only when \code{w} is \code{NULL}.
	  \item \code{curve_args}: An optional list of arguments passed to
	  \code{\link{lines}} (not \code{\link{curve}}!), used to create the
	  boundaries of the confidence band.
	  \item \code{polygon_args}: An optional list of arguments passed to
	  \code{\link{polygon}}, used to shade the confidence region. Setting equal
	  to \code{FALSE} prevents the call, and no shading is done.
	  \item \code{shade_col}: This provides a shortcut to the \code{col}
	  argument of \code{\link{polygon}} to change the color of the shaded
	  region.
	}
	Setting equal to \code{FALSE} prevents calculation and plotting of the
	confidence band.
}
\item{abline_args}{
	An optional list of arguments to pass to \code{\link{abline}}, used to draw
	a vertical line at the value of the observed statistic. Setting equal to
	\code{FALSE} prevents the call, and no line is drawn.
}
  \item{shade_col}{
	Character string or other value specifying the color to use to shade the
	upper tail of the distribution when \code{which} is "\code{dens}". Default
	value is red with 50\% transparency. This is a convenience argument, and the
	same functionality is available by defining a \code{col} element in the
	\code{polygon_args} argument.
}
  \item{text_args}{
	An optional list of arguments to pass to \code{\link{text}}, used to label
	the vertical line with the p-value of the observed statistic. Setting equal
	to \code{FALSE} prevents the call, and no text is printed.
}
  \item{\dots}{
	Additional arguments to pass to \code{\link{plot}}.
}
}
\details{
	This function produces one of three specified plots, depending on the value
	of \code{which}.
	
	When \code{which} is "\code{dens}" or "\code{ecdf}", a plot of the estimated
	density or empirical cumulative distribution function, respectively, of the
	simulated statistics is produced, including a vertical line at the value of
	the observed test statistic with the p-value displayed.
	
	The default placement of the p-value text is on the left side of the line
	indicating the statistic value. Specifically, the default values of \code{x}
	and \code{y} passed to \code{\link{text}} are the statistic value itself and
	the midpoint between zero and the maximum value of the density curve. The
	default value passed to \code{adj} is \code{c(1,0.5)}, meaning that the text
	is aligned to the left of the value \eqn{(x,y)} and centered vertically on
	it. (The default value for the text itself, which can be modified via the
	\code{label} argument of \code{\link{text}}, includes a space on the left
	and the right for padding so the text does not overlap the vertical line
	itself.) To align the text so it appears on the right side (for example, to
	avoid overlapping the density curve), use \code{text_args = list(adj =
	c(0,0.5))}. See documentation for \code{\link{text}} for details on this and
	other arguments.

    When \code{which} is "\code{residuals}", a time-series-like plot is produced
    showing transformed residuals in the order given by \code{x$res_order}. In
    the case that the null hypothesis is rejected, this plot can help determine
    where (in terms of the linearly ordered covariates) a discrepancy between
    the model and the data occurs.

    When \code{which} is "\code{epsp}", a plot of the empirical partial sum
    process is produced; that is, the \eqn{y}-values are \deqn{y_j =
    {1\over\sqrt{n}}\sum_{i=1}^j\hat e_i} where \eqn{\hat e_i} is the \eqn{i}th
    transformed residual in the order given by \code{x$res_order}.  Similar to
    the case when \code{which} is "\code{residuals}", this plot can help
    determine where (in terms of the linearly ordered covariates) a discrepancy
    between the model and the data occurs.
}

\value{
	When \code{which} is "\code{dens}" or "\code{ecdf}", the values used to
	create the plot are returned invisibly in a list with two named elements,
	\code{x} and \code{y}. If the confidence band is plotted, then it is
	included as an element named \code{confband}.
	
	For other values of \code{which}, nothing is returned.
}

\author{
	Jesse Miller
}

\references{
Flegal, James M. et al. \strong{Simultaneous confidence bands for (Markov chain)
Monte Carlo simulations}, forthcoming.
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
	\code{\link{distfreereg}}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory (show via RShowDoc("KEYWORDS")):
% \keyword{ ~kwd1 }
% \keyword{ ~kwd2 }
% Use only one keyword per line.
% For non-standard keywords, use \concept instead of \keyword:
% \concept{ ~cpt1 }
% \concept{ ~cpt2 }
% Use only one concept per line.

\name{distfreereg-package}
\alias{distfreereg-package}
\docType{package}
\title{
\packageTitle{distfreereg}
}
\description{
\packageDescription{distfreereg}
}
\details{

The DESCRIPTION file:
\packageDESCRIPTION{distfreereg}
\packageIndices{distfreereg}
}
\author{
\packageAuthor{distfreereg}

Maintainer: \packageMaintainer{distfreereg}
}

\name{compare}
\alias{compare}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
    Compare the Distributions of Empirical and Theoretical Statistics Used in
    Distribution-Free Parametric Regression Testing
}
\description{
	Simulate response data repeatedly with \code{true_mean} as the mean and
	\code{true_covariance} as the covariance structure, each time running
	\code{\link{distfreereg}} on the simulated data. The observed statistics and
	p-values are saved, as are the simulated statistics from the first
	replication.
	
	This function is intended to facilitate exploring sample size adequacy and
	the tests' powers. For a convenient wrapper to expedite investigating sample
	size adequacy using a \code{distfreereg} object, see
	\code{\link{asymptotics}}.
	
	See the \href{../doc/v2_compare.html}{Comparing Distributions with the
	\code{distfreereg} Package} vignette for an introduction.
}
\usage{
	compare(true_mean, true_method = NULL, true_method_args = NULL, true_covariance,
	true_X = NULL, true_data = NULL, theta = NULL, n = NULL, reps = 1e3,
	prog = reps/10, simulate_args = NULL, err_dist_fun = NULL,
	err_dist_args = NULL, keep = NULL, manual = NULL, update_args = NULL, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{true_mean}{

	Object specifying the mean structure of the true model. It is used to generate
	the true values of \code{Y} that are passed internally to
	\code{\link{distfreereg}}.

}
  \item{true_method}{

	Character vector of length one; specifies the function (e.g.,
	\code{\link{lm}}) to use to create a model when \code{true_mean} is a
	\code{formula}; should be \code{NULL} otherwise.

}
  \item{true_method_args}{

	Optional list; values are passed to the function specified by
	\code{true_method}. Should be \code{NULL} when \code{true_mean} is not a
    \code{formula}.

}
  \item{true_covariance}{

    Named list; specifies the covariance structures of the true error
    distribution in the format described in the documentation for the
    \code{covariance} argument of \code{\link{distfreereg}}. Required when
    \code{true_mean} is a \code{function} or \code{nls} object, or
    \code{true_method} is "\code{nls}"; should be \code{NULL} otherwise.

}
  \item{true_X, true_data}{

    Optional numeric matrix or data frame, respectively; specifies the covariate
    values for the true model. \code{true_X} is used when \code{true_mean} is a
    function that has an \code{X} or \code{x} argument, and the \code{data}
    argument is used when \code{true_mean} is a formula or model object. At most
    one of these should be non-\code{NULL}. Both can be \code{NULL} when
    \code{true_mean} is a constant function (that is, it has no covariate
    argument).

}
  \item{theta}{

	Numeric vector; used as the (true) parameter values for the model when
	\code{true_mean} is a \code{function}; should be \code{NULL} otherwise.

}
  \item{n}{

    Optional integer; indicates how long each simulated data vector should be.
    Required only when no covariate values are specified for either the true or
    test mean; should be \code{NULL} otherwise. Silently converted to integer if
    numeric.

}
  \item{reps}{

	Integer; specifies number of replications. Silently converted to integer if
	numeric.

}
  \item{prog}{

	Integer or \code{Inf}; if finite, a progress message is given when the current
	repetition is a multiple of \code{prog}. Default value is \code{reps/10},
	unless \code{reps} is less than 10, in which case the default is \code{1}. If
	\code{Inf}, no progress messages are given. Silently converted to integer if
	finite numeric.

}
  \item{simulate_args}{

	Optional list; specifies additional named arguments to pass to
	\code{\link{simulate}}.

}
  \item{err_dist_fun}{

	Character string; specifies the name of the function to be used to simulate
	errors when \code{true_mean} is a \code{function} or \code{nls} object, or
	\code{true_method} is "\code{nls}". See details.

}
  \item{err_dist_args}{

	Optional list; specifies additional named arguments to pass to
	\code{err_dist_fun}.

}
  \item{keep}{

	A vector of integers, or the character string "\code{all}". If not
	\code{NULL}, then the output of each replication's call to
	\code{\link{distfreereg}} is included in the output if its repetition number
	is included in \code{keep}. Using \code{keep = "all"} is equivalent to
	\code{keep = 1:reps}.

}
  \item{manual}{

	Optional function; applied to the \code{distfreereg} object created in each
	iteration, whose output is saved in the list \code{manual} in the output.

}
  \item{update_args}{

	Optional named list; specifies arguments to pass to
	\code{\link{update.distfreereg}}.

}
  \item{...}{

	Additional arguments passed to \code{\link{distfreereg}}. See details.
    
}
}
\details{

	This function allows the user to explore the asymptotic behavior of the
	distributions involved in the test conducted by \code{\link{distfreereg}}. If
	the sample size is large enough and the true covariance matrix of the errors
	is known or is estimated well enough, then the observed and simulated
	statistics have nearly the same distribution. How large the sample size must
	be depends on the details of the situation. This function can be used to
	determine how large the sample size must be to obtain approximately equal
	distributions, and to estimate the power of the test against a specific
	alternative.
	
	The user specifies a particular true model which is used to generate outcome
	values. There are three cases:
	\itemize{
	  \item When \code{true_mean} is a function, this function determines the mean
	    of the outcome values and \code{err_dist_fun} is used to generate errors.
	    The error-generating function will usually include an element of
	    \code{true_covariance} as an argument, and in that case must accept the
	    appropriate class of object. For example, if the true covariance is a list
	    of matrices corresponding to a block-diagonal covariance matrix, then
	    \code{err_dist_fun} must accept such a list as an argument.
	  \item When \code{true_mean} is an \code{nls} object, or when it is a
	    \code{formula} and \code{true_method} is "\code{nls}", the function
	    determined by the formula (in the model call or user-specified,
	    respectively) is used to determine the mean function, and
	    \code{err_dist_fun} generates the errors.
	  \item When \code{true_mean} is a model object that is not an \code{nls}
	    object, or a \code{formula} and \code{method} is not "\code{nls}", then
	    \code{\link{simulate}} is used to generate outcome values.
    }
    
    If none of these cases apply to \code{true_mean}, then \code{compare()}
    cannot be used. (E.g., \code{true_mean} cannot be a \code{glm} object fitted
    using a "quasi" \code{\link{family}}, because \code{\link{simulate}} does
    not work for that family.)

    The user also specifies arguments to pass to \code{\link{distfreereg}}, most
    notably a model to test comprising a mean function \code{test_mean} and a
    covariance structure specified by \code{covariance}. For each repetition,
    \code{compare} sends the simulated data, as \code{Y} or as part of
    \code{data}, to \code{\link{distfreereg}}.
	
    The \code{true_covariance} argument specifies the covariance structure that
    is available to \code{err_dist_fun} for generating errors. The needs of
    \code{err_dist_fun} can vary (for example, the default function uses
    \code{SqrtSigma} to generate multivariate normal errors), so any one of the
    elements \code{Sigma}, \code{SqrtSigma}, \code{P}, and \code{Q} (defined in
    the documentation of \code{\link{distfreereg}}) can be specified. Any
    element needed by \code{err_dist_fun} is calculated automatically if not
    supplied.
	
    The value of \code{err_dist_fun} must be a function whose output is a
    numeric matrix with \code{n} rows and \code{reps} columns. Each column is
    used as the vector of errors in one repetition. The error function's
    arguments can include the special values \code{n}, \code{reps},
    \code{Sigma}, \code{SqrtSigma}, \code{P}, and \code{Q}. These arguments are
    automatically assigned their corresponding values from the values passed to
    \code{compare}. For example, the default value \code{rmvnorm} uses
    \code{SqrtSigma} to generate multivariate normal values with mean 0 and
    covariance \code{Sigma}.
	
    The argument \code{keep} is useful for diagnosing problems, but caution
    should be used lest a very large object be created. It is often sufficient
    to save the \code{distfreereg} objects from only the first few replications.
	
    For more specialized needs, the \code{manual} argument allows the
    calculation and saving of objects during each repetition. For example, using
    \code{manual = function(x) residuals(x)} will save the (raw) residuals from
    each repetition.
	
    The first repetition creates a \code{distfreereg} object. During each
    subsequent repetition, this object is passed to
    \code{\link{update.distfreereg}} to create a new object. The
    \code{update_args} argument can be used to modify this call.
	
    If necessary, \code{global_override} can be used to pass an \code{override}
    argument to \code{\link{distfreereg}} in each repetition. For example, using
    \code{gobal_override = list(theta_hat = theta)} forces the estimated
    parameter vector used in the test in each call to be the true parameter
    vector \code{theta}.  } \value{ An object of class \code{compare} with the
    following components: \item{call}{The matched call.} \item{Y}{The matrix
        whose columns contain the model outcome values used for the
        corresponding repetitions.} \item{theta}{Supplied vector of parameter
        values.} \item{true_mean}{Supplied object specifying the true mean
        function.} \item{true_covariance}{List containing element(s) that
        specify the true covariance structure.} \item{true_X}{Supplied matrix of
        true covariate values.} \item{true_data}{Supplied data frame of true
        covariate values.} \item{test_mean}{Supplied object specifying the mean
        function being tested.} \item{covariance}{List containing element(s)
        that specify the test covariance structure.} \item{X}{Supplied matrix of
        test covariate values.} \item{data}{Supplied data frame of test
        covariate values.} \item{empirical_stats}{The statistics observed in
        each repetition.} \item{theoretical_stats}{The simulated statistics from
        the first repetition. (They are the same for each repetition, because
        \code{compare} uses \code{\link{update.distfreereg}}.)} \item{p}{The
        p-values for the observed statistics.} \item{dfrs}{A list containing the
        outputs of \code{\link{distfreereg}} for repetitions specified in
        \code{keep}. Included when \code{keep} is not \code{NULL}.}
    \item{manual}{A list containing the results of the function specified by the
        argument \code{manual}. Included when \code{manual} is not \code{NULL}.}
    } \author{ Jesse Miller }


\note{

	Some of the processing of the elements of \code{true_covariance} is analogous
	to the processing of \code{covariance} by \code{\link{distfreereg}}. Any
	values of \code{solve_tol} and \code{symmetric} specified in
	\code{\link{distfreereg}}'s \code{control} argument are used by \code{compare}
	to similar effect in processing \code{true_covariance}.
	
	Support for \code{glm} objects is limited to those created using a
	\code{\link{family}} that has a \code{simulate} element.
	
	The presence of \code{call} in the value allows a \code{compare} object to be
	passed to \code{\link{update}}.
    
}

\section{Warnings}{

    The generation of new outcome values requires specifying an error
    distribution.  The default behavior when \code{true_mean} is a
    \code{function}, an \code{nls} object, or a \code{formula} with
    \code{method} equal to "\code{nls}" is to use a multivariate normal error
    distribution, but different error-generating functions can be defined by the
    user. When \code{true_mean} is a model object that is not an \code{nls}
    object, or a \code{formula} and \code{method} is not "\code{nls}", then the
    errors are generated using \code{\link{simulate}} and are therefore
    distributed according to that function's specifications. In short, the
    asymptotic behavior is determined for a specific (true) error distribution,
    even though the test itself is distribution-free.

    Simulated outcome values for model using a \code{subset} argument can result
    in unexpected behavior when using a subsetting condition that involves the
    outcome variable. Such subsetting is highly discouraged, since the
    subsetting will occur after outcome values have been replaced and therefore
    different subsets might be selected for each repetition. Further, because of
    the way that a data set is retrieved from an \code{nls} object, subsetting
    with \code{\link{nls}} using "self-referential" conditions can cause
    unexpected results. For example, if a condition such as \code{x > median(x)}
    is used, then the median will be calculated on each subset in turn, which
    will shrink the sample size with each repetition and throw an error.

}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
	\code{\link{asymptotics}}, \code{\link{distfreereg}},
	\code{\link{ks.test.compare}}, \code{\link{plot.compare}},
	\code{\link{rejection}}
}
\examples{
set.seed(20240201)
n <- 100
func <- function(X, theta) theta[1] + theta[2]*X[,1]
Sig <- rWishart(1, df = n, Sigma = diag(n))[,,1]
theta <- c(2,5)
X <- matrix(rexp(n, rate = 1))
# In practice, 'reps' should be much larger
cdfr <- compare(true_mean = func, true_X = X, true_covariance = list(Sigma = Sig),
                test_mean = func, X = X, covariance = list(Sigma = Sig),
                reps = 10, prog = Inf, theta = theta, theta_init = rep(1, length(theta)))

cdfr$p
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory (show via RShowDoc("KEYWORDS")):
% \keyword{ ~kwd1 }
% \keyword{ ~kwd2 }
% Use only one keyword per line.
% For non-standard keywords, use \concept instead of \keyword:
% \concept{ ~cpt1 }
% \concept{ ~cpt2 }
% Use only one concept per line.

\name{vcov.distfreereg}
\alias{vcov.distfreereg}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
	Estimate Parameter Covariance Matrices from \code{distfreereg} Objects
}
\description{

    This is a \code{\link{vcov}} method for objects of class \code{distfreereg}.
    It estimates the covariance matrix of the estimated parameters in a model
    from a \code{distfreereg} object.

}
\usage{
	\method{vcov}{distfreereg}(object, ..., jacobian_args, hessian_args)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{object}{
	Object of class \code{distfreereg}.
}
  \item{\dots}{
	Additional parameters passed to other methods when \code{test_mean} element of
	\code{object} is not of class \code{function}.
}
  \item{jacobian_args, hessian_args}{
	Lists of additional arguments to pass to \code{\link[calculus]{jacobian}} and
	\code{\link[calculus]{hessian}}.
}
}
\details{

    When the \code{test_mean} element of \code{object} is of class
    \code{function}, the covariance matrix is estimated using the method
    described in section 5.3 of \cite{Van der Vaart (2007)}. Otherwise,
    \code{test_mean} is of a class that has its own method for
    \code{\link{vcov}}, which is used to calculate the output.

}
\note{
	If \code{object} was created by calling \code{\link{distfreereg}} with no
	test mean function (that is, with \code{test_mean} equal to \code{NULL}),
	there is no estimated parameter vector, and therefore this function does not
	apply.
}
\section{Warning}{
	This calculation can be computationally intensive when the sample size is
	large and \code{object$test_mean} is a function.
}
\value{
	Named numeric matrix equal to the estimated covariance matrix of the parameter
	estimates from \code{object}.
}
\references{
	Vaart, A. W. \strong{Asymptotic statistics}, 2007, \emph{Cambridge series on
	statistical and probabilistic mathematics}, Cambridge University Press.
}
\author{
	Jesse Miller
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
	\code{\link{distfreereg}}, \code{\link{confint.distfreereg}}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory (show via RShowDoc("KEYWORDS")):
% \keyword{ ~kwd1 }
% \keyword{ ~kwd2 }
% Use only one keyword per line.
% For non-standard keywords, use \concept instead of \keyword:
% \concept{ ~cpt1 }
% \concept{ ~cpt2 }
% Use only one concept per line.

\name{rejection}
\alias{rejection}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
    Compute Rejection Rates of a Distribution-Free Test from a \code{compare}
    Object
}
\description{
    Compute the rejection rates of the tests simulated in a \code{compare}
    object. Specifically, this function estimates the rejection rates of the
    tests conducted with specified statistics of the hypothesis that the mean
    function is \code{test_mean} when the true mean function is
    \code{true_mean}.
}
\usage{
	rejection(object, alpha = 0.05, stat = names(object[["empirical_stats"]]), ...)

}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{object}{
	Object of class \code{compare}.
}
  \item{alpha}{
    Numeric vector; specifies the \eqn{\alpha}-levels to use. Passed as
    \code{probs} argument to \code{\link{quantile}}.
  }
  \item{stat}{
    Character vector; specifies the names of the statistics to use. The default
    value computes the rejection rate associated with every statistic in
    \code{object}.
  }
  \item{...}{
    Additional arguments to pass to \code{\link{quantile}} to estimate the
    \eqn{1-\alpha} quantiles of the distribution of simulated statistics.
  }
}
\value{
    Data frame containing estimated rejection rates and associated Monte Carlo
    standard errors, with one row for each combination of \code{stat} and
    \code{alpha} elements.
}
\author{
	Jesse Miller
}

%% ~Make other sections like Warning with \section{Warning }{....} ~
\section{Warning}{

  The reported Monte Carlo standard error does not account for the uncertainty
  of the estimation of the \eqn{1-\alpha} quantiles of the distribution of
  simulated statistics. The number of Monte Carlo simulations should be large
  enough to make this estimate sufficiently accurate that it can be considered
  known for practical purposes. The standard errors of estimated quantiles can
  be calculated using the \pkg{mcmcse} package.

}

\seealso{
	\code{\link{distfreereg}}, \code{\link{compare}}
}
\examples{
# In practice, set "reps" much larger than 20.
set.seed(20240201)
n <- 100
func <- function(X, theta) theta[1] + theta[2]*X[,1]
Sig <- rWishart(1, df = n, Sigma = diag(n))[,,1]
theta <- c(2,5)
X <- matrix(rexp(n, rate = 1))
cdfr <- compare(true_mean = func, true_X = X, true_covariance = list(Sigma = Sig),
                test_mean = func, X = X, covariance = list(Sigma = Sig), reps = 20,
                prog = Inf, theta = theta, theta_init = rep(1, length(theta)))

rejection(cdfr)
rejection(cdfr, stat = "CvM")
rejection(cdfr, alpha = c(0.1, 0.2))
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory (show via RShowDoc("KEYWORDS")):
% \keyword{ ~kwd1 }
% \keyword{ ~kwd2 }
% Use only one keyword per line.
% For non-standard keywords, use \concept instead of \keyword:
% \concept{ ~cpt1 }
% \concept{ ~cpt2 }
% Use only one concept per line.

\name{distfreereg}
\alias{distfreereg}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
    Distribution-Free Parametric Regression Testing
}
\description{
    Conduct distribution-free parametric regression testing using the process
    introduced in \cite{Khmaladze (2021)}. A parametric model for the
    conditional mean (specified by \code{test_mean}) is checked against the data
    by fitting the model, transforming the resulting residuals, and then
    calculating a statistic on the empirical partial sum process of the
    transformed residuals. The statistic's null distribution can be simulated in
    a straight-forward way, thereby producing a p-value.

    Using \eqn{f} to denote the mean function being tested, the specific test
    has the following null and alternative hypotheses:
    \deqn{H_0\colon\ \exists\theta\in\Theta\subseteq\mathbb R^p
        \mathrel{\bigl|}\textrm{E}(Y| X)=f(X;\theta)
        \quad\hbox{against}\quad
        H_1\colon\ \forall\theta\in\Theta\subseteq\mathbb R^p
        \mathrel{\bigl|} \textrm{E}(Y| X)\neq f(X;\theta).}

    This assumes a known or consistently estimated covariance matrix.  See the
    \href{../doc/v1_introduction.html}{An Introduction to the \code{distfreereg}
        Package} vignette for an introduction.

}

\usage{

distfreereg(test_mean = NULL, covariance = NULL, Y = NULL, X = NULL,
            theta_init = NULL, data = NULL, method = NULL, method_args = NULL,
            stat = c("KS", "CvM"), B = 1e4, ordering = "simplex", group = TRUE,
            control = NULL, override = NULL, verbose = TRUE)

}

%- maybe also 'usage' for other objects documented here.
\arguments{

    \item{test_mean}{

        A specification of the mean function to be tested. Accepted classes are
        \code{function}, \code{formula}, \code{glm}, \code{lm}, \code{lmerMod} (from
        the \pkg{lme4} package), and \code{nls}. Can be \code{NULL} if
        \code{override} contains \code{fitted_values} and \code{J} elements. See
        Details and Warnings.

    }

    \item{covariance}{

        Named list; specifies the covariance structure of the model's error
        distribution. Must be `NULL` when `test_mean` is a model object, in
        which case the covariance structure is extracted automatically from the
        model object. Valid element names are "\code{Sigma}", "\code{SqrtSigma}",
        "\code{P}", and "\code{Q}", corresponding to the covariance matrix, the
        square root of the covariance matrix, the precision matrix, and the square
        root of the precision matrix, respectively. Each element must be one of the
        following:
        \itemize{
            \item a numeric matrix.
            \item a list of numeric matrices.
            \item a numeric vector whose length is the sample size.
            \item a numeric vector of length 1.
        }
        See Details.
    }

    \item{Y}{
    
        Numeric vector of observations used when \code{test_mean} is a
        \code{function} or \code{NULL}; should be \code{NULL} otherwise. A
        non-vector value is converted to a vector.
    
    }
    
    \item{X}{
    
        Optional numeric matrix of covariates used when \code{test_mean} is a
        \code{function} or \code{NULL}; should be \code{NULL} otherwise. A
        non-matrix value is converted to a single-column matrix.
    
    }
    
    \item{theta_init}{
    
        Numeric vector; specifies the starting parameter values passed to the
        optimization function used to estimate the parameter vector. Used when
        \code{test_mean} is a \code{function}; should be \code{NULL} otherwise.
   
    }

    \item{data}{
    
        Data frame of covariate values used when \code{test_mean} is a
        \code{formula}; should be \code{NULL} otherwise.
    
    }
    
    \item{method}{
    
        Character vector; specifies the function to use for fitting the model
        when \code{test_mean} is a formula; should be \code{NULL} otherwise.
        Possible values are "\code{\link{glm}}", "\code{\link{lm}}",
        "\code{\link[lme4]{lmer}}", and "\code{\link{nls}}".
    
    }
    
    \item{method_args}{
        
        Optional list of argument values to be passed to the function specified
        by the \code{method} argument. Should be \code{NULL} when
        \code{test_mean} is not a formula.
    
    }

    \item{stat}{

        Character vector; specifies the names of the functions used to calculate
        the desired statistics. By default, a Kolmogorov--Smirnov statistic and
        a Cramer--von Mises-like statistic are calculated: \deqn{\hbox{KS} =
            \max_{i}|a_i|\qquad\hbox{and}\qquad\hbox{CvM} = {1\over
                n}\sum_{i=1}^na_i^2} where \eqn{a_i} is term \eqn{i} in the
        empirical partial sum process and \eqn{n} is the sample size.

    }

    \item{B}{

        Numeric vector of length one; specifies the Monte Carlo sample size used
        when simulating statistics. Silently converted to integer.
    
    }

    \item{ordering}{

        A character string or a list; specifies how to order the residuals to form
        the empirical partial sum process. Valid character strings are:
        \itemize{

            \item "\code{asis}": leaves the order unchanged (that is, in the order
            in which the observations appear in the supplied data).
            
            \item "\code{natural}": orders residuals using column-wise ordering of the
            covariates.
            
            \item "\code{optimal}": orders the residuals by ordering the
            observations using optimal transport on the covariates. The solution
            is estimated using the Hungarian method as implemented by
            \code{\link[clue]{solve_LSAP}}.  This option can be very slow for
            large sets of covariates.
            
            \item "\code{simplex}": (the default) orders residuals in order of
            increasing row sums of the covariates after each column has been
            scaled to the interval \eqn{[0,1]}.
        
        }
        If \code{ordering} is a list, then its elements specify names columns of
        \code{X} or \code{data} to use to determine the order.

    }

    \item{group}{

        Logical; if \code{TRUE}, then residuals with the same rank (as determined
        by \code{ordering}) are grouped when calculating the empirical partial sum
        process.

    }

    \item{control}{
    
        Optional named list of elements that control the details of the
        algorithm's computations. The following elements are accepted for all
        methods:
        \itemize{
           
            \item \code{symmetric}:	Optional named list or \code{FALSE}; if a
            named list, its elements are passed as arguments to
            \code{\link{isSymmetric}} when testing elements of \code{covariance}
            for symmetry. If \code{FALSE}, then this test is skipped.
           
            \item \code{matsqrt_tol}: Numeric; specifies the threshold for
            considering an eigenvalue "too negative" when calculating the square
            root of a matrix. Must be non-positive. The default value is
            \code{-.Machine$double.eps^0.25}.
           
            \item \code{solve_tol}: Numeric; passed as \code{tol} argument to
            \code{\link{solve}}, used in particular to invert \code{Sigma}. The
            default value is \code{.Machine$double.eps}.
           
            \item \code{qr_tol}: Numeric; passed as \code{tol} argument to
            \code{\link{qr}}. The default value is
            \code{sqrt(.Machine$double.eps)}. This value might need to be
            decreased when the dimensions of the Jacobian are sufficiently
            large.
           
            \item \code{orth_tol}: Numeric; passed as \code{tolerance} argument
            to \code{\link{all.equal}} when testing whether or not \eqn{r^Tr}
            and \eqn{\mu^T\mu} are the identity matrix. The default value is
            \code{sqrt(.Machine$double.eps)}.
           
            \item \code{trans_tol}: Numeric; passed as \code{tolerance} argument
            to \code{\link{all.equal}} to internal transformation function when
            determining whether the normalizing scalar is non-zero. The default
            value is \code{sqrt(.Machine$double.eps)}.
           
            \item \code{sym_tol, sym_tol1}: Numeric; passed as \code{tol} and
            \code{tol1} arguments, respectively, to \code{\link{isSymmetric}}
            for checking \code{Sigma} and \code{P} elements of
            \code{covariance}. The default values are
            \code{sqrt(.Machine$double.eps)} and
            \code{8*sqrt(.Machine$double.eps)}, respectively. See Warnings.
           
            \item \code{return_on_error}: Logical; determines whether or not
            partial output is returned when an error is encountered. Default
            value is \code{TRUE}.
        }
        The following named elements, all but the first of which control the
        process of calculating the generalized least squares estimation of the
        parameter vector, are accepted for the \code{function} method:
        \itemize{
           
            \item \code{jacobian_args}: Optional list; specifies arguments to
            pass to \code{\link[calculus]{jacobian}}.
           
            \item \code{optimization_fun}: Optional function; specifies the
            function used to estimate the parameters. If not specified,
            \code{\link{optim}} is used with \code{method = "BFGS"}.
           
            \item \code{fun_to_optimize_arg}: Optional character string,
            required when \code{optimization_fun} is specified; specifies the
            name of the argument of \code{optimization_fun} that is assigned the
            function to optimize. For example, \code{\link{optim}} uses
            "\code{fn}".
           
            \item \code{theta_init_arg}: Optional character string, required
            when \code{optimization_fun} is specified; specifies the name of the
            argument of \code{optimization_fun} that is assigned the initial
            parameter values for optimization. For example, \code{\link{optim}}
            uses "\code{par}".
           
            \item \code{theta_hat_name}: Optional character string, required
            when \code{optimization_fun} is specified; specifies the name of the
            element of the output of \code{optimization_fun} that contains the
            estimated parameters. For example, \code{\link{optim}} uses
            "\code{par}" (the same character string that specifies the argument
            containing the initial values). See Warnings.
           
            \item \code{optimization_args}: Optional list; specifies additional
            arguments to pass to \code{optimization_fun}.

        }
    }

    \item{override}{

        Optional named list of arguments that override internally calculated
        values. Used primarily by \code{\link{update.distfreereg}}, but can be
        accessed directly. The following named elements are accepted:
        \itemize{

            \item \code{J}: Numeric matrix. Overrides the calculation of the
            Jacobian.
            
            \item \code{fitted_values}: Numeric vector. Overrides the
            calculation of the fitted values.

            \item \code{res_order}: Integer vector specifying the ordering used
            to order the residuals for the computation of the empirical partial
            sum process. Overrides \code{ordering}. See Details.
            
            \item \code{theta_hat}: Numeric vector. Used as the estimated
            parameter vector when \code{test_mean} has class \code{function},
            overriding internal computation that would have used
            \code{optimization_fun}.
            
            \item \code{r}: Orthogonal (numeric) matrix. Overrides the
            construction of the transformation anchor matrix.
            
            \item \code{theoretical_stats}: List. Overrides the creation of the list
            of simulated statistics.
        
        }
    }

    \item{verbose}{
    
        Logical; if \code{TRUE}, progress messages are printed. Default value is
        \code{TRUE}.
    
    }
    
}

\details{

    This function implements distribution-free parametric regression testing.
    The model is specified by a mean structure and a covariance structure.

    The mean structure is specified by the argument \code{test_mean}. This can
    be an object of class \code{function}, \code{formula}, \code{glm},
    \code{lm}, \code{lmerMod}, or \code{nls}. It can also be \code{NULL}.

    If \code{test_mean} is a function, then it must have one or two arguments:
    either \code{theta} only, or \code{theta} and either \code{X} (uppercase) or
    \code{x} (lowercase). An uppercase \code{X} is interpreted in the function
    definition as a matrix, while a lowercase \code{x} is interpreted as a
    vector. (See examples and \href{../doc/v5_advanced-options.html}{this
        vignette}.) The primary reason to use a lowercase \code{x} is to allow
    for a function definition using an \code{R} function that is not vectorized.
    In general, an uppercase \code{X} should be preferred for speed.

    If \code{test_mean} is a formula, then it must be a formula that can be
    passed to \code{\link{glm}}, \code{\link{lm}}, \code{\link[lme4]{lmer}}, or
    \code{\link{nls}}, and the \code{data} argument must be specified. The
    appropriate model is created and is then sent to \code{distfreereg()} for
    method dispatch.

    The \code{function} method estimates parameter values, and then uses those
    to evaluate the Jacobian of the mean function and to calculate fitted
    values.

    All of these methods create a Jacobian matrix and a vector of fitted values.
    These, along with the covariance structure, are sent the default method. The
    default method also allows the user to implement the algorithm even when the
    mean structure is not specified in \code{R}, as long as the Jacobian, fitted
    values, and covariance structure can be imported. (This is useful if a
    particularly complicated function is defined in another language and cannot
    easily be copied into \code{R}.)

    The covariance structure for \eqn{Y|X} must be specified using the
    \code{covariance} argument for the \code{function} and default methods. For
    other methods, the covariance is estimated automatically.

    Any element of \code{covariance} can be a numeric matrix, a list of numeric
    matrices, or a numeric vector. If it is a vector, its length must be either
    1 or the sample size. This option is mathematically equivalent to setting a
    covariance list element to a diagonal matrix with the specified value(s)
    along the diagonal. Using vectors, when possible, is more efficient than
    using the corresponding matrix. If an element of \code{covariance} is a list
    of numeric matrices, then these matrices are interpreted as the blocks of a
    block diagonal matrix.

    Internally, \code{distfreereg()} only needs \code{Q}, so some efficiency can
    be gained by supplying that directly when available. When \code{Q} is not
    specified, it is calculated using whichever element is specified. When more
    than one of the other elements are specified, no verification of
    compatibility of the elements is done.  \code{Q} is calculated using the
    supplied element(s) by preferring one-step operations and operations on the
    square-root level. (For example, if both \code{P} and \code{SqrtSigma}
    elements are supplied, then \code{Q} is calculated using \code{SqrtSigma}
    only.  If \code{Sigma} and \code{P} are supplied, then \code{Q} is
    calculated using \code{P} only.)

    The \code{override} argument is used primarily by
    \code{\link{update.distfreereg}} to avoid unnecessary and potentially
    computationally expensive recomputation. This \code{\link{update}} method
    imports appropriate values automatically from a previously created object of
    class \code{distfreereg}, and therefore validation is not always done. Use
    manually with caution.

    The \code{res_order} element of \code{override} must be a vector of integers
    from 1 to \eqn{n} (the sample size) that determines the order of the
    residuals to use when forming the empirical partial sum process. Elements of
    the vector can be repeated, in which case the residuals corresponding to
    matching \code{res_order} values are grouped when \code{group} is
    \code{TRUE}.

}

\section{Warnings}{

    Methods for model objects (e.g., \code{lm} objects) are intended to be used
    with objects created using a \code{data} argument that contains all
    variables used by the model. This data argument is assumed to be defined in
    the same environment as the model object, and this is assumed to be the
    environment in which \code{distfreereg()} is called.

    Consistency between \code{test_mean} and \code{theta_init} is verified only
    indirectly. Uninformative errors can occur when, for example,
    \code{theta_init} does not have the correct length. The most common error
    message that arises in this case is "\code{f_out cannot have NA values}",
    which occurs when \code{theta_init} is too short. To be safe, always define
    \code{test_mean} to use every element of \code{theta}.

    No verification of consistency is done when multiple elements of
    \code{coviariance} are specified. For example, if \code{P} and \code{Sigma}
    are both specified, then the code will use only one of these, and will not
    verify that \code{P} is the inverse of \code{Sigma}.

    When using the \code{control} argument element \code{optimization_fun} to
    specify an optimization function other than \code{\link{optim}}, the
    verification that \code{theta_hat_name} actually matches the name of an
    element of the optimization function's output is done only after the
    optimization has been done. If this optimization will likely take a long
    time, it is important to verify the value of \code{theta_hat_name} before
    running \code{distfreereg()}.

    The default values of \code{sym_tol} and \code{sym_tol1} are intended to
    check for substantial asymmetry such as would be caused by the user
    inputting the wrong matrix. Therefore, they do not check for symmetry with
    high precision. In particular, these default values are orders of magnitude
    larger than the default values in \code{\link{isSymmetric}}.

    The theory described by \cite{Khmaladze (2021)} does not apply directly to
    generalized linear models nor linear mixed-effects models, but extensive
    simulations indicate that the method applies nonetheless to these models.
    Further investigation can be done using \code{\link{asymptotics}}. Non-null
    values for the \code{weights} argument in \code{\link{glm}} are not
    supported.

}

\value{

    An object of class \code{distfreereg} with the following components:

    \item{call}{The matched call.}

    \item{data}{A list containing the \code{data} argument value, if present,
        and \code{Y} and \code{X}.}
    
    \item{test_mean}{The value supplied to the argument \code{test_mean}.}
    
    \item{model}{The model built when using the \code{formula} method; present
        if and only if using that method.}
    
    \item{covariance}{The list of covariance matrices, containing at least
        \code{Q}.}
    
    \item{theta_hat}{The estimated parameter vector. When the model being tested
        has class \code{lmerMod}, this is the vector of fixed effect parameters
        as returned by \code{\link[lme4]{fixef}}.}
    
    \item{optimization_output}{The output of \code{optimization_fun} or
        \code{\link{nls}} from calculating \code{theta_hat}.}
    
    \item{fitted_values}{The vector of fitted values,
        \eqn{f\bigl(X;\hat\theta\bigr)}. When testing a generalized linear
        model, these are the fitted values on the response scale.}
    
    \item{J}{The Jacobian matrix, \eqn{{\partial
                f\over\partial\theta}(X;\hat\theta)}.}
    
    \item{mu}{The mu matrix.}
    
    \item{r}{The matrix of transformation anchor vectors.}
    
    \item{r_tilde}{The matrix of modified transformation anchor vectors.}
    
    \item{residuals}{A named list of three vectors containing raw, sphered, and
        transformed residuals.}
    
    \item{res_order}{A numeric vector indicating the ranking of the residuals
        used to form the empirical partial sum process, in a format to be used
        as the input of \code{\link{order}}.}
    
    \item{grouping_matrix}{The matrix used to group residuals; present if
        \code{group} is \code{TRUE} and some covariate combinations are
        repeated.}
    
    \item{epsp}{The empirical partial sum process formed by calculating the
        scaled partial sums of the transformed residuals ordered according to
        \code{res_order}.}
    
    \item{observed_stats}{A named list of the observed statistic(s) corresponding
        to the transformed residuals.}
    
    \item{theoretical_stats}{A named list, each element of which contains the values
        of a simulated statistic.}
    
    \item{p}{A named list with two elements: \code{value}, which contains the
        p-values for each observed statistic, and \code{mcse}, which contains
        the Monte Carlo standard errors for the p-values.}

}

\references{

    Khmaladze, Estate V. \emph{Distribution-free testing in linear and
        parametric regression}, 2021-03, Annals of the Institute of Statistical
    Mathematics, Vol. 73, No. 6, p. 1063--1087. \doi{10.1007/s10463-021-00786-3}

}

\author{

    Jesse Miller

}

\seealso{

    \code{\link{coef.distfreereg}}, \code{\link{confint.distfreereg}},
    \code{\link{fitted.distfreereg}}, \code{\link{formula.distfreereg}},
    \code{\link{plot.distfreereg}}, \code{\link{predict.distfreereg}},
    \code{\link{print.distfreereg}}, \code{\link{residuals.distfreereg}},
    \code{\link{update.distfreereg}}, \code{\link{vcov.distfreereg}}

}

%% ~Make other sections like Warning with \section{Warning }{....} ~

    \examples{

        set.seed(20240218)
        n <- 1e2
        func <- function(X, theta) X[,1]^theta[1] + theta[2]*X[,2]
        Sig <- runif(n, min = 1, max = 3)
        theta <- c(2,5)
        X <- matrix(runif(2*n, min = 1, max = 5), nrow = n)
        Y <- X[,1]^theta[1] + theta[2]*X[,2] + rnorm(n, sd = sqrt(Sig))
        (dfr <- distfreereg(Y = Y, X = X, test_mean = func,
        covariance = list(Sigma = Sig),
        theta_init = c(1,1)))

        # Same test with lowercase "x" for reference;
        # use uppercase whenever possible.
        func_lower <- function(x, theta) x[1]^theta[1] + theta[2]*x[2]
        (dfr_lower <- distfreereg(Y = Y, X = X, test_mean = func_lower,
        covariance = list(Sigma = Sig),
        theta_init = c(1,1)))

    }
    % Add one or more standard keywords, see file 'KEYWORDS' in the
    % R documentation directory (show via RShowDoc("KEYWORDS")):
    % \keyword{ ~kwd1 }
    % \keyword{ ~kwd2 }
    % Use only one keyword per line.
    % For non-standard keywords, use \concept instead of \keyword:
    % \concept{ ~cpt1 }
    % \concept{ ~cpt2 }
    % Use only one concept per line.

\name{confint.distfreereg}
\alias{confint.distfreereg}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
	Calculate Confidence Intervals with a \code{distfreereg} Object
}
\description{
	This is a \code{\link{confint}} method for objects of class
	\code{distfreereg}. It calculates confidence intervals for the estimated
	parameters of a model in a \code{distfreereg} object.
}
\usage{
	\method{confint}{distfreereg}(object, parm, level = 0.95, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{object}{
	Object of class \code{distfreereg}.
}
  \item{parm}{
	Numeric or character vector; specifies which parameters are to be given
	confidence intervals. If missing, all parameters are considered.
}
  \item{level}{
	Numeric vector of length one; specifies the confidence level.
}
  \item{\dots}{
	Additional parameters passed to other methods. Currently ignored.
}
}
\details{
	When \code{object} contains a model object (either in the \code{test_mean} or
	\code{model} element), then this model is sent to \code{\link{confint}}.
	Otherwise, \code{object} is sent to \code{\link{confint.default}}.
}
\note{
	If \code{object} was created by calling \code{\link{distfreereg}} with no
	test mean function (that is, with \code{test_mean} equal to \code{NULL}),
	there is no estimated parameter vector, and therefore this function does not
	apply.
}
\value{
	The output from the appropriate \code{\link{confint}} method.
}
\author{
	Jesse Miller
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
	\code{\link{distfreereg}}, \code{\link{vcov.distfreereg}}, \code{\link{confint}}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory (show via RShowDoc("KEYWORDS")):
% \keyword{ ~kwd1 }
% \keyword{ ~kwd2 }
% Use only one keyword per line.
% For non-standard keywords, use \concept instead of \keyword:
% \concept{ ~cpt1 }
% \concept{ ~cpt2 }
% Use only one concept per line.

\name{fitted.distfreereg}
\alias{fitted.distfreereg}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
	Extract Fitted Values from \code{distfreereg} Objects
}
\description{
	This is a \code{\link{fitted}} method for objects of class \code{distfreereg}.
}
\usage{
	\method{fitted}{distfreereg}(object, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{object}{
	Object of class \code{distfreereg}.
}
  \item{\dots}{
	Additional parameters passed to or from other methods. Currently ignored.
}
}
\value{
	Numeric vector of fitted values.
}
\author{
	Jesse Miller
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
	\code{\link{distfreereg}}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory (show via RShowDoc("KEYWORDS")):
% \keyword{ ~kwd1 }
% \keyword{ ~kwd2 }
% Use only one keyword per line.
% For non-standard keywords, use \concept instead of \keyword:
% \concept{ ~cpt1 }
% \concept{ ~cpt2 }
% Use only one concept per line.
